---@type SeasonOfDiscovery
local SeasonOfDiscovery = QuestieLoader:ImportModule("SeasonOfDiscovery")
---@type QuestieDB
local QuestieDB = QuestieLoader:ImportModule("QuestieDB")
---@type ZoneDB
local ZoneDB = QuestieLoader:ImportModule("ZoneDB")

--- Load the base quests for Season of Discovery
--- These are generated, do NOT EDIT the data entries here.
--- If you want to edit an item, do so in sodItemFixes.lua
function SeasonOfDiscovery:LoadBaseItems()
    local itemKeys = QuestieDB.itemKeys
    local itemClasses = QuestieDB.itemClasses

    return {
        [202251] = {
            [itemKeys.name] = "Bag of Pet Treats",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [202254] = {
            [itemKeys.name] = "Bracers of Redirection",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [202256] = {
            [itemKeys.name] = "Privateer's Ornate Pistol",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [202316] = {
            [itemKeys.name] = "Runecarving Test - Crippling Poison",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [202641] = {
            [itemKeys.name] = "Mud-covered notebook",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203723] = {
            [itemKeys.name] = "Cutty's Note",
            [itemKeys.npcDrops] = {103,473},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203726] = {
            [itemKeys.name] = "Cutty's Rune",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203745] = {
            [itemKeys.name] = "Spell Notes: Ice Lance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203746] = {
            [itemKeys.name] = "Spell Notes: Living Flame",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203747] = {
            [itemKeys.name] = "Spell Notes: Fingers of Frost",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203748] = {
            [itemKeys.name] = "Spell Notes: Burnout",
            [itemKeys.npcDrops] = {202060,208275,208752},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203749] = {
            [itemKeys.name] = "Spell Notes: Enlightenment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203750] = {
            [itemKeys.name] = "Elwynn Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203751] = {
            [itemKeys.name] = "Spell Notes: CALE ENCI",
            [itemKeys.npcDrops] = {38,1507},
            [itemKeys.objectDrops] = {404695,405633},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203752] = {
            [itemKeys.name] = "Spell Notes: MILEGIN VALF",
            [itemKeys.npcDrops] = {476,1124,1397,1535,1536,1537,3195,3196,3197,3198,3199},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203753] = {
            [itemKeys.name] = "Spell Notes: RING SEFF OSTROF",
            [itemKeys.npcDrops] = {448,1119,1130,1132,1137,1260,1271,1388,1961,3203,3204,3205,5808,8503,208196,208638},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203754] = {
            [itemKeys.name] = "Archmage Antonidas: The Unabridged Autobiography",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {386691},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203755] = {
            [itemKeys.name] = "Archmage Theocritus's Research Journal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {386759},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203756] = {
            [itemKeys.name] = "Azora Apprentice Notes: Page 1",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203784] = {
            [itemKeys.name] = "Top-Right Map Piece",
            [itemKeys.npcDrops] = {6,40,80,257,327,475,476},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203785] = {
            [itemKeys.name] = "Top-Left Map Piece",
            [itemKeys.npcDrops] = {38,94,103,116,473,474,6866,6927},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203786] = {
            [itemKeys.name] = "Bottom-Right Map Piece",
            [itemKeys.npcDrops] = {46,285,732,735},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203787] = {
            [itemKeys.name] = "Bottom-Left Map Piece",
            [itemKeys.npcDrops] = {97,448,478},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203960] = {
            [itemKeys.name] = "Azora Apprentice Notes: Page 2",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203961] = {
            [itemKeys.name] = "Azora Apprentice Notes: Page 3",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203962] = {
            [itemKeys.name] = "Azora Apprentice Notes: Page 4",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203990] = {
            [itemKeys.name] = "Rune of Mutilation",
            [itemKeys.npcDrops] = {2038},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203991] = {
            [itemKeys.name] = "Rune of Quick Draw",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {386675},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203992] = {
            [itemKeys.name] = "Rune of Trickery",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203993] = {
            [itemKeys.name] = "Rune of Slaughter",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {387466,404941,407734},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [203994] = {
            [itemKeys.name] = "Rune of Deadly Brew",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204174] = {
            [itemKeys.name] = "Rune of Precision",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {386777,404830,405201,405946,407352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204270] = {
            [itemKeys.name] = "Fathom Core",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204441] = {
            [itemKeys.name] = "Rune of Blood Frenzy",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {392029},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204476] = {
            [itemKeys.name] = "Severed Kobold Head",
            [itemKeys.npcDrops] = {40,327,475},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204477] = {
            [itemKeys.name] = "Severed Murloc Head",
            [itemKeys.npcDrops] = {46,285,732,735,1543,1544,1545},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204478] = {
            [itemKeys.name] = "Severed Gnoll Head",
            [itemKeys.npcDrops] = {97,448,478,1674,1675,1941,2949,2950,2951},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204688] = {
            [itemKeys.name] = "Monster Hunter's First Rune Fragment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204689] = {
            [itemKeys.name] = "Monster Hunter's Second Rune Fragment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204690] = {
            [itemKeys.name] = "Monster Hunter's Third Rune Fragment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204703] = {
            [itemKeys.name] = "Rune of Devastate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204716] = {
            [itemKeys.name] = "Rune of Frenzied Assault",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204795] = {
            [itemKeys.name] = "Rune of Shadowstrike",
            [itemKeys.npcDrops] = {1506},
            [itemKeys.objectDrops] = {387477,404911,405628,406006},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204804] = {
            [itemKeys.name] = "Hydraxian Bangles",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204806] = {
            [itemKeys.name] = "Rune of Victory Rush",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {397987,403102,404911,406736},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204807] = {
            [itemKeys.name] = "Fathomblade",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204809] = {
            [itemKeys.name] = "Rune of Furious Thunder",
            [itemKeys.npcDrops] = {79,100,327,448,1130,1137,1271,1388,1961,3056,3058,3068,3131,3204,3205,3281,5785,5786,5807,5822,5823,5826,6113,207515,208196,208638},
            [itemKeys.objectDrops] = {407734},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204864] = {
            [itemKeys.name] = "Azora Apprentice Notes",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204905] = {
            [itemKeys.name] = "Ominous Tome",
            [itemKeys.npcDrops] = {474,476,881},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204906] = {
            [itemKeys.name] = "Gnoll Blood",
            [itemKeys.npcDrops] = {97,478,1674,1675,1941},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204907] = {
            [itemKeys.name] = "Wolf Jawbone",
            [itemKeys.npcDrops] = {69,118,299,525,704,705,1131,1133,1138,1922},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204910] = {
            [itemKeys.name] = "Monster - Trident, Naga (Gold)",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [204912] = {
            [itemKeys.name] = "Rune of Grace",
            [itemKeys.npcDrops] = {204070},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205019] = {
            [itemKeys.name] = "Tainted Soul Shard",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205020] = {
            [itemKeys.name] = "Pure Soul Shard",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205022] = {
            [itemKeys.name] = "Rune of Soul Siphon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205181] = {
            [itemKeys.name] = "Unidentified Artifact",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {205364,208223},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205182] = {
            [itemKeys.name] = "Powerless Artifact",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205183] = {
            [itemKeys.name] = "Fel-Powered Artifact",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205184] = {
            [itemKeys.name] = "Acolyte's Note",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {205364},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205215] = {
            [itemKeys.name] = "Rune of Tactics",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205218] = {
            [itemKeys.name] = "Libram of Discovery",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205228] = {
            [itemKeys.name] = "Rune of Chaos Bolt",
            [itemKeys.npcDrops] = {202060,208275,208752},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205230] = {
            [itemKeys.name] = "Rune of Haunting",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {387477,404695,405633,406736},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205364] = {
            [itemKeys.name] = "Acolyte's Knapsack",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205420] = {
            [itemKeys.name] = "Libram of Judgement",
            [itemKeys.npcDrops] = {38,706},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205683] = {
            [itemKeys.name] = "Rune of Rebuke",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205685] = {
            [itemKeys.name] = "Rune of Aegis",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205863] = {
            [itemKeys.name] = "Corrupted Libram",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205864] = {
            [itemKeys.name] = "Charred Note",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205897] = {
            [itemKeys.name] = "Rune of Martyrdom",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205905] = {
            [itemKeys.name] = "Memory of a Devout Champion",
            [itemKeys.npcDrops] = {210537,212809},
            [itemKeys.objectDrops] = {414663},
            [itemKeys.itemDrops] = {208766},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205932] = {
            [itemKeys.name] = "Prophecy of a King's Demise",
            [itemKeys.npcDrops] = {209742},
            [itemKeys.objectDrops] = {411674,414646},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205940] = {
            [itemKeys.name] = "Memory of a Dark Purpose",
            [itemKeys.npcDrops] = {327,1211,3128,3129,208196},
            [itemKeys.objectDrops] = {407734},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205942] = {
            [itemKeys.name] = "Chimaeric Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205943] = {
            [itemKeys.name] = "Prophecy of a Desecrated Citadel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205944] = {
            [itemKeys.name] = "Reciprocal Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205945] = {
            [itemKeys.name] = "Memory of an Imprisoned Savior",
            [itemKeys.npcDrops] = {40,43,60,471,476,1119,1260,1934,1935,2038,3204,3205,5808,5822,5826,6124,8503,13157},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205946] = {
            [itemKeys.name] = "Memory of a Leader's Betrayal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205947] = {
            [itemKeys.name] = "Prophecy of a Desecrated Citadel",
            [itemKeys.npcDrops] = {474,1124,1397,1535,1536,1537,2002,2003,2004,2005,3113,3118,3206,3207},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205948] = {
            [itemKeys.name] = "Anodyne Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205949] = {
            [itemKeys.name] = "Manifold Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205950] = {
            [itemKeys.name] = "Tenebrous Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [205951] = {
            [itemKeys.name] = "Memory of a Troubled Acolyte",
            [itemKeys.npcDrops] = {80},
            [itemKeys.objectDrops] = {405633},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205952] = {
            [itemKeys.name] = "Memory of a Shattered World",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205953] = {
            [itemKeys.name] = "Prophecy of a Broken Vow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205961] = {
            [itemKeys.name] = "Mulgore Bird Meat",
            [itemKeys.npcDrops] = {2956,2957,2969,2970,2971},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205979] = {
            [itemKeys.name] = "Rune of Flanking",
            [itemKeys.npcDrops] = {205382,208124,208812,209928},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [205995] = {
            [itemKeys.name] = "Prairie Dog Musk",
            [itemKeys.npcDrops] = {2949,2950,2951,207515},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206032] = {
            [itemKeys.name] = "Rune of Carve",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206155] = {
            [itemKeys.name] = "Rune of Marksmanship",
            [itemKeys.npcDrops] = {205700,208180,209812},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206157] = {
            [itemKeys.name] = "Seaforium Mining Charge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206159] = {
            [itemKeys.name] = "Venture Co Disguise",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206168] = {
            [itemKeys.name] = "Rune of the Chimera",
            [itemKeys.npcDrops] = {1994,3124},
            [itemKeys.objectDrops] = {405628},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206169] = {
            [itemKeys.name] = "Rune of Explosive Shot",
            [itemKeys.npcDrops] = {1130,1132,1137,1271,1388,1961,3056,3058,3068,3281,5807,7318,208638},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206170] = {
            [itemKeys.name] = "Windfury Cone",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206175] = {
            [itemKeys.name] = "Pine Salve",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206176] = {
            [itemKeys.name] = "Mortar and Pestle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206177] = {
            [itemKeys.name] = "Pine Nut Butter",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206264] = {
            [itemKeys.name] = "Rune of Inspiration",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206344] = {
            [itemKeys.name] = "Knife Set",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206345] = {
            [itemKeys.name] = "Fish Chunks",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206376] = {
            [itemKeys.name] = "Bulwark Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206381] = {
            [itemKeys.name] = "Dyadic Icon",
            [itemKeys.npcDrops] = {2953,3124},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206382] = {
            [itemKeys.name] = "Tempest Icon",
            [itemKeys.npcDrops] = {1972},
            [itemKeys.objectDrops] = {3642,152608,152618},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206383] = {
            [itemKeys.name] = "Lithic Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206384] = {
            [itemKeys.name] = "Astral Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206385] = {
            [itemKeys.name] = "Clastic Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206386] = {
            [itemKeys.name] = "Galvanic Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206387] = {
            [itemKeys.name] = "Kajaric Icon",
            [itemKeys.npcDrops] = {2373},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206388] = {
            [itemKeys.name] = "Sulfurous Icon",
            [itemKeys.npcDrops] = {208275},
            [itemKeys.objectDrops] = {404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206389] = {
            [itemKeys.name] = "Brimstone Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206390] = {
            [itemKeys.name] = "Ichthyan Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206391] = {
            [itemKeys.name] = "Syzygic Icon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206466] = {
            [itemKeys.name] = "Prairie Crown",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206469] = {
            [itemKeys.name] = "Prairie Flower",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206954] = {
            [itemKeys.name] = "Idol of Ursine Rage",
            [itemKeys.npcDrops] = {2960,2964,2965,2971,2979,3232,3566,7318},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206963] = {
            [itemKeys.name] = "Rune of Natural Potential",
            [itemKeys.npcDrops] = {206245},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206966] = {
            [itemKeys.name] = "Satyrweed Bulb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206970] = {
            [itemKeys.name] = "Rune of Life",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206975] = {
            [itemKeys.name] = "Artifact Storage Key",
            [itemKeys.npcDrops] = {2989,2990},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206985] = {
            [itemKeys.name] = "Echo of the Alpha",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206989] = {
            [itemKeys.name] = "Rune of the Sun",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {404433},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206992] = {
            [itemKeys.name] = "Rune of Skull Bash",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [206994] = {
            [itemKeys.name] = "Severed Quilboar Head",
            [itemKeys.npcDrops] = {3111,3112,3113,3114,3232},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [206995] = {
            [itemKeys.name] = "Severed Harpy Head",
            [itemKeys.npcDrops] = {2962,2963,2964,2965,3115,3116,3117,3118},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207062] = {
            [itemKeys.name] = "Severed Centaur Head",
            [itemKeys.npcDrops] = {3119,3120},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207098] = {
            [itemKeys.name] = "Note from Ba'so",
            [itemKeys.npcDrops] = {3183,3195,3196,3197,3198,3199,3203},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207106] = {
            [itemKeys.name] = "Bottom-Left Map Piece",
            [itemKeys.npcDrops] = {3206,3207},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207107] = {
            [itemKeys.name] = "Bottom-Right Map Piece",
            [itemKeys.npcDrops] = {3111,3112,3113,3114},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207108] = {
            [itemKeys.name] = "Top-Left Map Piece",
            [itemKeys.npcDrops] = {3195,3196,3197,3198,3199},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207109] = {
            [itemKeys.name] = "Top-Right Map Piece",
            [itemKeys.npcDrops] = {3128,3129},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207110] = {
            [itemKeys.name] = "Durotar Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207590] = {
            [itemKeys.name] = "Durotar Pig Meat",
            [itemKeys.npcDrops] = {3099,3100,3225},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207631] = {
            [itemKeys.name] = "Adder Pheromone",
            [itemKeys.npcDrops] = {3111,3112,3113,3114,5824,5826},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207731] = {
            [itemKeys.name] = "Ominous Tome",
            [itemKeys.npcDrops] = {3113,3118,3198,3206,3207},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207732] = {
            [itemKeys.name] = "Makrura Leg",
            [itemKeys.npcDrops] = {3103,3104,3105},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207733] = {
            [itemKeys.name] = "Kul Tiran Skull",
            [itemKeys.npcDrops] = {3128,3129},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207972] = {
            [itemKeys.name] = "The Lessons of Ta'zo",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207973] = {
            [itemKeys.name] = "Hound Jawbone",
            [itemKeys.npcDrops] = {1547,1548,1549},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207974] = {
            [itemKeys.name] = "Ominous Tome",
            [itemKeys.npcDrops] = {1522},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [207975] = {
            [itemKeys.name] = "Severed Bat Head",
            [itemKeys.npcDrops] = {1553,1554},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208005] = {
            [itemKeys.name] = "Agamand Relic Coffer Key",
            [itemKeys.npcDrops] = {1501,1502,1520,1522,1523,1525,1526,1527,1528,1529,1530,1534,1657,1674,1675,1890,1916,1917,1918,1941},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208007] = {
            [itemKeys.name] = "Shipwreck Cache Key",
            [itemKeys.npcDrops] = {1543,1544,1545},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208034] = {
            [itemKeys.name] = "Tirisfal Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208035] = {
            [itemKeys.name] = "Top-Right Map Piece",
            [itemKeys.npcDrops] = {1535,1536,1537,1538,1539,1540},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208036] = {
            [itemKeys.name] = "Top-Left Map Piece",
            [itemKeys.npcDrops] = {1934,1935},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208037] = {
            [itemKeys.name] = "Bottom-Right Map Piece",
            [itemKeys.npcDrops] = {1543,1544,1545},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208038] = {
            [itemKeys.name] = "Bottom-Left Map Piece",
            [itemKeys.npcDrops] = {1674,1675,1941},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208085] = {
            [itemKeys.name] = "Scarlet Lieutenant Signet Ring",
            [itemKeys.npcDrops] = {1662,1664},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208086] = {
            [itemKeys.name] = "Forged Scarlet Memorandum",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208139] = {
            [itemKeys.name] = "Ominous Tome",
            [itemKeys.npcDrops] = {1124,1397},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208140] = {
            [itemKeys.name] = "Wendigo Blood",
            [itemKeys.npcDrops] = {1134,1135},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208158] = {
            [itemKeys.name] = "Pristine Trogg Heart",
            [itemKeys.npcDrops] = {1115,1116,1117},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208159] = {
            [itemKeys.name] = "Severed Troll Head",
            [itemKeys.npcDrops] = {1120,1121,1122,1123,1124,1397},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208160] = {
            [itemKeys.name] = "Severed Wendigo Paw",
            [itemKeys.npcDrops] = {1134,1135},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208180] = {
            [itemKeys.name] = "Rabbit Musk",
            [itemKeys.npcDrops] = {1115,1116,1117,1118},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208183] = {
            [itemKeys.name] = "Apothecary Notes",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208185] = {
            [itemKeys.name] = "The Apothecary's Metaphysical Primer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {405879},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208192] = {
            [itemKeys.name] = "Dun Morogh Pig Meat",
            [itemKeys.npcDrops] = {1125,1126,1127,1689,208638},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208205] = {
            [itemKeys.name] = "Blackrat's Note",
            [itemKeys.npcDrops] = {6123},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208213] = {
            [itemKeys.name] = "Top-Right Map Piece",
            [itemKeys.npcDrops] = {706,946,1120,1121,1122,1123,1124,1397},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208215] = {
            [itemKeys.name] = "Top-Left Map Piece",
            [itemKeys.npcDrops] = {707,724,1115,1116,1718},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208218] = {
            [itemKeys.name] = "Bottom-Right Map Piece",
            [itemKeys.npcDrops] = {1211},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208219] = {
            [itemKeys.name] = "Bottom-Left Map Piece",
            [itemKeys.npcDrops] = {6123},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208220] = {
            [itemKeys.name] = "Dun Morogh Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208223] = {
            [itemKeys.name] = "Acolyte's Knapsack",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208224] = {
            [itemKeys.name] = "Acolyte's Note",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {208223},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208414] = {
            [itemKeys.name] = "Lunar Idol",
            [itemKeys.npcDrops] = {1989},
            [itemKeys.objectDrops] = {403102},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208601] = {
            [itemKeys.name] = "Top-Right Map Piece",
            [itemKeys.npcDrops] = {2022,2025,2027,2029,2030},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208602] = {
            [itemKeys.name] = "Top-Left Map Piece",
            [itemKeys.npcDrops] = {2009,2010,2011,2012,2013,2152},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208603] = {
            [itemKeys.name] = "Bottom-Right Map Piece",
            [itemKeys.npcDrops] = {2015,2017,2018,2019,2020,2021},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208604] = {
            [itemKeys.name] = "Bottom-Left Map Piece",
            [itemKeys.npcDrops] = {2002,2003,2004,2005},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208605] = {
            [itemKeys.name] = "Teldrassil Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208607] = {
            [itemKeys.name] = "Deer Musk",
            [itemKeys.npcDrops] = {2009,2010,2011,2012,2013,2014,2152},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208608] = {
            [itemKeys.name] = "Teldrassil Bird Meat",
            [itemKeys.npcDrops] = {1995,1996,1997},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208609] = {
            [itemKeys.name] = "Glade Flower",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208610] = {
            [itemKeys.name] = "Severed Owl Head",
            [itemKeys.npcDrops] = {1995,1996,1997},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208611] = {
            [itemKeys.name] = "Severed Tiger Head",
            [itemKeys.npcDrops] = {2033,2042,2043},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208612] = {
            [itemKeys.name] = "Severed Spider Head",
            [itemKeys.npcDrops] = {1998,1999,2001},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208682] = {
            [itemKeys.name] = "Abandoned Snapjaw Egg",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208687] = {
            [itemKeys.name] = "Rune of Lacerate",
            [itemKeys.npcDrops] = {210483},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208689] = {
            [itemKeys.name] = "Ferocious Idol",
            [itemKeys.npcDrops] = {500,1972,6788},
            [itemKeys.objectDrops] = {3642,152608,152618},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208701] = {
            [itemKeys.name] = "Rune of Beast Mastery",
            [itemKeys.npcDrops] = {209524,210533,211736},
            [itemKeys.objectDrops] = {407918},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208739] = {
            [itemKeys.name] = "Horde Warbanner",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208741] = {
            [itemKeys.name] = "Rune of Endless Rage",
            [itemKeys.npcDrops] = {391,506,519,520,1424,2478,209607,210451,211146},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208743] = {
            [itemKeys.name] = "Soul of Greed",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208744] = {
            [itemKeys.name] = "Rune of Shadowbolts",
            [itemKeys.npcDrops] = {2191,3662,10373,209958,210487,212753},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208749] = {
            [itemKeys.name] = "Gnarlpine Stash Key",
            [itemKeys.npcDrops] = {2011,2012,2013},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208750] = {
            [itemKeys.name] = "Rune of Channeling",
            [itemKeys.npcDrops] = {2336,210501,212763},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208753] = {
            [itemKeys.name] = "Spell Notes: Regeneration",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208754] = {
            [itemKeys.name] = "Spell Notes: TENGI RONEERA",
            [itemKeys.npcDrops] = {450,589,1867},
            [itemKeys.objectDrops] = {3642,152608,152618,407983},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208758] = {
            [itemKeys.name] = "Earthen Rune",
            [itemKeys.npcDrops] = {209742,211875},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208760] = {
            [itemKeys.name] = "Glade Crown",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208764] = {
            [itemKeys.name] = "Rune of the Mind",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208765] = {
            [itemKeys.name] = "Helping Hand",
            [itemKeys.npcDrops] = {3265,3266,3267,3268,3457},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208766] = {
            [itemKeys.name] = "Helping Hand",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208767] = {
            [itemKeys.name] = "Rune of Mending",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208768] = {
            [itemKeys.name] = "Buccaneer's Matchbox",
            [itemKeys.npcDrops] = {3381,3382,3383,3384},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208771] = {
            [itemKeys.name] = "Rune of Blade Dance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {414532},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208772] = {
            [itemKeys.name] = "Rune of Saber Slash",
            [itemKeys.npcDrops] = {210549},
            [itemKeys.objectDrops] = {407731,409131,414624},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208773] = {
            [itemKeys.name] = "Fishing Harpoon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3497},
            [itemKeys.startQuest] = nil,
        },
        [208777] = {
            [itemKeys.name] = "Rune of the Sniper",
            [itemKeys.npcDrops] = {209797,210107,210482,210549},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208778] = {
            [itemKeys.name] = "Rune of Quick Strike",
            [itemKeys.npcDrops] = {209797,210482,210487},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {208848},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208799] = {
            [itemKeys.name] = "Spell Notes: Living Bomb",
            [itemKeys.npcDrops] = {210501},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208800] = {
            [itemKeys.name] = "Baxtan: On Destructive Magics",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {407566},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208823] = {
            [itemKeys.name] = "Offering Coin",
            [itemKeys.npcDrops] = {1174,1175,1177},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208828] = {
            [itemKeys.name] = "Memory of a Broken Vow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208833] = {
            [itemKeys.name] = "Malevolent Pie",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3291},
            [itemKeys.startQuest] = nil,
        },
        [208838] = {
            [itemKeys.name] = "Dark Iron Lockbox",
            [itemKeys.npcDrops] = {1169,1222},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208843] = {
            [itemKeys.name] = "Battle Totem",
            [itemKeys.npcDrops] = {1178,1179,1180,1181,1183},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208847] = {
            [itemKeys.name] = "Skull-Shaped Geode",
            [itemKeys.npcDrops] = {1161,1162,1163,1164},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208848] = {
            [itemKeys.name] = "Cracked Skull-Shaped Geode",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208849] = {
            [itemKeys.name] = "Libram of Blessings",
            [itemKeys.npcDrops] = {210537},
            [itemKeys.objectDrops] = {407844},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208851] = {
            [itemKeys.name] = "Libram of Justice",
            [itemKeys.npcDrops] = {572,846,7050},
            [itemKeys.objectDrops] = {407850},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208853] = {
            [itemKeys.name] = "Spell Notes: Brain Freeze",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208854] = {
            [itemKeys.name] = "Chewed Spell Notes",
            [itemKeys.npcDrops] = {1166},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [208855] = {
            [itemKeys.name] = "Rainbow Fin Albacore Chum",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {1684},
            [itemKeys.startQuest] = nil,
        },
        [208860] = {
            [itemKeys.name] = "Rumi of Gnomeregan: The Collected Works",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {408014},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209027] = {
            [itemKeys.name] = "Crab Treats",
            [itemKeys.npcDrops] = {2167,2168,2169,2170,2171,2324},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209028] = {
            [itemKeys.name] = "Spell Notes: BIVOLG NIMB",
            [itemKeys.npcDrops] = {210451},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209029] = {
            [itemKeys.name] = "Message from Emily",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {209031},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209030] = {
            [itemKeys.name] = "Equipment Stash Key",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {209031},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209031] = {
            [itemKeys.name] = "Discreet Envelope",
            [itemKeys.npcDrops] = {95,121,449,481,504,589,590},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209041] = {
            [itemKeys.name] = "Magic Pumpkin Seeds",
            [itemKeys.npcDrops] = {95,121,122,449,481,504,589,590},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209042] = {
            [itemKeys.name] = "Fishy Bonemeal",
            [itemKeys.npcDrops] = {830,831,1216},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209043] = {
            [itemKeys.name] = "Fertile Soil Sample",
            [itemKeys.npcDrops] = {1236},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209045] = {
            [itemKeys.name] = "Soul of the Sea",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209047] = {
            [itemKeys.name] = "Gnarled Harpoon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {408802},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209056] = {
            [itemKeys.name] = "Spare Reaper Parts",
            [itemKeys.npcDrops] = {36,114,115,480},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209057] = {
            [itemKeys.name] = "Prototype Engine",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209058] = {
            [itemKeys.name] = "Elemental Core",
            [itemKeys.npcDrops] = {832},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209059] = {
            [itemKeys.name] = "Goretusk Haunch",
            [itemKeys.npcDrops] = {157,454},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209418] = {
            [itemKeys.name] = "Adamantine Tortoise Armor",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209420] = {
            [itemKeys.name] = "Gillsbane",
            [itemKeys.npcDrops] = {95,121,449,481,504,589,590},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209421] = {
            [itemKeys.name] = "Cord of Aquanis",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209422] = {
            [itemKeys.name] = "High Tide Choker",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209423] = {
            [itemKeys.name] = "Flowing Scarf",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209424] = {
            [itemKeys.name] = "Shell Plate Barrier",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209432] = {
            [itemKeys.name] = "Ghamoo-ra's Cinch",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209436] = {
            [itemKeys.name] = "Chipped Bite of Serra'kis",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209523] = {
            [itemKeys.name] = "Shimmering Thresher Cape",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209524] = {
            [itemKeys.name] = "Bindings of Serra'kis",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209525] = {
            [itemKeys.name] = "Honed Darkwater Talwar",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209526] = {
            [itemKeys.name] = "TEST Darkwater Talwar",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209527] = {
            [itemKeys.name] = "Naga Battle Gauntlets",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209534] = {
            [itemKeys.name] = "Azshari Arbalest",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209559] = {
            [itemKeys.name] = "Twilight Sage's Walking Stick",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209560] = {
            [itemKeys.name] = "Hammer of Righteous Judgement",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209561] = {
            [itemKeys.name] = "Rod of the Ancient Sleepwalker",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209562] = {
            [itemKeys.name] = "Deadly Strike of the Hydra",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209563] = {
            [itemKeys.name] = "Naga Heartrender",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209564] = {
            [itemKeys.name] = "Guardian's Trident",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209565] = {
            [itemKeys.name] = "Band of Deep Places",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209566] = {
            [itemKeys.name] = "Leggings of the Faithful",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209567] = {
            [itemKeys.name] = "Coral Reef Axe",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209568] = {
            [itemKeys.name] = "Algae Gauntlets",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209569] = {
            [itemKeys.name] = "Murloc Hide Kneeboots",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209570] = {
            [itemKeys.name] = "Tome of Cavern Lore",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209571] = {
            [itemKeys.name] = "Deadlight",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209572] = {
            [itemKeys.name] = "Black Boiled Leathers",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209573] = {
            [itemKeys.name] = "Wrathful Spire",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209574] = {
            [itemKeys.name] = "Discarded Tenets of the Silver Hand",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209575] = {
            [itemKeys.name] = "Carved Driftwood Icon",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209576] = {
            [itemKeys.name] = "Mind-Expanding Mushroom",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209577] = {
            [itemKeys.name] = "Fist of the Wild",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209578] = {
            [itemKeys.name] = "Glowing Leather Bands",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209579] = {
            [itemKeys.name] = "Crashing Thunder",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209580] = {
            [itemKeys.name] = "Gusting Wind",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209581] = {
            [itemKeys.name] = "Silver Hand Sabatons",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209590] = {
            [itemKeys.name] = "Cracked Water Globe",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209608] = {
            [itemKeys.name] = "Captain's Skeleton Key",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209667] = {
            [itemKeys.name] = "Gaze Dreamer Leggings",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209668] = {
            [itemKeys.name] = "Signet of the Twilight Lord",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209669] = {
            [itemKeys.name] = "Twilight Invoker's Shoes",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209670] = {
            [itemKeys.name] = "Skinwalkers",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209671] = {
            [itemKeys.name] = "Twilight Invoker's Robes",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209672] = {
            [itemKeys.name] = "Black Fingerless Gloves",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209673] = {
            [itemKeys.name] = "Glowing Fetish Amulet",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209674] = {
            [itemKeys.name] = "Phoenix Ignition",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209675] = {
            [itemKeys.name] = "Clamweave Tunic",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209676] = {
            [itemKeys.name] = "Shoulderguards of Crushing Depths",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209677] = {
            [itemKeys.name] = "Loop of Swift Currents",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209678] = {
            [itemKeys.name] = "Mantle of the Thresher Slayer",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209679] = {
            [itemKeys.name] = "Azshari Novice's Shoulderpads",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209680] = {
            [itemKeys.name] = "Waterproof Scarf",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209681] = {
            [itemKeys.name] = "Black Murloc Egg",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209682] = {
            [itemKeys.name] = "Sturdy Hood",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209683] = {
            [itemKeys.name] = "Twilight Invoker's Shawl",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209684] = {
            [itemKeys.name] = "Soul Leech Pants",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209685] = {
            [itemKeys.name] = "Ancient Moss Cinch",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209686] = {
            [itemKeys.name] = "Jagged Bone Necklace",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209687] = {
            [itemKeys.name] = "Hydra Hide Cuirass",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209688] = {
            [itemKeys.name] = "Bael Modan Blunderbuss",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209689] = {
            [itemKeys.name] = "Crabshell Waders",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209690] = {
            [itemKeys.name] = "Shadowscale Coif",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209691] = {
            [itemKeys.name] = "Vampiric Boot Knife",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209692] = {
            [itemKeys.name] = "Sentinel Pauldrons",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209693] = {
            [itemKeys.name] = "Perfect Blackfathom Pearl",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 78916,
        },
        [209694] = {
            [itemKeys.name] = "Blackfathom Ritual Dagger",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209748] = {
            [itemKeys.name] = "Althalaxx Orb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209778] = {
            [itemKeys.name] = "Summoner's Salt",
            [itemKeys.npcDrops] = {334,435,436,615,4065,4462,4463,4464},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209779] = {
            [itemKeys.name] = "Mote of Mannoroth",
            [itemKeys.npcDrops] = {6071,6073,6115,11697},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209800] = {
            [itemKeys.name] = "Orb Fragments",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209816] = {
            [itemKeys.name] = "Fetish of Mischief",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209817] = {
            [itemKeys.name] = "Voidwalker Brooch",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209818] = {
            [itemKeys.name] = "Sun-Touched Crescent",
            [itemKeys.npcDrops] = {207356},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209819] = {
            [itemKeys.name] = "Wyvern Taming Wand",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209820] = {
            [itemKeys.name] = "Black Shroud Choker",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209821] = {
            [itemKeys.name] = "Ring of Shadowsight",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209822] = {
            [itemKeys.name] = "Strength of Purpose",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209823] = {
            [itemKeys.name] = "Signet of Beasts",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209824] = {
            [itemKeys.name] = "Shimmering Shoulderpads",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209825] = {
            [itemKeys.name] = "Droplet Choker",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209828] = {
            [itemKeys.name] = "Sub-Zero Pauldrons",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209830] = {
            [itemKeys.name] = "Ironhide Arbalest",
            [itemKeys.npcDrops] = {201722},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209836] = {
            [itemKeys.name] = "Althalaxx Orb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409289},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209838] = {
            [itemKeys.name] = "Hypnotic Crystal",
            [itemKeys.npcDrops] = {3654},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209840] = {
            [itemKeys.name] = "Gnarled Wand of Wild Magic",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209841] = {
            [itemKeys.name] = "Wild Magic Essence",
            [itemKeys.npcDrops] = {3748,3750,3922,3924},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209843] = {
            [itemKeys.name] = "Nar'thalas Almanac, Vol. 74",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409496},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209844] = {
            [itemKeys.name] = "The Dalaran Digest, Vol. 23",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409501},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209845] = {
            [itemKeys.name] = "Bewitchments and Glamours",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409562},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209846] = {
            [itemKeys.name] = "Secrets of the Dreamers",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209847] = {
            [itemKeys.name] = "Arcanic Systems Manual",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409700},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209848] = {
            [itemKeys.name] = "Goaz Scrolls",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409717},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209849] = {
            [itemKeys.name] = "Crimes Against Anatomy",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409735},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209850] = {
            [itemKeys.name] = "Runes of the Sorcerer-Kings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409731},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209851] = {
            [itemKeys.name] = "Fury of the Land",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409711},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209852] = {
            [itemKeys.name] = "Rune of Kill Command",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209872] = {
            [itemKeys.name] = "Dragonslayer's Helm",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409758},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209873] = {
            [itemKeys.name] = "Dragonslayer's Shield",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {409754},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [209874] = {
            [itemKeys.name] = "Dragonslayer's Lance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3180},
            [itemKeys.startQuest] = nil,
        },
        [210015] = {
            [itemKeys.name] = "Rune of Raging Blow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210026] = {
            [itemKeys.name] = "Symbol of the Third Owl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210043] = {
            [itemKeys.name] = "Symbol of the Second Owl",
            [itemKeys.npcDrops] = {211200},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210044] = {
            [itemKeys.name] = "Symbol of the First Owl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210045] = {
            [itemKeys.name] = "Shadeleaf",
            [itemKeys.npcDrops] = {3780,3781,3782,3783,3784},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210053] = {
            [itemKeys.name] = "Dire Wolf Paw",
            [itemKeys.npcDrops] = {213,565,628,923},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210055] = {
            [itemKeys.name] = "Hillsbrad Human Bones",
            [itemKeys.npcDrops] = {2266,2267,2268,2360,2387},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210056] = {
            [itemKeys.name] = "Tincture of Waking Death",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210080] = {
            [itemKeys.name] = "Voodoo Offering",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210137] = {
            [itemKeys.name] = "Rune of Wild Growth",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210138] = {
            [itemKeys.name] = "Dark Iron Ordinance",
            [itemKeys.npcDrops] = {1051,1052,1053,1054},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210146] = {
            [itemKeys.name] = "Shredder Turbocharger",
            [itemKeys.npcDrops] = {642,4073,4074,11684,214129},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210147] = {
            [itemKeys.name] = "Shredder Autosalvage Unit",
            [itemKeys.npcDrops] = {3988,3992},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210177] = {
            [itemKeys.name] = "Ataeric: On Arcane Curiosities",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {410299},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210178] = {
            [itemKeys.name] = "Schematic: Shredder Autosalvage Unit",
            [itemKeys.npcDrops] = {643},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210179] = {
            [itemKeys.name] = "Spell Notes: ILMEGAI SARRBES",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210183] = {
            [itemKeys.name] = "Horn of Xelthos",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {410528},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210186] = {
            [itemKeys.name] = "Breaching Charge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210187] = {
            [itemKeys.name] = "Venture Co. Work Order",
            [itemKeys.npcDrops] = {3988,3992,3993,4070},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210188] = {
            [itemKeys.name] = "Shredded Work Order",
            [itemKeys.npcDrops] = {3988,3992,3993,4070,4202},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210195] = {
            [itemKeys.name] = "Unbalanced Idol",
            [itemKeys.npcDrops] = {1769,1770},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210209] = {
            [itemKeys.name] = "Twin Key",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210210] = {
            [itemKeys.name] = "Grizzwerks Defense Industries Founder's Certificate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210212] = {
            [itemKeys.name] = "Brother's Half-Key",
            [itemKeys.npcDrops] = {211764},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210213] = {
            [itemKeys.name] = "Sister's Half-Key",
            [itemKeys.npcDrops] = {211765},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210229] = {
            [itemKeys.name] = "Rune of Mutilation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210250] = {
            [itemKeys.name] = "Engraved Gold Ring",
            [itemKeys.npcDrops] = {215,909,910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210251] = {
            [itemKeys.name] = "Engraved Silver Ring",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210252] = {
            [itemKeys.name] = "Rune of Shiving",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210253] = {
            [itemKeys.name] = "Rot Totem",
            [itemKeys.npcDrops] = {1773,1940},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210322] = {
            [itemKeys.name] = "Rune of Venom",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {410847},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210323] = {
            [itemKeys.name] = "Safe Combination",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {210330},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210329] = {
            [itemKeys.name] = "Hillsbrad Treasure Map",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {210330},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210330] = {
            [itemKeys.name] = "Hot Tip",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3536},
            [itemKeys.startQuest] = nil,
        },
        [210410] = {
            [itemKeys.name] = "Freshwater Snapper Bait",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3537},
            [itemKeys.startQuest] = nil,
        },
        [210499] = {
            [itemKeys.name] = "Marshroom",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210500] = {
            [itemKeys.name] = "Rune of the Stars",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210534] = {
            [itemKeys.name] = "Idol of the Wild",
            [itemKeys.npcDrops] = {11910,11911,11912,11913},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210568] = {
            [itemKeys.name] = "Decrepit Phylactery",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {411348},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210569] = {
            [itemKeys.name] = "Rune of Flagellation",
            [itemKeys.npcDrops] = {212261},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210573] = {
            [itemKeys.name] = "Rune of Consuming Rage",
            [itemKeys.npcDrops] = {211965},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210589] = {
            [itemKeys.name] = "Echo of the Ancestors",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210596] = {
            [itemKeys.name] = "Rune of Cobra Strikes",
            [itemKeys.npcDrops] = {211951},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210653] = {
            [itemKeys.name] = "Rune of Main Gauche",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210654] = {
            [itemKeys.name] = "Spell Notes: Rewind Time",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210655] = {
            [itemKeys.name] = "Spell Notes: TERWEM DINI",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210665] = {
            [itemKeys.name] = "Mote of Torrential Rage",
            [itemKeys.npcDrops] = {3917},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210667] = {
            [itemKeys.name] = "Mote of Infernal Rage",
            [itemKeys.npcDrops] = {4036,4037,4038},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210668] = {
            [itemKeys.name] = "Mote of Seismic Rage",
            [itemKeys.npcDrops] = {4034,4035},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210696] = {
            [itemKeys.name] = "Rot Bane",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210708] = {
            [itemKeys.name] = "Elixir of Coalesced Regret",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210709] = {
            [itemKeys.name] = "Recipe: Elixir of Coalesced Regret",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3134,3537},
            [itemKeys.startQuest] = nil,
        },
        [210712] = {
            [itemKeys.name] = "Elixir of Insight",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210713] = {
            [itemKeys.name] = "Tortured Soul",
            [itemKeys.npcDrops] = {1973,1974},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210724] = {
            [itemKeys.name] = "Hirzek's Staff",
            [itemKeys.npcDrops] = {212694},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210741] = {
            [itemKeys.name] = "Automatic Crowd Pummeler",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210746] = {
            [itemKeys.name] = "Rune of Earth Shield",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210763] = {
            [itemKeys.name] = "Bough of Altek",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {411715},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210765] = {
            [itemKeys.name] = "Orb of Des",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {411710},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210771] = {
            [itemKeys.name] = "Waylaid Supplies: Copper Bars",
            [itemKeys.npcDrops] = {40,43,60,95,97,114,116,121,123,126,390,456,473,478,480,500,501,504,515,524,550,589,590,712,735,830,833,880,881,1115,1116,1117,1122,1124,1135,1161,1162,1163,1166,1172,1174,1175,1177,1186,1188,1190,1193,1195,1202,1211,1236,1380,1426,1520,1522,1523,1527,1529,1530,1531,1532,1534,1535,1538,1543,1544,1655,1656,1662,1674,1675,1765,1768,1769,1772,1773,1778,1779,1782,1797,1866,1867,1934,1953,1971,1993,1996,1997,1999,2000,2004,2005,2008,2009,2010,2011,2014,2015,2017,2018,2019,2020,2021,2025,2029,2030,2033,2038,2039,2043,2069,2070,2149,2152,2167,2177,2178,2185,2189,2190,2202,2204,2231,2232,2234,2321,2324,2950,2957,2959,2960,2963,2970,2976,2977,2978,2979,2989,2990,3035,3058,3100,3103,3104,3107,3110,3113,3114,3118,3120,3121,3123,3126,3129,3196,3197,3199,3205,3206,3207,3227,3232,3243,3246,3265,3266,3267,3268,3271,3272,3274,3282,3284,3285,3379,3381,3382,3397,3415,3425,3566,4008,5807,6093,6123,6846,6866,7235,10157,10158,10159,10160,11910,11911,11913,12138,14428,14429},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210772] = {
            [itemKeys.name] = "Supply Shipment: Copper Bars",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210773] = {
            [itemKeys.name] = "Mantle of the Second War",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210779] = {
            [itemKeys.name] = "Plans: Mantle of the Second War",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [210781] = {
            [itemKeys.name] = "Phoenix Bindings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210794] = {
            [itemKeys.name] = "Shifting Silver Breastplate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210795] = {
            [itemKeys.name] = "Extraplanar Spidersilk Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210811] = {
            [itemKeys.name] = "Rune of Primordial Fury",
            [itemKeys.npcDrops] = {212837},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210817] = {
            [itemKeys.name] = "Rune of Survival",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210818] = {
            [itemKeys.name] = "Rune of Lone Wolf",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210820] = {
            [itemKeys.name] = "Rune of Sacrifice",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210822] = {
            [itemKeys.name] = "Harmonious Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210823] = {
            [itemKeys.name] = "Rune of Dual Wield Specialization",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210824] = {
            [itemKeys.name] = "Rune of the Pact",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210825] = {
            [itemKeys.name] = "Rune of the Warbringer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {211653},
            [itemKeys.startQuest] = nil,
        },
        [210953] = {
            [itemKeys.name] = "Eye of Bhossca",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {412261},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210955] = {
            [itemKeys.name] = "Scarlet Initiate's Uniform",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210963] = {
            [itemKeys.name] = "Scryer's Key",
            [itemKeys.npcDrops] = {4293},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210966] = {
            [itemKeys.name] = "Blood of the Legion",
            [itemKeys.npcDrops] = {6071,6073,6115,11697},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210967] = {
            [itemKeys.name] = "Confidential Message",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {412759},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210968] = {
            [itemKeys.name] = "Reliquary Key",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {412198},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210979] = {
            [itemKeys.name] = "Rune of Shadowstep",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [210980] = {
            [itemKeys.name] = "Rune of Metamorphosis",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211205] = {
            [itemKeys.name] = "Rune of Heart of the Lion",
            [itemKeys.npcDrops] = {211965},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211247] = {
            [itemKeys.name] = "Pattern: Phoenix Bindings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211263] = {
            [itemKeys.name] = "Judgement Chestguard",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211269] = {
            [itemKeys.name] = "Primitive Drawing",
            [itemKeys.npcDrops] = {2701,2715,2716,2717,2718,2893,2894,2906,2907},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 78823,
        },
        [211272] = {
            [itemKeys.name] = "Empty Bait Cage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211273] = {
            [itemKeys.name] = "Trapped Critter",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211293] = {
            [itemKeys.name] = "Crimson Trophy Quill",
            [itemKeys.npcDrops] = {213795},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211301] = {
            [itemKeys.name] = "Rune of Expose Weakness",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211315] = {
            [itemKeys.name] = "Waylaid Supplies: Light Leather",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,114,116,117,119,199,285,330,456,473,474,476,478,480,500,501,502,504,513,515,524,589,590,712,732,735,830,881,1117,1122,1123,1124,1135,1161,1162,1163,1164,1166,1172,1174,1176,1177,1188,1190,1195,1196,1197,1211,1236,1380,1426,1522,1528,1529,1531,1532,1533,1535,1536,1539,1544,1554,1654,1656,1658,1662,1664,1689,1765,1767,1768,1769,1770,1772,1773,1779,1780,1781,1782,1865,1866,1867,1868,1869,1934,1941,1971,1981,1983,1993,1997,2008,2009,2010,2011,2012,2013,2014,2015,2018,2019,2020,2021,2025,2027,2029,2030,2033,2038,2062,2065,2069,2070,2149,2163,2164,2167,2176,2177,2178,2179,2185,2189,2190,2202,2212,2234,2321,2322,2324,2950,2951,2960,2963,2964,2965,2970,2971,2977,2978,2979,2990,3035,3051,3058,3103,3107,3111,3112,3114,3117,3120,3121,3122,3129,3130,3197,3198,3207,3225,3231,3232,3241,3246,3265,3267,3269,3271,3272,3274,3284,3379,3381,3397,3425,3566,5865,6123,6789,6846,6866,7234,7235,10158,10159,10160,10356,10358,11910,11911,12138,14429,14431},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211316] = {
            [itemKeys.name] = "Waylaid Supplies: Peacebloom",
            [itemKeys.npcDrops] = {36,40,43,46,79,95,97,114,117,123,126,199,285,327,390,456,472,473,478,500,513,550,589,590,712,732,735,830,833,881,1065,1115,1117,1121,1122,1124,1127,1131,1135,1161,1163,1166,1172,1174,1175,1176,1177,1186,1188,1190,1201,1202,1236,1397,1520,1522,1523,1530,1531,1535,1543,1545,1656,1657,1658,1660,1689,1693,1765,1769,1770,1772,1773,1778,1779,1781,1782,1797,1865,1866,1868,1870,1922,1924,1934,1953,1996,1997,1999,2000,2001,2004,2008,2009,2010,2011,2012,2015,2018,2020,2029,2030,2034,2038,2043,2061,2066,2069,2070,2149,2152,2162,2164,2167,2176,2177,2178,2179,2185,2189,2190,2201,2202,2234,2321,2322,2324,2950,2951,2957,2959,2962,2963,2965,2971,2972,2978,2979,2990,3056,3103,3107,3112,3113,3114,3117,3118,3121,3122,3123,3129,3131,3195,3197,3198,3199,3203,3204,3206,3207,3226,3232,3243,3246,3265,3266,3267,3268,3269,3271,3274,3276,3284,3381,3382,3397,3425,3438,4316,6093,6123,6789,6846,6866,6927,7235,7318,7319,10157,10158,10159,10160,11910,11911,11912,12432,208638},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211317] = {
            [itemKeys.name] = "Waylaid Supplies: Silverleaf",
            [itemKeys.npcDrops] = {40,46,95,97,114,116,117,118,123,285,390,423,456,478,480,481,500,501,502,504,513,515,550,590,712,732,735,824,880,1115,1117,1118,1122,1132,1135,1161,1162,1163,1164,1166,1172,1174,1175,1177,1193,1201,1202,1236,1426,1520,1527,1528,1530,1532,1534,1535,1539,1545,1549,1654,1656,1660,1662,1664,1665,1674,1675,1768,1769,1770,1772,1773,1778,1779,1781,1782,1797,1866,1868,1869,1910,1912,1936,1993,2000,2004,2008,2009,2010,2014,2015,2018,2019,2020,2021,2025,2027,2029,2030,2033,2038,2039,2043,2062,2069,2070,2149,2152,2164,2167,2176,2177,2178,2179,2189,2190,2191,2201,2202,2203,2204,2212,2234,2321,2324,2950,2951,2957,2959,2960,2962,2963,2964,2965,2970,2971,2974,2976,2977,2978,2979,2989,3068,3099,3103,3104,3107,3110,3112,3114,3116,3117,3118,3120,3122,3126,3127,3197,3198,3204,3206,3226,3232,3234,3265,3266,3267,3268,3271,3273,3282,3284,3379,3380,3381,3382,3383,3397,3415,3425,3438,3566,4316,6093,6123,6789,6866,7234,7235,7318,8503,10157,10159,11910,11911,11912,11913,12320,202060,209742},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211318] = {
            [itemKeys.name] = "Waylaid Supplies: Minor Healing Potions",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,116,117,121,123,157,199,285,327,330,454,456,471,473,474,476,478,501,504,513,515,524,550,589,590,712,732,834,880,1065,1109,1115,1117,1124,1126,1135,1161,1162,1163,1164,1166,1172,1174,1175,1177,1188,1201,1202,1211,1236,1397,1426,1520,1527,1528,1529,1530,1531,1532,1538,1539,1544,1545,1548,1549,1655,1656,1660,1664,1753,1769,1770,1772,1773,1778,1779,1782,1869,1870,1934,1953,1981,1983,1996,1997,1999,2000,2008,2009,2010,2011,2012,2015,2017,2025,2027,2029,2030,2033,2034,2069,2070,2152,2163,2164,2167,2177,2178,2185,2189,2190,2202,2203,2212,2231,2232,2321,2324,2950,2951,2960,2963,2964,2965,2970,2971,2976,2977,2978,2979,2989,2990,3035,3056,3099,3103,3104,3111,3112,3118,3119,3120,3121,3122,3123,3126,3130,3196,3197,3198,3204,3206,3207,3232,3241,3242,3243,3246,3248,3254,3266,3267,3269,3271,3273,3274,3276,3277,3282,3379,3381,3382,3397,3415,4263,4316,5786,6093,6846,6866,6927,7234,10158,10160,11911,12320,12431,202060,208638,208875,210549},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211319] = {
            [itemKeys.name] = "Waylaid Supplies: Copper Shortswords",
            [itemKeys.npcDrops] = {40,43,95,99,114,116,117,121,126,285,327,423,454,456,473,476,478,481,501,504,515,524,589,590,712,732,831,833,834,880,881,1065,1115,1117,1123,1124,1131,1133,1135,1161,1162,1166,1172,1173,1174,1175,1176,1191,1194,1195,1197,1202,1211,1380,1426,1522,1527,1528,1529,1530,1531,1534,1538,1539,1540,1544,1545,1549,1655,1656,1658,1662,1675,1693,1753,1767,1768,1769,1772,1773,1778,1779,1782,1865,1866,1867,1924,1934,1971,1972,1996,1999,2000,2004,2005,2008,2009,2010,2011,2012,2013,2014,2017,2018,2019,2025,2027,2029,2033,2069,2070,2152,2164,2167,2173,2176,2177,2178,2179,2185,2189,2190,2191,2201,2203,2212,2231,2321,2322,2324,2950,2951,2957,2959,2963,2964,2965,2971,2976,2978,2979,2989,2990,3100,3103,3104,3107,3110,3111,3112,3115,3116,3117,3119,3121,3122,3123,3126,3129,3196,3197,3203,3204,3205,3206,3226,3231,3232,3242,3243,3246,3265,3266,3267,3269,3273,3284,3381,3382,3415,3425,3438,3566,4263,4316,6093,6494,6846,6909,7234,7235,10158,10159,10160,11910,11911,14428,208638},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211320] = {
            [itemKeys.name] = "Waylaid Supplies: Runed Copper Pants",
            [itemKeys.npcDrops] = {36,40,46,95,97,114,116,117,118,121,123,124,126,157,285,327,330,390,456,473,476,481,500,501,504,513,515,524,550,589,590,712,732,830,833,1115,1126,1127,1131,1133,1135,1161,1162,1163,1164,1172,1174,1175,1176,1177,1186,1188,1190,1193,1202,1211,1236,1397,1426,1522,1526,1528,1529,1535,1537,1538,1543,1544,1554,1555,1655,1662,1665,1675,1689,1693,1765,1766,1767,1769,1770,1772,1773,1778,1779,1780,1781,1782,1865,1866,1868,1870,1924,1934,1953,1972,1981,2000,2005,2008,2009,2010,2017,2018,2019,2025,2029,2030,2033,2039,2043,2060,2069,2070,2152,2164,2167,2176,2177,2189,2190,2202,2203,2212,2231,2234,2321,2324,2950,2951,2956,2960,2963,2965,2968,2976,2977,2978,2979,2989,2990,3035,3056,3058,3100,3103,3104,3112,3114,3115,3117,3129,3130,3195,3196,3197,3198,3199,3203,3204,3205,3206,3207,3225,3227,3231,3232,3248,3255,3265,3266,3267,3268,3269,3272,3273,3284,3285,3381,3383,3425,3566,6093,6123,6789,6866,7235,9523,10158,10159,10160,10357,11910,11911,11913,12138,208196},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211321] = {
            [itemKeys.name] = "Waylaid Supplies: Lesser Magic Wands",
            [itemKeys.npcDrops] = {36,40,43,79,95,97,114,116,119,123,126,157,171,285,327,390,423,454,456,473,476,478,481,500,502,504,515,550,589,590,732,735,833,1065,1117,1122,1123,1124,1135,1138,1161,1162,1163,1164,1166,1173,1174,1175,1176,1177,1186,1194,1196,1397,1426,1520,1533,1534,1538,1543,1545,1554,1654,1657,1665,1689,1765,1769,1770,1773,1778,1782,1868,1869,1870,1934,1993,1996,2001,2004,2008,2010,2012,2013,2014,2015,2017,2018,2021,2025,2029,2030,2033,2043,2067,2069,2070,2149,2152,2163,2167,2176,2177,2178,2179,2189,2190,2201,2202,2203,2204,2205,2212,2231,2234,2321,2324,2950,2951,2957,2959,2960,2962,2963,2971,2972,2974,2976,2978,2979,2989,2990,3103,3104,3105,3111,3112,3113,3114,3115,3116,3117,3118,3119,3120,3122,3126,3129,3192,3195,3197,3198,3203,3205,3207,3232,3242,3246,3265,3266,3267,3268,3269,3271,3272,3273,3275,3276,3282,3284,3285,3381,3382,3384,3397,3415,3425,3535,3566,4263,6093,6123,6124,6789,6866,6927,7235,7319,10157,10160,11910,11911,11913,14268,202060},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211322] = {
            [itemKeys.name] = "Waylaid Supplies: Minor Wizard Oil",
            [itemKeys.npcDrops] = {40,43,46,95,97,114,116,118,119,121,124,126,157,285,330,390,423,454,456,476,478,480,504,513,515,524,550,589,590,712,732,735,833,834,1115,1117,1123,1124,1135,1138,1161,1162,1163,1166,1172,1174,1175,1177,1188,1190,1195,1196,1197,1201,1202,1211,1236,1380,1426,1522,1523,1526,1527,1529,1530,1531,1532,1533,1534,1535,1536,1538,1540,1544,1548,1553,1554,1662,1674,1693,1753,1765,1766,1767,1768,1769,1770,1772,1779,1780,1781,1782,1866,1868,1912,1941,1953,1997,2001,2004,2005,2008,2010,2011,2012,2015,2018,2019,2020,2025,2027,2033,2039,2043,2069,2070,2152,2163,2164,2167,2176,2177,2178,2185,2189,2190,2203,2204,2212,2234,2321,2324,2950,2951,2956,2957,2959,2960,2962,2964,2965,2976,2978,2979,3035,3058,3099,3103,3107,3110,3111,3113,3114,3115,3117,3118,3119,3120,3122,3123,3126,3127,3129,3192,3196,3197,3198,3203,3204,3205,3207,3226,3232,3234,3244,3246,3265,3267,3268,3269,3271,3274,3275,3276,3277,3282,3284,3379,3380,3381,3382,3397,3425,4316,6123,6789,7235,7318,10159},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211323] = {
            [itemKeys.name] = "Waylaid Supplies: Rough Copper Bombs",
            [itemKeys.npcDrops] = {40,43,46,95,97,114,116,117,118,119,123,124,126,157,171,285,390,471,474,478,481,500,501,513,515,550,732,735,830,881,1115,1116,1117,1123,1124,1126,1135,1161,1162,1163,1164,1166,1173,1174,1177,1184,1190,1195,1196,1197,1211,1236,1397,1426,1522,1523,1528,1529,1533,1535,1537,1538,1544,1549,1555,1654,1657,1658,1662,1674,1675,1689,1765,1766,1769,1770,1772,1773,1778,1779,1782,1797,1866,1924,1934,1953,1996,1997,2004,2005,2008,2009,2010,2011,2012,2013,2014,2015,2017,2020,2025,2027,2030,2034,2038,2039,2043,2066,2069,2070,2163,2164,2167,2176,2177,2178,2190,2202,2203,2234,2321,2322,2324,2957,2959,2960,2963,2965,2970,2971,2976,2978,2979,2989,2990,3035,3103,3104,3107,3110,3111,3112,3114,3119,3120,3121,3122,3123,3129,3131,3192,3195,3196,3197,3198,3199,3203,3205,3206,3207,3232,3242,3246,3254,3265,3266,3267,3268,3269,3273,3282,3284,3379,3380,3381,3382,3383,3397,3415,3425,3566,4263,4316,5826,6494,6846,6927,7234,7235,7318,7319,10159,11910,11911,11913,12138,208124},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211324] = {
            [itemKeys.name] = "Waylaid Supplies: Rough Boomsticks",
            [itemKeys.npcDrops] = {40,43,95,97,114,116,118,119,285,327,330,454,473,474,476,478,480,500,504,515,589,590,712,735,833,1109,1115,1116,1117,1118,1124,1127,1132,1135,1161,1162,1163,1166,1172,1173,1174,1175,1176,1177,1184,1186,1190,1193,1195,1196,1201,1236,1426,1520,1522,1523,1530,1532,1534,1535,1539,1540,1543,1549,1553,1656,1657,1658,1660,1662,1674,1675,1765,1766,1770,1772,1779,1780,1782,1797,1867,1868,1869,1922,1934,1941,1953,1972,1999,2004,2005,2009,2010,2011,2012,2013,2015,2020,2021,2027,2029,2030,2038,2060,2064,2067,2069,2070,2149,2163,2164,2167,2176,2177,2178,2185,2190,2201,2204,2232,2234,2321,2322,2324,2950,2951,2959,2960,2962,2963,2965,2970,2971,2977,2978,2979,2989,3035,3056,3104,3110,3111,3112,3115,3116,3119,3120,3122,3126,3127,3129,3192,3195,3196,3204,3205,3206,3207,3231,3232,3234,3246,3266,3267,3268,3269,3271,3272,3273,3284,3285,3380,3381,3382,3383,3397,3415,3425,3438,3566,4316,6123,6789,6846,6866,7234,7235,7318,10158,10159,11910,11911,208638},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211325] = {
            [itemKeys.name] = "Waylaid Supplies: Handstitched Leather Belts",
            [itemKeys.npcDrops] = {43,46,61,95,97,116,118,119,126,285,327,454,473,478,481,500,501,504,513,524,550,589,590,712,732,735,830,880,1065,1116,1117,1118,1122,1123,1126,1127,1131,1133,1135,1161,1162,1163,1164,1166,1172,1175,1176,1184,1186,1195,1196,1197,1202,1236,1397,1426,1523,1530,1531,1532,1538,1539,1540,1543,1545,1554,1555,1655,1662,1664,1675,1689,1766,1767,1768,1769,1772,1773,1779,1781,1797,1866,1867,1868,1934,1971,1981,1996,2001,2004,2008,2009,2010,2011,2012,2015,2018,2019,2025,2027,2030,2033,2034,2066,2069,2070,2152,2163,2164,2167,2176,2177,2178,2179,2185,2189,2190,2201,2204,2212,2232,2234,2235,2321,2324,2951,2956,2957,2960,2963,2964,2965,2970,2971,2976,2978,2979,2989,2990,3035,3051,3058,3103,3104,3110,3111,3112,3114,3115,3116,3117,3118,3119,3122,3126,3131,3192,3196,3197,3198,3199,3204,3205,3225,3232,3234,3243,3246,3248,3254,3265,3266,3268,3271,3272,3274,3284,3285,3379,3381,3382,3397,3425,6123,6128,6789,6846,6866,6927,7235,7318,10158,10160,11910,11911,12138,14428},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211326] = {
            [itemKeys.name] = "Waylaid Supplies: Embossed Leather Vests",
            [itemKeys.npcDrops] = {43,46,60,97,114,116,117,118,123,126,285,327,390,456,472,474,476,478,500,501,502,504,515,524,550,589,735,830,880,1115,1116,1117,1123,1127,1135,1161,1162,1163,1166,1172,1173,1174,1175,1184,1195,1196,1197,1211,1236,1380,1397,1424,1426,1520,1523,1527,1528,1529,1530,1532,1538,1539,1540,1544,1545,1654,1655,1656,1658,1662,1674,1753,1769,1770,1772,1773,1779,1782,1797,1865,1867,1868,1936,1941,1953,1972,1993,1996,1997,1999,2000,2004,2005,2008,2009,2011,2014,2015,2017,2018,2019,2020,2027,2029,2030,2034,2043,2061,2069,2070,2149,2152,2164,2167,2176,2177,2178,2179,2185,2189,2190,2201,2202,2203,2204,2234,2321,2322,2324,2951,2956,2959,2960,2963,2964,2971,2973,2974,2976,2977,2978,2979,2990,3035,3056,3068,3107,3110,3111,3112,3113,3114,3117,3119,3120,3122,3123,3126,3129,3195,3196,3197,3198,3199,3203,3204,3207,3227,3232,3234,3243,3246,3255,3266,3268,3269,3271,3274,3275,3284,3380,3381,3382,3397,3425,3566,4316,6123,6494,6789,6866,7235,7318,7319,11910,12432,202060,210107},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211327] = {
            [itemKeys.name] = "Waylaid Supplies: Brown Linen Pants",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,116,117,121,123,124,126,285,327,456,476,478,480,500,501,589,590,712,830,833,880,1117,1124,1126,1127,1130,1131,1135,1161,1162,1166,1172,1173,1174,1177,1184,1186,1201,1202,1211,1236,1380,1397,1426,1526,1527,1528,1529,1532,1533,1534,1535,1536,1540,1545,1549,1655,1656,1657,1664,1674,1675,1689,1765,1766,1767,1768,1769,1770,1772,1773,1781,1782,1797,1866,1867,1869,1914,1934,1941,1953,1971,1972,2000,2001,2004,2005,2008,2009,2010,2012,2020,2021,2027,2030,2066,2069,2070,2149,2152,2163,2164,2167,2176,2177,2178,2185,2189,2190,2201,2202,2204,2232,2321,2322,2324,2950,2951,2956,2957,2960,2962,2963,2964,2965,2968,2970,2971,2972,2976,2977,2978,2979,2989,2990,3035,3058,3068,3103,3107,3110,3111,3113,3114,3115,3116,3117,3118,3119,3122,3126,3129,3131,3141,3195,3196,3197,3204,3205,3206,3207,3232,3242,3246,3265,3266,3269,3271,3272,3274,3275,3276,3282,3284,3285,3381,3382,3397,3415,3425,3566,4316,5786,6123,6866,7235,7318,10157,10158,11911,11913,12433,208638},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211328] = {
            [itemKeys.name] = "Waylaid Supplies: Brown Linen Robes",
            [itemKeys.npcDrops] = {40,43,46,95,97,114,116,117,121,123,126,157,285,327,330,390,456,473,476,478,480,481,500,504,513,550,589,590,732,735,830,833,880,1115,1117,1123,1131,1135,1161,1162,1163,1164,1166,1172,1174,1186,1195,1196,1211,1236,1380,1426,1520,1522,1526,1527,1528,1530,1531,1532,1534,1535,1536,1538,1545,1553,1554,1656,1660,1664,1675,1765,1766,1768,1770,1773,1779,1781,1782,1797,1866,1914,1922,1981,1996,2001,2004,2005,2009,2011,2012,2014,2017,2025,2029,2030,2033,2034,2043,2060,2069,2070,2163,2164,2167,2176,2177,2178,2185,2190,2201,2203,2204,2205,2231,2232,2234,2321,2324,2950,2951,2959,2960,2962,2963,2965,2970,2971,2976,2978,2979,2989,2990,3051,3103,3104,3107,3110,3111,3112,3113,3114,3116,3118,3120,3122,3127,3129,3130,3131,3192,3195,3196,3197,3206,3225,3232,3241,3243,3246,3254,3255,3265,3266,3267,3268,3269,3271,3272,3273,3274,3276,3284,3380,3381,3382,3383,3397,3415,3566,5807,6866,6927,7235,7318,10158,10159,10160,11910,11911,11912,11913,202060,208876},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211329] = {
            [itemKeys.name] = "Waylaid Supplies: Herb Baked Eggs",
            [itemKeys.npcDrops] = {36,40,43,95,97,114,116,118,121,123,126,285,473,474,478,480,501,502,504,513,515,589,590,712,732,735,830,833,881,1115,1118,1119,1123,1124,1126,1131,1133,1135,1161,1163,1164,1166,1175,1177,1186,1196,1201,1236,1426,1520,1522,1523,1527,1528,1531,1532,1536,1538,1540,1654,1655,1656,1657,1658,1660,1674,1689,1769,1770,1772,1773,1778,1779,1781,1782,1797,1866,1867,1868,1912,1924,1981,1983,1996,2000,2004,2008,2011,2013,2014,2017,2018,2019,2020,2027,2029,2030,2039,2043,2060,2061,2069,2070,2149,2152,2164,2167,2176,2177,2185,2189,2190,2202,2203,2205,2212,2231,2232,2234,2321,2324,2951,2960,2963,2964,2968,2970,2977,2978,2979,2989,2990,3035,3058,3068,3103,3107,3111,3112,3113,3114,3115,3116,3117,3120,3122,3123,3126,3131,3196,3197,3198,3203,3206,3227,3232,3243,3246,3248,3254,3255,3265,3266,3268,3269,3271,3273,3274,3275,3276,3284,3380,3381,3382,3438,4263,4316,6123,6789,6846,6866,6927,7234,7235,7319,10157,10158,10159,10160,10356,11911,11912,11913,12138,14429,202060,208196},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211330] = {
            [itemKeys.name] = "Waylaid Supplies: Spiced Wolf Meat",
            [itemKeys.npcDrops] = {36,40,46,95,97,114,116,117,123,126,285,390,473,474,476,478,480,501,504,515,524,550,589,712,732,735,880,881,1065,1115,1116,1117,1121,1122,1124,1127,1131,1135,1161,1162,1164,1166,1172,1173,1174,1175,1176,1177,1190,1195,1197,1201,1211,1236,1426,1522,1523,1526,1527,1531,1534,1535,1536,1537,1538,1539,1540,1543,1544,1655,1656,1657,1660,1664,1665,1689,1766,1768,1769,1770,1773,1779,1781,1782,1797,1865,1866,1867,1868,1924,1941,1971,1996,2000,2005,2009,2010,2011,2012,2013,2015,2017,2018,2020,2021,2027,2030,2033,2038,2043,2063,2065,2069,2070,2149,2152,2163,2164,2167,2174,2176,2177,2178,2185,2189,2190,2203,2212,2321,2322,2324,2950,2951,2957,2959,2960,2962,2963,2964,2965,2970,2971,2977,2978,2979,2989,2990,3035,3104,3107,3112,3113,3114,3115,3117,3195,3196,3197,3198,3203,3204,3206,3207,3226,3242,3243,3246,3248,3265,3267,3268,3269,3272,3274,3282,3284,3285,3380,3381,3382,3425,4263,5823,5826,6123,6124,6846,6866,7318,9523,10158,10159,10160,11910,11912,12138,208638,209928},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211331] = {
            [itemKeys.name] = "Waylaid Supplies: Brilliant Smallfish",
            [itemKeys.npcDrops] = {40,46,95,97,116,117,119,121,123,126,285,330,476,478,481,500,504,515,524,589,590,712,732,822,830,834,1109,1115,1117,1123,1124,1127,1133,1135,1161,1162,1163,1164,1166,1174,1175,1176,1177,1184,1190,1194,1201,1202,1211,1236,1380,1426,1520,1522,1523,1526,1527,1530,1531,1535,1536,1539,1540,1544,1545,1654,1657,1660,1662,1664,1665,1674,1675,1689,1765,1769,1770,1773,1779,1781,1782,1797,1866,1936,1972,1981,1996,1999,2000,2004,2008,2009,2010,2011,2015,2017,2018,2019,2020,2021,2029,2030,2033,2038,2043,2067,2069,2070,2152,2163,2164,2167,2174,2176,2177,2178,2185,2189,2190,2201,2202,2203,2212,2231,2234,2321,2324,2950,2951,2956,2957,2959,2962,2963,2964,2965,2970,2971,2976,2977,2978,2979,2989,2990,3035,3056,3058,3104,3111,3113,3117,3118,3119,3120,3121,3127,3129,3195,3197,3198,3199,3206,3241,3242,3254,3255,3265,3266,3267,3268,3269,3274,3275,3276,3284,3379,3381,3382,3383,3384,3397,3425,3438,3566,4008,5807,5865,6124,6789,6846,6866,7235,7318,9523,10158,10159,11910,11911,209742},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211332] = {
            [itemKeys.name] = "Waylaid Supplies: Heavy Linen Bandages",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,99,116,117,118,119,123,126,157,171,330,423,456,473,474,476,478,480,481,500,501,504,513,515,524,589,590,712,732,735,880,881,1117,1122,1123,1124,1126,1135,1161,1163,1166,1172,1173,1175,1177,1190,1196,1202,1236,1426,1520,1523,1530,1535,1538,1539,1544,1545,1548,1654,1655,1657,1662,1689,1753,1765,1769,1770,1772,1773,1779,1780,1781,1782,1797,1865,1866,1867,1868,1870,1912,1922,1972,1981,1997,1999,2005,2008,2009,2010,2011,2012,2013,2015,2017,2021,2027,2030,2033,2034,2043,2061,2062,2067,2069,2070,2149,2152,2163,2167,2176,2177,2178,2185,2189,2190,2201,2202,2204,2232,2321,2324,2950,2951,2957,2959,2960,2964,2965,2970,2974,2976,2977,2978,2979,2989,2990,3035,3051,3103,3111,3113,3114,3115,3117,3118,3119,3120,3122,3123,3129,3196,3197,3198,3199,3206,3207,3226,3232,3242,3243,3265,3266,3267,3268,3273,3274,3282,3284,3285,3380,3382,3397,3415,3566,4008,4263,4316,5786,6123,6133,6789,6846,6866,7318,10158,10160,14431,210549},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211365] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211367] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211368] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211382] = {
            [itemKeys.name] = "Small Courier Satchel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211384] = {
            [itemKeys.name] = "Sturdy Courier Bag",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211385] = {
            [itemKeys.name] = "Rune of Serpent Spread",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211386] = {
            [itemKeys.name] = "Spell Notes: Arcane Surge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211387] = {
            [itemKeys.name] = "Rune of Beckoning Light",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211390] = {
            [itemKeys.name] = "Rune of Teasing",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211391] = {
            [itemKeys.name] = "Rune of Healing Rain",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211392] = {
            [itemKeys.name] = "Rune of Everlasting Affliction",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211393] = {
            [itemKeys.name] = "Rune of Single-Minded Fury",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [211419] = {
            [itemKeys.name] = "Handful of Shifting Scales",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211420] = {
            [itemKeys.name] = "Shifting Scale Talisman",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {214954},
            [itemKeys.startQuest] = nil,
        },
        [211421] = {
            [itemKeys.name] = "The Box",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211422] = {
            [itemKeys.name] = "Shard of the Void",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211423] = {
            [itemKeys.name] = "Void-Touched Leather Gloves",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211426] = {
            [itemKeys.name] = "Bough of Shadows",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {414197},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211427] = {
            [itemKeys.name] = "Soul Vessel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211443] = {
            [itemKeys.name] = "Blade of Rage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211444] = {
            [itemKeys.name] = "Summoner's Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211445] = {
            [itemKeys.name] = "Barbaric Recurve",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211447] = {
            [itemKeys.name] = "Arms Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211448] = {
            [itemKeys.name] = "Signed Procurement Contract",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211449] = {
            [itemKeys.name] = "Avenger's Void Pearl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {214954},
            [itemKeys.startQuest] = nil,
        },
        [211450] = {
            [itemKeys.name] = "Invoker's Void Pearl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {214954},
            [itemKeys.startQuest] = nil,
        },
        [211451] = {
            [itemKeys.name] = "Acolyte's Void Pearl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {214954},
            [itemKeys.startQuest] = nil,
        },
        [211452] = {
            [itemKeys.name] = "Perfect Blackfathom Pearl",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 78917,
        },
        [211454] = {
            [itemKeys.name] = "Strange Water Globe",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 78920,
        },
        [211455] = {
            [itemKeys.name] = "Slick Fingerless Gloves",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211456] = {
            [itemKeys.name] = "Dagger of Willing Sacrifice",
            [itemKeys.npcDrops] = {213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211457] = {
            [itemKeys.name] = "Twilight Defender's Girdle",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211458] = {
            [itemKeys.name] = "Tome of Shadow Warding",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211460] = {
            [itemKeys.name] = "Ancient Arctic Buckler",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211461] = {
            [itemKeys.name] = "Inscribed Gravestone Scepter",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211462] = {
            [itemKeys.name] = "Ever-Sustaining Ring",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211463] = {
            [itemKeys.name] = "Chittering Beetle Clasps",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211464] = {
            [itemKeys.name] = "Worn Prelacy Cape",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211465] = {
            [itemKeys.name] = "Nimbus Boots of Insight",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211466] = {
            [itemKeys.name] = "Tender's Heartwood Girdle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211467] = {
            [itemKeys.name] = "Band of the Iron Fist",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211468] = {
            [itemKeys.name] = "Frayed Chestnut Mantle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211471] = {
            [itemKeys.name] = "Cliffspring Key",
            [itemKeys.npcDrops] = {2179,2180},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211472] = {
            [itemKeys.name] = "Libram of Banishment",
            [itemKeys.npcDrops] = {215,909,910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211476] = {
            [itemKeys.name] = "Rune of Fires Wake",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {414713},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211477] = {
            [itemKeys.name] = "Rune of Incinerate",
            [itemKeys.npcDrops] = {214519},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211482] = {
            [itemKeys.name] = "Shatterspear Offering",
            [itemKeys.npcDrops] = {2181,2182,2183},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211487] = {
            [itemKeys.name] = "Demolition Explosives",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3537},
            [itemKeys.startQuest] = nil,
        },
        [211488] = {
            [itemKeys.name] = "Rune of the Avenger",
            [itemKeys.npcDrops] = {214456},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211490] = {
            [itemKeys.name] = "Prophecy of a Thousand Lights",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211491] = {
            [itemKeys.name] = "Bottomless Murloc Skin Bag",
            [itemKeys.npcDrops] = {204921},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211492] = {
            [itemKeys.name] = "Kelris's Satchel",
            [itemKeys.npcDrops] = {209678},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211498] = {
            [itemKeys.name] = "Trainee's Sentinel Nightsaber",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753},
            [itemKeys.startQuest] = nil,
        },
        [211499] = {
            [itemKeys.name] = "Trainee's Outrider Wolf",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14754},
            [itemKeys.startQuest] = nil,
        },
        [211500] = {
            [itemKeys.name] = "Resilient Cloth Headband",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753,14754},
            [itemKeys.startQuest] = nil,
        },
        [211501] = {
            [itemKeys.name] = "Chestguard of Might",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211502] = {
            [itemKeys.name] = "Void-Touched Leather Gauntlets",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211504] = {
            [itemKeys.name] = "Twilight Avenger's Chain",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211505] = {
            [itemKeys.name] = "Twilight Avenger's Helm",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211506] = {
            [itemKeys.name] = "Twilight Avenger's Boots",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211507] = {
            [itemKeys.name] = "Twilight Elementalist's Cowl",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211508] = {
            [itemKeys.name] = "Twilight Elementalist's Footpads",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211509] = {
            [itemKeys.name] = "Twilight Elementalist's Robe",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211510] = {
            [itemKeys.name] = "Twilight Slayer's Cowl",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211511] = {
            [itemKeys.name] = "Twilight Slayer's Footpads",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211512] = {
            [itemKeys.name] = "Twilight Slayer's Tunic",
            [itemKeys.npcDrops] = {204921,207356,209678,213334},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211514] = {
            [itemKeys.name] = "Spell Notes: Mass Regeneration",
            [itemKeys.npcDrops] = {212261},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211527] = {
            [itemKeys.name] = "Cozy Sleeping Bag",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211528] = {
            [itemKeys.name] = "Dark Insight",
            [itemKeys.npcDrops] = {215,909,910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211530] = {
            [itemKeys.name] = "Prophecy of a City Enthralled",
            [itemKeys.npcDrops] = {436,214519},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211531] = {
            [itemKeys.name] = "Prophecy of Seven Visitors",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211534] = {
            [itemKeys.name] = "Primal Insight",
            [itemKeys.npcDrops] = {3748,3750,3922,3924},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211691] = {
            [itemKeys.name] = "Spell Notes: Arcane Blast",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211777] = {
            [itemKeys.name] = "Naga Manuscript",
            [itemKeys.npcDrops] = {3715,3717},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211779] = {
            [itemKeys.name] = "Comprehension Charm",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {958,983,1257,1275,1307,1308,1351,1463,1673,2805,3323,3335,3351,3490,3500,3542,3562,3700,3970,4083,4220,4562,4575,4878,5110,5151,8361,13476,14739,15175},
            [itemKeys.startQuest] = nil,
        },
        [211780] = {
            [itemKeys.name] = "Scroll: KWYJIBO",
            [itemKeys.npcDrops] = {36,40,43,46,61,95,97,114,116,117,118,123,126,285,327,330,390,454,473,474,476,478,480,500,504,513,515,589,590,712,732,735,822,830,833,834,881,1115,1116,1117,1121,1122,1123,1124,1127,1135,1161,1162,1163,1164,1166,1172,1173,1174,1175,1176,1177,1184,1186,1190,1195,1196,1197,1201,1202,1211,1397,1426,1520,1522,1523,1526,1527,1528,1529,1530,1532,1534,1535,1536,1537,1538,1540,1543,1545,1553,1554,1555,1662,1664,1665,1674,1675,1689,1693,1765,1767,1769,1772,1779,1782,1797,1866,1867,1912,1914,1922,1934,1941,1949,1953,1971,1981,2008,2009,2011,2013,2017,2020,2033,2069,2070,2152,2167,2176,2177,2178,2179,2185,2189,2190,2201,2202,2203,2204,2212,2231,2232,2234,2321,2322,2951,2956,2963,2964,2965,2971,2978,2979,2990,3056,3103,3107,3111,3112,3113,3115,3116,3117,3118,3120,3129,3192,3195,3196,3197,3205,3206,3242,3255,3266,3267,3269,3271,3273,3274,3284,3380,3381,3397,3425,4316,5787,6789,6866,7234,7235,7318,9523,10159,10160,11911,11913,210537,210549},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211784] = {
            [itemKeys.name] = "Scroll: WUBBA WUBBA",
            [itemKeys.npcDrops] = {98,124,205,218,232,345,422,423,424,429,430,440,441,445,452,453,517,518,533,544,545,548,568,578,580,598,785,819,898,909,910,923,1007,1008,1011,1015,1017,1018,1019,1022,1023,1024,1025,1027,1028,1031,1038,1044,1110,1164,1165,1167,1197,1206,1216,1259,1353,1393,1487,1888,1889,1908,1915,1939,1942,1943,1953,1955,1956,1973,2054,2102,2103,2156,2157,2158,2165,2168,2170,2180,2181,2183,2206,2207,2208,2233,2237,2260,2265,2337,2339,2360,2372,2373,2387,2427,2428,2449,2450,3247,3252,3257,3261,3263,3278,3282,3374,3375,3376,3377,3393,3394,3416,3424,3435,3452,3454,3456,3457,3458,3466,3471,3476,3711,3712,3713,3717,3728,3730,3732,3733,3734,3737,3739,3740,3742,3743,3745,3748,3750,3767,3772,3780,3781,3782,3783,3784,3810,3812,3817,3818,3819,3879,3921,3923,3944,3989,3991,3999,4001,4003,4012,4013,4015,4016,4022,4025,4051,4070,4073,4093,4096,4126,4416,4463,4627,6071,6115,9524,10323,10760,11697,11714,11858,11913,11917,12896,12897,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211785] = {
            [itemKeys.name] = "Scroll: CWAL",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,99,114,116,117,118,123,124,126,171,285,327,390,456,473,474,476,478,480,481,500,501,504,513,515,550,589,590,712,735,830,834,1109,1115,1117,1121,1122,1123,1126,1133,1135,1161,1162,1163,1164,1166,1172,1173,1174,1175,1177,1184,1186,1188,1190,1195,1197,1201,1211,1236,1397,1425,1426,1520,1522,1523,1528,1530,1531,1532,1534,1535,1536,1537,1538,1540,1543,1544,1545,1549,1555,1654,1656,1674,1675,1693,1753,1765,1767,1768,1769,1770,1772,1773,1778,1779,1780,1781,1782,1797,1865,1866,1867,1869,1914,1922,1941,1953,2004,2008,2009,2011,2012,2017,2019,2030,2033,2043,2063,2069,2149,2152,2167,2176,2177,2178,2185,2190,2191,2201,2204,2212,2232,2234,2324,2950,2951,2957,2964,2965,2971,2977,2978,2979,2990,3056,3104,3107,3108,3110,3111,3113,3115,3118,3121,3122,3126,3192,3197,3198,3204,3207,3242,3246,3265,3273,3274,3276,3284,3285,3379,3380,3381,3397,3415,3425,3438,4263,6093,6123,6866,9523,10158,10159,10358,11911,12433,202060,208196,209742},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211786] = {
            [itemKeys.name] = "Scroll: CHAP BALK WELLES",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,114,116,117,119,121,123,126,157,285,327,330,423,454,456,474,478,480,500,504,515,550,583,589,590,712,732,735,830,833,834,880,1115,1116,1117,1120,1123,1124,1127,1135,1161,1162,1163,1164,1166,1172,1173,1174,1175,1176,1177,1186,1190,1195,1196,1201,1202,1211,1236,1380,1426,1520,1522,1523,1528,1529,1530,1534,1535,1537,1538,1540,1543,1544,1545,1548,1554,1555,1674,1693,1753,1766,1767,1768,1769,1770,1772,1778,1779,1780,1781,1782,1797,1865,1866,1867,1869,1870,1911,1912,1934,1936,1941,1972,1996,1997,2004,2008,2013,2020,2021,2025,2027,2029,2038,2043,2060,2061,2062,2069,2152,2164,2167,2177,2178,2185,2189,2190,2204,2321,2322,2324,2950,2956,2962,2963,2965,2970,2971,2976,2977,2979,2989,3035,3103,3104,3107,3110,3113,3118,3119,3120,3127,3129,3131,3195,3196,3197,3198,3203,3204,3205,3206,3241,3242,3246,3265,3266,3268,3269,3274,3275,3277,3284,3379,3380,3381,3382,3383,3397,3415,3425,4316,6124,6789,6866,7235,7318,9523,10160,11910,11911,12431,208875},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211787] = {
            [itemKeys.name] = "Scroll: LOWER PING WHOMEVER",
            [itemKeys.npcDrops] = {36,40,43,46,95,97,114,116,118,123,126,285,330,390,454,456,473,474,476,481,504,513,550,590,712,732,735,822,830,833,834,880,1109,1115,1117,1118,1120,1121,1122,1123,1126,1127,1135,1138,1161,1162,1163,1166,1172,1173,1174,1175,1177,1186,1190,1195,1196,1197,1202,1211,1236,1380,1426,1520,1522,1523,1527,1529,1530,1532,1534,1535,1536,1537,1539,1540,1543,1553,1554,1555,1654,1656,1657,1660,1664,1674,1675,1753,1765,1766,1767,1769,1770,1772,1773,1778,1779,1781,1782,1866,1867,1868,1914,1922,1934,1941,1953,1999,2000,2004,2008,2010,2012,2013,2017,2018,2021,2027,2029,2030,2033,2039,2061,2063,2067,2069,2070,2152,2167,2176,2177,2178,2185,2189,2190,2202,2204,2205,2212,2232,2324,2951,2957,2960,2964,2965,2971,2976,2978,2979,3058,3103,3104,3110,3113,3114,3118,3119,3121,3122,3195,3205,3207,3228,3265,3266,3268,3271,3272,3273,3276,3284,3380,3381,3382,3397,3415,3425,3438,4008,6123,6866,7235,7318,10157,10158,10159,10160,11910,11913,14428,208196},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211789] = {
            [itemKeys.name] = "Artemis Cowl",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211799] = {
            [itemKeys.name] = "Sack of Stolen Goods",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211800] = {
            [itemKeys.name] = "Scroll of Reintegration",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211809] = {
            [itemKeys.name] = "Comprehension Primer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211813] = {
            [itemKeys.name] = "Silverwing Sentinel Charm",
            [itemKeys.npcDrops] = {212699,212703,212706,212707,212803,212804},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79098,
        },
        [211814] = {
            [itemKeys.name] = "Warsong Outrider Mark",
            [itemKeys.npcDrops] = {212727,212728,212729,212730,212801,212802},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79090,
        },
        [211815] = {
            [itemKeys.name] = "Silverwing Battle Hymn",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211816] = {
            [itemKeys.name] = "Warsong Battle Drum",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211818] = {
            [itemKeys.name] = "Strange Water Globe",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211819] = {
            [itemKeys.name] = "Waylaid Supplies: Bronze Bars",
            [itemKeys.npcDrops] = {98,213,232,424,429,431,435,437,440,441,445,449,453,458,485,533,544,545,548,579,598,615,832,891,892,898,1009,1011,1012,1015,1016,1017,1018,1019,1024,1027,1035,1039,1044,1069,1164,1169,1185,1216,1222,1251,1400,1727,1889,1909,1924,1939,1942,1943,1954,1955,1973,2053,2103,2156,2157,2158,2159,2160,2168,2169,2170,2180,2183,2187,2206,2207,2208,2233,2236,2260,2261,2323,2332,2336,2338,2339,2370,2372,2373,2427,2428,2503,3240,3247,3283,3374,3386,3396,3455,3457,3458,3503,3662,3664,3667,3711,3712,3715,3721,3725,3727,3728,3730,3732,3734,3737,3740,3742,3743,3746,3748,3750,3772,3780,3781,3782,3783,3784,3808,3809,3812,3814,3816,3817,3819,3823,3893,3922,3924,3932,3943,3944,3986,3987,3988,3989,3991,3992,3993,3998,3999,4004,4007,4009,4011,4013,4019,4022,4027,4028,4032,4051,4057,4073,4074,4094,4096,4099,4114,4129,4416,4463,4627,5053,5682,5834,6071,6073,6115,6180,6547,6606,8886,9524,10323,11680,11697,11858,11912,12896,12897,14280,209797,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3714,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211820] = {
            [itemKeys.name] = "Waylaid Supplies: Silver Bars",
            [itemKeys.npcDrops] = {98,217,232,423,426,435,445,452,458,518,533,568,579,598,615,703,832,889,898,909,1011,1015,1018,1019,1024,1025,1026,1028,1033,1040,1042,1043,1044,1069,1167,1185,1189,1192,1206,1400,1418,1487,1727,1870,1889,1913,1924,1940,1942,1943,1954,1955,1956,1973,2071,2102,2120,2156,2157,2159,2160,2168,2169,2170,2171,2180,2181,2183,2207,2208,2250,2269,2270,2305,2332,2336,2338,2339,2370,2372,2373,2427,2428,2448,2449,2503,3235,3279,3283,3376,3386,3394,3396,3416,3434,3455,3456,3459,3466,3472,3473,3664,3711,3712,3725,3727,3728,3733,3734,3743,3745,3746,3749,3750,3774,3780,3782,3783,3784,3809,3810,3812,3816,3819,3834,3879,3893,3923,3928,3932,3944,3987,3989,3991,3992,3999,4003,4004,4005,4006,4009,4011,4012,4013,4014,4015,4021,4029,4031,4034,4053,4070,4073,4074,4112,4114,4264,4416,4463,4627,5053,6071,6073,6115,6170,6180,6547,6606,6788,8886,9336,9523,10323,10373,10758,11681,11697,11913,12856,12897,14270,14271,209797,210107,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211821] = {
            [itemKeys.name] = "Waylaid Supplies: Medium Leather",
            [itemKeys.npcDrops] = {48,98,115,345,430,435,437,440,441,445,449,450,452,453,485,517,545,548,568,579,580,598,615,628,889,891,910,920,1017,1018,1019,1024,1025,1027,1032,1033,1038,1039,1040,1042,1043,1044,1165,1205,1207,1216,1222,1224,1259,1393,1400,1727,1755,1915,1923,1939,1940,1943,1954,1973,1974,2054,2071,2102,2156,2157,2165,2168,2169,2170,2171,2174,2179,2180,2181,2182,2183,2206,2207,2208,2233,2237,2244,2260,2265,2269,2336,2338,2339,2368,2370,2372,2373,2403,2503,3236,3239,3249,3252,3279,3282,3283,3286,3374,3378,3386,3455,3456,3457,3473,3503,3711,3712,3715,3725,3727,3728,3730,3732,3733,3734,3740,3743,3745,3746,3748,3749,3750,3763,3772,3774,3781,3782,3784,3789,3791,3809,3814,3816,3817,3818,3820,3823,3893,3899,3923,3925,3926,3928,3931,3943,3944,3989,3991,3992,3999,4006,4008,4009,4012,4013,4016,4031,4032,4053,4057,4074,4096,4113,4120,4202,4416,4463,4627,5053,5683,6071,6073,6115,6167,6547,8886,9523,10323,10373,11656,11680,11697,11915,12677,12759,12860,12940,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211822] = {
            [itemKeys.name] = "Waylaid Supplies: Bruiseweed",
            [itemKeys.npcDrops] = {98,122,203,206,215,424,432,434,435,437,441,445,446,449,452,453,511,545,598,889,1008,1011,1015,1016,1017,1018,1019,1024,1026,1027,1028,1030,1032,1039,1040,1042,1043,1044,1083,1164,1165,1169,1206,1216,1400,1727,1870,1888,1889,1909,1913,1924,1939,1954,1957,1958,1973,1974,2071,2102,2120,2156,2157,2160,2169,2170,2171,2180,2186,2187,2206,2207,2237,2323,2336,2338,2339,2356,2368,2370,2372,2387,2403,2428,2449,3237,3252,3253,3261,3374,3377,3386,3393,3455,3456,3466,3472,3473,3474,3475,3660,3662,3711,3712,3721,3725,3727,3728,3730,3733,3734,3740,3743,3745,3746,3749,3750,3765,3772,3780,3781,3782,3784,3809,3812,3814,3816,3817,3819,3823,3824,3825,3833,3879,3893,3925,3926,3943,3944,3986,3987,3989,3991,3992,3993,3999,4001,4002,4003,4004,4005,4013,4014,4019,4021,4029,4031,4057,4070,4096,4097,4099,4111,4112,4129,4416,4463,4541,4627,5053,5830,6071,6073,6115,6167,6547,6606,8886,9456,9524,10323,11680,11697,11858,11915,12897,14272,185334,212837,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211823] = {
            [itemKeys.name] = "Waylaid Supplies: Swiftthistle",
            [itemKeys.npcDrops] = {98,122,215,232,424,429,431,432,433,435,437,441,445,449,452,453,458,485,533,545,548,579,598,615,703,785,831,832,889,892,1012,1018,1019,1025,1026,1028,1030,1038,1040,1042,1043,1044,1069,1165,1167,1169,1189,1206,1216,1222,1224,1393,1755,1908,1909,1913,1915,1939,1942,1943,1953,1954,1955,1957,2071,2102,2158,2160,2168,2169,2170,2171,2180,2181,2182,2183,2208,2237,2265,2269,2336,2338,2339,2354,2368,2370,2372,2403,2427,2428,3239,3240,3283,3286,3374,3393,3396,3424,3452,3454,3467,3473,3474,3619,3662,3711,3712,3721,3725,3727,3730,3732,3733,3734,3742,3743,3745,3746,3748,3750,3772,3780,3782,3783,3784,3799,3801,3803,3809,3812,3814,3816,3817,3819,3823,3825,3919,3923,3926,3987,3988,3989,3991,3998,3999,4002,4003,4012,4013,4014,4016,4021,4028,4031,4051,4070,4093,4096,4097,4099,4202,4416,4463,4541,4627,5053,6071,6073,6115,6180,6547,6788,8886,9524,10373,10617,11680,11697,11858,11912,12676,15692,209524,210451,210482,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211824] = {
            [itemKeys.name] = "Waylaid Supplies: Lesser Mana Potions",
            [itemKeys.npcDrops] = {115,121,122,127,154,203,213,215,430,431,432,435,441,452,458,545,579,598,615,628,819,831,889,892,898,1009,1016,1018,1019,1021,1024,1027,1030,1033,1035,1038,1040,1044,1069,1169,1189,1197,1216,1259,1393,1400,1727,1909,1913,1924,1939,1940,1942,1943,1954,1955,1956,1973,1974,2053,2071,2102,2156,2157,2158,2169,2179,2180,2181,2182,2183,2187,2207,2208,2236,2261,2264,2265,2270,2332,2336,2338,2339,2368,2370,2372,2427,3247,3280,3282,3283,3286,3374,3426,3435,3452,3454,3455,3457,3458,3467,3471,3472,3473,3474,3503,3667,3712,3715,3721,3725,3727,3730,3732,3742,3745,3748,3750,3781,3782,3783,3784,3797,3799,3802,3804,3808,3809,3810,3814,3815,3816,3817,3819,3823,3824,3825,3833,3879,3893,3926,3944,3986,3987,3988,3989,3991,3999,4004,4009,4013,4016,4017,4019,4025,4028,4031,4035,4053,4074,4094,4110,4117,4248,4264,4416,4463,4627,5053,6071,6073,6115,6180,6547,6668,8886,9336,9523,10323,11681,11697,11714,11913,11921,12677,12678,12860,12940,185334,210697,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211825] = {
            [itemKeys.name] = "Waylaid Supplies: Rough Bronze Boots",
            [itemKeys.npcDrops] = {98,203,206,212,217,218,315,345,422,433,435,437,440,441,445,446,449,450,453,485,517,518,544,568,579,598,703,711,891,898,1010,1012,1016,1018,1019,1024,1026,1028,1029,1032,1036,1038,1041,1044,1069,1169,1185,1205,1224,1259,1353,1400,1755,1888,1908,1924,1939,1954,1955,1956,1974,2102,2120,2158,2168,2169,2170,2179,2180,2181,2182,2183,2187,2206,2233,2237,2264,2270,2323,2336,2338,2339,2354,2356,2368,2372,2387,2428,2451,3251,3256,3275,3283,3286,3374,3375,3395,3396,3434,3445,3452,3457,3458,3471,3503,3667,3712,3725,3727,3728,3730,3732,3733,3734,3740,3743,3745,3749,3750,3772,3774,3780,3782,3783,3784,3804,3809,3814,3816,3817,3819,3820,3823,3924,3926,3928,3931,3988,3989,3991,3992,3993,3998,3999,4002,4003,4013,4014,4019,4026,4029,4031,4044,4051,4070,4073,4094,4096,4117,4129,4264,4416,4463,4541,4627,5053,6071,6073,6115,6180,6547,6788,8886,9336,9456,9524,10896,11697,11858,11918,11921,12759,12856,14270,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211826] = {
            [itemKeys.name] = "Waylaid Supplies: Silver Skeleton Keys",
            [itemKeys.npcDrops] = {115,203,212,232,424,429,430,432,435,440,441,445,452,453,485,517,533,545,568,578,579,598,615,703,889,898,909,1007,1012,1015,1016,1018,1019,1022,1024,1025,1027,1028,1031,1033,1040,1042,1044,1069,1083,1164,1165,1167,1169,1205,1207,1216,1222,1259,1353,1400,1418,1727,1913,1942,1943,1954,1955,1956,1958,1973,2071,2102,2103,2156,2158,2165,2169,2170,2171,2180,2181,2183,2206,2208,2237,2244,2260,2261,2264,2269,2336,2338,2339,2372,2373,2404,2449,2451,3236,3249,3252,3283,3286,3374,3375,3376,3394,3396,3435,3452,3456,3458,3459,3463,3473,3475,3503,3662,3667,3711,3715,3721,3725,3727,3732,3733,3739,3743,3748,3750,3752,3767,3771,3772,3774,3780,3781,3782,3783,3784,3789,3799,3806,3808,3809,3814,3816,3817,3818,3820,3823,3879,3893,3900,3924,3943,3944,3989,4002,4003,4006,4013,4015,4016,4019,4021,4031,4042,4057,4070,4074,4093,4094,4110,4128,4416,4463,4627,4969,5053,6071,6073,6115,6180,6547,8886,10373,10761,11697,11858,11918,11921,185333,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211827] = {
            [itemKeys.name] = "Waylaid Supplies: Runed Silver Rods",
            [itemKeys.npcDrops] = {48,98,122,127,202,213,215,232,422,423,424,426,428,429,430,431,432,433,434,435,440,441,446,449,453,458,485,505,518,531,545,547,548,565,568,578,598,615,711,824,889,909,923,930,948,1007,1008,1017,1020,1021,1022,1024,1025,1027,1028,1040,1083,1111,1164,1165,1167,1194,1197,1400,1417,1727,1888,1889,1914,1915,1939,1940,1942,1953,1974,2071,2156,2157,2159,2169,2180,2181,2182,2187,2207,2208,2233,2235,2237,2244,2260,2261,2266,2267,2268,2322,2323,2336,2338,2339,2350,2351,2384,3239,3240,3245,3249,3256,3258,3260,3263,3275,3277,3278,3279,3280,3282,3283,3286,3374,3376,3385,3394,3396,3457,3458,3459,3461,3713,3721,3725,3732,3739,3743,3750,3762,3772,3774,3781,3809,3814,3816,3824,3921,3928,3944,3988,3989,3991,3992,4003,4004,4005,4006,4007,4012,4013,4014,4022,4024,4025,4036,4040,4044,4051,4074,4094,4096,4112,4416,4463,4627,5053,8886,9523,9524,11912,11917,214129},
            [itemKeys.objectDrops] = {2849,2850,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211828] = {
            [itemKeys.name] = "Waylaid Supplies: Minor Mana Oil",
            [itemKeys.npcDrops] = {121,122,127,171,213,422,424,426,428,429,430,431,432,434,435,437,440,441,442,446,449,485,505,531,539,547,548,565,568,598,615,819,948,1007,1008,1010,1011,1020,1021,1024,1025,1028,1040,1083,1111,1164,1165,1189,1192,1194,1205,1216,1393,1400,1417,1908,1939,1940,1973,2089,2157,2158,2165,2168,2170,2179,2180,2182,2183,2233,2235,2237,2244,2260,2261,2266,2267,2268,2332,2336,2338,2339,2349,2356,2360,2384,3239,3240,3241,3245,3252,3256,3257,3258,3260,3261,3263,3277,3278,3280,3282,3283,3286,3374,3375,3385,3393,3394,3416,3426,3435,3457,3459,3461,3466,3467,3667,3711,3712,3713,3717,3728,3730,3739,3743,3746,3748,3749,3762,3817,3819,3823,3917,3921,3944,3988,3989,3991,3999,4002,4003,4005,4007,4012,4013,4019,4020,4022,4025,4040,4051,4067,4070,4093,4127,4128,4129,4416,4463,4627,5053,6180,8886,9336,9523,10323,11912,11913,11917,11918,12856,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211829] = {
            [itemKeys.name] = "Waylaid Supplies: Small Bronze Bombs",
            [itemKeys.npcDrops] = {48,98,122,215,218,424,426,429,435,437,441,445,449,453,458,485,533,545,598,615,628,703,898,910,1012,1016,1018,1019,1021,1024,1025,1026,1027,1030,1031,1036,1040,1069,1164,1165,1185,1192,1197,1206,1259,1727,1888,1889,1908,1924,1940,1942,1954,1955,1956,1973,1974,2053,2102,2157,2158,2159,2168,2169,2170,2180,2181,2183,2208,2235,2244,2260,2264,2269,2270,2336,2338,2339,2351,2368,2373,2387,2428,3245,3249,3250,3257,3286,3375,3376,3378,3386,3395,3396,3452,3456,3459,3466,3467,3472,3475,3711,3713,3715,3725,3727,3728,3730,3733,3734,3737,3743,3745,3746,3749,3750,3765,3770,3772,3780,3781,3782,3783,3784,3797,3804,3807,3809,3810,3812,3815,3816,3817,3819,3820,3821,3823,3824,3825,3879,3900,3924,3925,3926,3928,3988,3989,3991,3999,4003,4004,4005,4006,4011,4012,4019,4020,4021,4026,4031,4032,4053,4057,4073,4094,4095,4110,4116,4126,4128,4273,4416,4463,4627,5053,6020,6071,6073,6115,6547,8886,10896,11680,11697,11858,11913,11918,12918,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211830] = {
            [itemKeys.name] = "Waylaid Supplies: Ornate Spyglasses",
            [itemKeys.npcDrops] = {48,98,122,127,203,206,218,424,426,429,430,434,435,440,441,446,449,450,485,511,517,533,598,615,628,787,889,898,910,1009,1012,1018,1019,1024,1025,1026,1027,1028,1032,1033,1042,1165,1167,1169,1189,1205,1251,1393,1400,1720,1754,1870,1914,1923,1924,1940,1942,1953,1955,1956,1974,2054,2071,2157,2165,2168,2169,2170,2171,2179,2181,2182,2183,2205,2206,2208,2236,2237,2261,2336,2338,2339,2368,2370,2372,2373,2374,2428,2449,2503,3247,3258,3283,3374,3375,3394,3396,3426,3436,3454,3456,3457,3458,3463,3466,3474,3475,3503,3711,3717,3725,3727,3730,3733,3737,3740,3743,3745,3748,3749,3750,3762,3765,3771,3774,3780,3781,3782,3783,3784,3797,3809,3810,3812,3814,3815,3816,3817,3818,3823,3824,3825,3879,3932,3943,3944,3988,3989,3991,3992,3998,3999,4003,4011,4012,4013,4014,4035,4036,4044,4057,4070,4074,4093,4094,4096,4097,4114,4116,4117,4126,4273,4416,4463,4627,5053,6020,6071,6073,6115,6606,7017,8886,9523,9524,11681,11697,11913,11915,11918,12856,12896,14270,210451,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211831] = {
            [itemKeys.name] = "Waylaid Supplies: Dark Leather Cloaks",
            [itemKeys.npcDrops] = {98,115,122,218,232,424,430,431,432,433,435,441,446,449,452,453,485,518,545,548,578,580,598,615,832,889,891,892,898,1012,1015,1016,1018,1019,1024,1025,1027,1030,1031,1033,1038,1040,1069,1083,1164,1206,1258,1353,1400,1417,1418,1727,1754,1909,1914,1915,1924,1942,1954,1955,1956,2071,2102,2103,2156,2157,2159,2165,2168,2169,2170,2180,2181,2183,2187,2206,2207,2208,2260,2269,2323,2336,2338,2339,2372,2450,3238,3239,3241,3250,3252,3263,3283,3286,3375,3392,3394,3416,3455,3458,3459,3467,3471,3475,3503,3711,3712,3715,3728,3732,3734,3739,3742,3743,3745,3749,3750,3758,3770,3772,3774,3780,3782,3784,3799,3806,3810,3812,3814,3816,3817,3819,3820,3823,3833,3834,3879,3921,3925,3926,3931,3943,3944,3988,3989,3991,3992,3993,3999,4001,4003,4004,4007,4008,4009,4019,4025,4035,4036,4057,4067,4070,4073,4094,4096,4099,4112,4128,4129,4264,4416,4463,4541,4627,5053,5835,6020,6071,6073,6115,6180,6547,8886,9524,10323,10896,11656,11680,11697,11918,11921,12897,14271,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211832] = {
            [itemKeys.name] = "Waylaid Supplies: Hillman's Shoulders",
            [itemKeys.npcDrops] = {98,122,215,300,424,426,431,434,435,441,449,450,453,458,485,545,548,565,568,578,580,598,889,891,892,898,1013,1018,1019,1024,1025,1026,1027,1030,1035,1040,1042,1043,1044,1057,1069,1083,1140,1167,1169,1259,1353,1400,1487,1727,1889,1913,1940,1953,1954,1956,1958,1974,2053,2071,2089,2103,2120,2157,2158,2159,2168,2170,2180,2181,2182,2183,2187,2205,2206,2208,2237,2264,2269,2270,2336,2338,2339,2351,2368,2372,2373,2387,2428,3239,3240,3241,3247,3261,3279,3282,3283,3286,3375,3396,3416,3424,3426,3445,3455,3457,3463,3467,3473,3474,3475,3664,3667,3711,3715,3717,3727,3730,3733,3737,3739,3743,3745,3746,3748,3749,3750,3763,3782,3784,3791,3802,3807,3809,3812,3817,3819,3824,3924,3925,3928,3986,3988,3991,3992,3999,4004,4005,4009,4012,4014,4021,4035,4036,4037,4057,4070,4073,4074,4094,4099,4264,4416,4463,4541,4627,5053,6071,6073,6115,6167,6547,6606,6788,8886,9524,10323,10373,10761,11656,11680,11697,11714,12856,12940,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211833] = {
            [itemKeys.name] = "Waylaid Supplies: Gray Woolen Shirts",
            [itemKeys.npcDrops] = {98,115,122,215,218,232,424,433,434,435,437,441,445,452,453,458,485,511,518,545,548,598,615,787,831,889,891,892,898,1012,1017,1018,1019,1024,1025,1027,1028,1030,1032,1033,1035,1036,1040,1044,1069,1169,1216,1222,1400,1755,1888,1908,1924,1940,1942,1943,1954,1955,1956,1973,1974,2054,2071,2102,2156,2157,2158,2160,2165,2168,2170,2171,2179,2180,2181,2206,2233,2236,2237,2244,2270,2323,2336,2338,2339,2368,2370,2372,2373,2403,2404,2449,2503,3240,3248,3249,3279,3283,3286,3375,3377,3395,3396,3424,3426,3445,3454,3455,3466,3467,3471,3474,3503,3667,3711,3712,3717,3721,3725,3727,3728,3730,3733,3737,3743,3745,3748,3750,3771,3772,3774,3780,3782,3783,3791,3797,3806,3809,3810,3812,3814,3816,3817,3819,3823,3825,3879,3893,3926,3943,3944,3986,3988,3989,3992,3993,4002,4004,4006,4009,4012,4016,4026,4028,4044,4057,4074,4094,4113,4117,4128,4202,4416,4463,4627,5053,6071,6073,6115,6180,6547,6788,8886,10617,11858,11917,11918,12677,12759,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211834] = {
            [itemKeys.name] = "Waylaid Supplies: Pearl-clasped Cloaks",
            [itemKeys.npcDrops] = {125,154,218,424,429,430,432,435,437,440,441,445,446,449,517,580,598,615,703,889,891,898,909,1012,1018,1019,1028,1032,1040,1042,1044,1164,1169,1194,1216,1222,1259,1400,1418,1727,1754,1755,1889,1908,1913,1915,1940,1943,1953,1954,1955,1956,1973,1974,2054,2102,2103,2120,2157,2158,2160,2168,2170,2171,2180,2181,2182,2206,2207,2208,2264,2269,2322,2335,2336,2337,2338,2339,2368,2370,2372,2373,2427,2428,2451,3238,3239,3249,3258,3260,3279,3283,3286,3374,3386,3396,3416,3434,3435,3455,3457,3458,3473,3475,3711,3713,3725,3727,3728,3730,3732,3733,3734,3739,3745,3746,3748,3749,3750,3758,3772,3774,3780,3781,3782,3783,3784,3789,3802,3804,3809,3810,3812,3814,3816,3817,3819,3823,3893,3919,3924,3928,3987,3988,3991,3992,3993,3999,4002,4004,4035,4051,4053,4070,4073,4093,4095,4097,4111,4117,4273,4416,4463,4969,5053,5683,6071,6073,6115,6180,6547,6606,6668,8886,9523,9524,10373,10761,11697,11714,11858,11912,11915,11918,11921,12940,209797,210697,214129},
            [itemKeys.objectDrops] = {2849,2850,3714,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211835] = {
            [itemKeys.name] = "Waylaid Supplies: Smoked Sagefish",
            [itemKeys.npcDrops] = {98,115,122,124,206,217,218,232,424,426,429,430,431,434,435,440,441,453,518,548,578,598,711,785,787,889,891,898,1016,1018,1019,1024,1026,1027,1028,1030,1032,1033,1035,1039,1043,1044,1069,1140,1165,1205,1207,1251,1259,1353,1720,1727,1909,1915,1939,1940,1942,1943,1954,1955,1956,1957,1973,2071,2089,2102,2156,2158,2165,2168,2169,2170,2171,2181,2206,2207,2208,2233,2237,2244,2270,2323,2336,2337,2338,2339,2368,2370,2372,2373,2387,2449,3235,3236,3241,3247,3249,3275,3278,3282,3378,3386,3393,3394,3396,3436,3457,3463,3466,3467,3471,3472,3473,3503,3662,3721,3725,3727,3728,3730,3732,3734,3737,3743,3745,3748,3749,3750,3758,3772,3782,3783,3784,3789,3797,3804,3806,3809,3810,3811,3812,3816,3817,3818,3819,3823,3879,3922,3928,3931,3944,3988,3989,3991,3992,3993,3999,4003,4011,4014,4015,4025,4028,4035,4044,4053,4073,4074,4095,4096,4116,4117,4126,4416,4463,5053,6071,6073,6115,6167,6547,8886,9336,9523,9524,11697,11858,11915,12678,12940,214129},
            [itemKeys.objectDrops] = {2849,2850,3714,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211836] = {
            [itemKeys.name] = "Waylaid Supplies: Smoked Bear Meat",
            [itemKeys.npcDrops] = {98,115,171,218,429,430,431,433,435,440,441,446,453,458,485,533,565,598,615,785,832,889,892,909,910,1008,1009,1015,1016,1017,1018,1019,1024,1025,1030,1033,1036,1040,1043,1069,1165,1185,1197,1206,1216,1418,1720,1727,1909,1940,1943,1953,1954,1955,1958,2054,2071,2102,2103,2156,2158,2168,2169,2180,2181,2182,2183,2187,2205,2206,2233,2235,2236,2237,2260,2261,2265,2269,2336,2338,2339,2368,2370,2372,2373,2404,2428,2450,2503,3241,3248,3250,3252,3256,3257,3280,3374,3375,3386,3392,3395,3445,3454,3455,3463,3466,3467,3471,3472,3474,3662,3667,3711,3712,3715,3717,3725,3727,3728,3730,3732,3733,3734,3737,3739,3740,3743,3745,3746,3748,3750,3772,3774,3781,3782,3783,3784,3809,3810,3816,3817,3819,3823,3879,3922,3924,3944,3988,3989,3991,3993,4004,4013,4015,4022,4028,4034,4037,4051,4070,4073,4074,4096,4112,4113,4129,4416,4463,4627,5053,6071,6073,6115,6180,6547,6606,8886,9456,9524,11714,11858,11912,11917,12856,12896,12940,210451,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211837] = {
            [itemKeys.name] = "Waylaid Supplies: Goblin Deviled Clams",
            [itemKeys.npcDrops] = {48,98,122,205,206,213,215,218,422,423,424,426,429,431,434,435,437,440,441,446,449,452,453,485,517,569,578,579,598,615,785,889,891,892,910,1009,1011,1015,1018,1019,1024,1027,1028,1036,1038,1039,1040,1044,1069,1169,1185,1192,1216,1222,1720,1727,1754,1889,1913,1915,1939,1940,1942,1954,1955,1956,1973,2054,2071,2156,2157,2158,2168,2169,2170,2179,2180,2181,2182,2183,2206,2207,2260,2270,2323,2336,2338,2339,2370,2372,2373,2428,3241,3249,3283,3394,3396,3414,3416,3426,3434,3445,3454,3455,3457,3459,3466,3472,3475,3503,3662,3711,3721,3725,3727,3728,3732,3734,3737,3739,3743,3745,3749,3750,3752,3772,3782,3783,3784,3802,3808,3809,3810,3816,3817,3818,3819,3823,3825,3893,3900,3919,3923,3924,3925,3932,3944,3988,3989,3991,3998,3999,4002,4003,4004,4011,4013,4015,4028,4031,4074,4093,4094,4096,4097,4114,4129,4264,4416,4463,4627,5053,6071,6073,6115,6180,6547,6606,7015,8886,9456,9524,11697,11858,11912,11917,12856,12860,185333,211965,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211838] = {
            [itemKeys.name] = "Waylaid Supplies: Heavy Wool Bandages",
            [itemKeys.npcDrops] = {98,232,300,424,426,429,433,441,445,446,449,452,453,458,485,545,578,579,580,598,615,703,785,819,824,889,891,892,1015,1016,1017,1018,1019,1024,1025,1027,1031,1032,1038,1044,1069,1164,1165,1167,1197,1205,1216,1222,1258,1393,1400,1487,1727,1754,1888,1889,1913,1915,1939,1940,1942,1954,1957,1974,2071,2089,2102,2103,2156,2157,2165,2169,2170,2171,2180,2181,2182,2207,2208,2237,2264,2265,2323,2336,2338,2339,2360,2368,2370,2372,2373,2387,2403,2404,2427,2450,3248,3257,3260,3283,3286,3378,3392,3416,3434,3436,3456,3457,3463,3466,3467,3475,3667,3713,3717,3721,3727,3728,3732,3734,3737,3743,3745,3746,3749,3750,3772,3774,3781,3782,3783,3784,3799,3804,3809,3810,3816,3817,3818,3819,3820,3823,3834,3879,3893,3924,3926,3932,3940,3988,3989,3991,3999,4004,4012,4013,4014,4019,4021,4031,4032,4057,4073,4097,4117,4128,4416,4463,4627,5053,5682,6020,6071,6073,6115,6180,6547,6606,8886,9336,9524,10323,10373,11697,11915,11917,210697,211965,212837,214129},
            [itemKeys.objectDrops] = {2849,2850,3714,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211839] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211840] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211841] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211842] = {
            [itemKeys.name] = "Rakkamar's Tattered Thinking Cap",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211843] = {
            [itemKeys.name] = "Mask of Scorn",
            [itemKeys.npcDrops] = {204068},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211845] = {
            [itemKeys.name] = "Blackfathom Sharpening Stone",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211846] = {
            [itemKeys.name] = "Plans: Blackfathom Sharpening Stone",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {415614},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211848] = {
            [itemKeys.name] = "Blackfathom Mana Oil",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211849] = {
            [itemKeys.name] = "Formula: Blackfathom Mana Oil",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {415614},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211852] = {
            [itemKeys.name] = "Handwraps of Befouled Water",
            [itemKeys.npcDrops] = {202699},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211853] = {
            [itemKeys.name] = "Scroll: VOCE WELL",
            [itemKeys.npcDrops] = {98,115,121,127,171,215,232,345,422,424,428,429,431,432,435,437,440,441,445,446,452,505,517,531,545,548,565,568,578,579,580,598,615,785,787,819,832,889,910,1007,1008,1012,1013,1018,1019,1022,1023,1024,1025,1026,1027,1040,1069,1083,1110,1165,1167,1169,1185,1197,1216,1353,1393,1400,1754,1914,1939,1940,1942,1943,1953,1954,1956,1958,1973,2054,2071,2089,2102,2103,2158,2159,2165,2168,2169,2170,2179,2181,2206,2207,2237,2260,2261,2265,2267,2269,2323,2332,2338,2339,2372,2427,3239,3247,3250,3252,3261,3275,3279,3283,3374,3375,3376,3395,3416,3452,3457,3459,3466,3475,3476,3503,3713,3715,3721,3725,3728,3730,3733,3734,3735,3739,3740,3743,3745,3746,3749,3750,3759,3772,3774,3780,3781,3783,3784,3809,3816,3820,3823,3825,3879,3923,3924,3926,3928,3944,3988,3989,3991,3992,3998,4003,4005,4007,4011,4012,4013,4020,4032,4053,4093,4124,4416,4463,4627,6020,6071,6115,6180,8886,9524,10758,10759,11697,11858,11912,12896,12897,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211854] = {
            [itemKeys.name] = "Scroll: OMIT KESA",
            [itemKeys.npcDrops] = {98,115,122,125,127,203,205,424,428,431,433,435,440,441,445,449,452,505,517,531,578,579,598,615,628,819,832,889,891,892,898,920,949,1007,1008,1009,1012,1018,1019,1022,1023,1024,1025,1028,1029,1031,1032,1035,1036,1039,1040,1069,1165,1167,1169,1185,1216,1258,1393,1666,1908,1909,1913,1914,1939,1940,1942,1953,1954,1955,1956,1973,1974,2071,2102,2103,2156,2157,2160,2169,2181,2182,2183,2206,2207,2233,2235,2236,2244,2260,2265,2267,2336,2337,2338,2339,2356,2368,2372,2373,2387,2404,2427,3237,3247,3251,3256,3260,3263,3275,3278,3286,3374,3375,3376,3395,3424,3434,3436,3452,3455,3456,3457,3458,3459,3466,3467,3473,3711,3713,3725,3727,3728,3730,3734,3737,3739,3740,3742,3743,3746,3771,3780,3781,3782,3784,3804,3808,3816,3818,3819,3824,3825,3834,3893,3924,3926,3928,3944,3986,3988,3991,3999,4002,4004,4014,4020,4025,4038,4044,4051,4057,4093,4094,4095,4099,4116,4248,4416,4627,6071,6073,6115,6547,6788,8886,9523,9524,11913,12856,12896,12897,12940,14270,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211855] = {
            [itemKeys.name] = "Scroll: STHENIC LUNATE",
            [itemKeys.npcDrops] = {98,127,215,422,423,424,426,430,432,435,440,441,449,452,453,485,517,531,548,569,578,580,598,831,892,1007,1008,1010,1012,1018,1019,1023,1024,1027,1028,1030,1031,1035,1040,1069,1083,1157,1158,1167,1169,1185,1192,1216,1258,1400,1417,1663,1888,1913,1939,1940,1942,1943,1953,1956,1973,1974,2071,2102,2103,2157,2160,2165,2170,2171,2182,2206,2207,2208,2233,2236,2237,2261,2267,2269,2322,2323,2336,2337,2338,2339,2360,2370,2373,2404,2451,3237,3239,3247,3250,3252,3263,3279,3283,3286,3375,3376,3378,3394,3395,3416,3445,3457,3458,3459,3463,3466,3467,3473,3475,3711,3715,3717,3725,3727,3730,3732,3733,3734,3737,3739,3742,3746,3774,3780,3781,3782,3783,3784,3789,3797,3809,3812,3814,3817,3819,3820,3823,3825,3898,3900,3921,3928,3986,3987,3988,3989,3991,3992,3999,4002,4004,4021,4031,4042,4051,4073,4074,4094,4097,4116,4128,4264,4416,4463,4627,5053,5848,6020,6071,6073,6115,6180,6547,6606,6788,8886,9524,11917,11921,12897,210451,214129},
            [itemKeys.objectDrops] = {2849,2850,3714,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211856] = {
            [itemKeys.name] = "Resilient Mail Coif",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753,14754},
            [itemKeys.startQuest] = nil,
        },
        [211857] = {
            [itemKeys.name] = "Resilient Leather Mask",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753,14754},
            [itemKeys.startQuest] = nil,
        },
        [211933] = {
            [itemKeys.name] = "Waylaid Supplies: Rough Stone",
            [itemKeys.npcDrops] = {40,43,46,79,95,97,116,117,118,126,157,285,330,390,456,476,502,504,524,589,590,712,732,735,830,833,846,880,881,1115,1117,1122,1124,1127,1130,1135,1161,1163,1164,1166,1172,1174,1175,1176,1177,1186,1193,1201,1202,1211,1236,1426,1534,1535,1536,1543,1545,1548,1553,1660,1662,1674,1675,1693,1765,1766,1767,1768,1769,1770,1772,1778,1779,1782,1797,1865,1866,1869,1924,1936,1953,1971,1983,1993,1996,1999,2001,2004,2005,2009,2010,2011,2012,2017,2018,2019,2020,2021,2025,2027,2029,2030,2033,2034,2039,2043,2069,2070,2149,2152,2163,2167,2176,2177,2178,2179,2185,2189,2190,2202,2205,2212,2231,2232,2234,2321,2322,2324,2950,2951,2957,2959,2960,2962,2963,2964,2973,2974,2976,2978,2979,2989,2990,3035,3103,3107,3110,3111,3112,3114,3115,3118,3119,3126,3131,3196,3197,3198,3199,3206,3207,3225,3232,3242,3243,3246,3266,3267,3268,3269,3273,3274,3284,3285,3379,3381,3382,3384,3397,3415,3438,4316,6093,6123,6866,6927,7234,7235,7318,9523,10158,10160,11911,11913,14428,14429,202060,202116,207515,208071},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211934] = {
            [itemKeys.name] = "Waylaid Supplies: Healing Potions",
            [itemKeys.npcDrops] = {36,43,46,95,97,114,116,118,119,123,126,285,330,390,456,478,481,500,502,515,550,589,732,735,830,834,880,1115,1117,1121,1123,1124,1126,1127,1131,1135,1161,1162,1163,1166,1174,1177,1186,1188,1193,1195,1196,1197,1201,1236,1380,1426,1520,1522,1523,1527,1528,1532,1536,1537,1540,1543,1545,1548,1553,1554,1655,1657,1658,1660,1662,1664,1674,1675,1693,1753,1765,1766,1767,1768,1769,1770,1772,1773,1778,1779,1780,1782,1797,1866,1868,1869,1912,1953,1999,2001,2005,2008,2009,2010,2011,2015,2017,2027,2029,2033,2039,2043,2067,2070,2149,2152,2164,2167,2177,2178,2179,2185,2189,2190,2201,2203,2232,2234,2321,2324,2950,2951,2959,2960,2962,2963,2964,2965,2970,2972,2976,2977,2978,2979,2989,3035,3051,3107,3111,3112,3113,3114,3116,3117,3119,3121,3123,3127,3192,3195,3197,3198,3203,3206,3207,3232,3234,3246,3254,3255,3265,3266,3267,3268,3269,3273,3275,3284,3285,3397,3425,3438,4316,6093,6123,6466,6494,6789,7234,7318,7319,9523,10157,10158,10160,11910,11911,11912,11913,14428,202060},
            [itemKeys.objectDrops] = {106318,106319,404352},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211935] = {
            [itemKeys.name] = "Waylaid Supplies: Elixir of Firepower",
            [itemKeys.npcDrops] = {98,121,122,154,345,424,431,432,434,435,441,445,446,449,452,453,485,517,598,628,889,898,1007,1009,1011,1018,1019,1025,1026,1027,1031,1032,1033,1035,1036,1039,1040,1043,1069,1083,1160,1165,1167,1189,1216,1259,1393,1400,1418,1727,1908,1909,1913,1920,1939,1942,1943,1956,1958,1973,1974,2054,2102,2156,2157,2165,2169,2170,2171,2181,2182,2183,2208,2236,2237,2244,2269,2270,2323,2336,2338,2339,2370,2372,2373,2428,2449,3239,3250,3252,3256,3257,3258,3286,3374,3375,3393,3396,3416,3426,3434,3454,3456,3458,3459,3466,3667,3711,3713,3715,3721,3725,3727,3733,3739,3743,3745,3748,3750,3772,3780,3781,3782,3783,3784,3791,3797,3806,3809,3816,3817,3818,3820,3823,3825,3834,3899,3919,3923,3926,3928,3932,3944,3988,3989,3991,3993,4003,4004,4005,4006,4009,4011,4012,4014,4020,4021,4024,4036,4037,4038,4042,4051,4053,4057,4070,4094,4099,4113,4117,4202,4416,4463,5053,6020,6071,6073,6115,6547,6788,8886,9523,9524,10373,10760,10761,11682,11697,11858,12759,12896,210107,214129},
            [itemKeys.objectDrops] = {2849,2850,3715,111095},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211940] = {
            [itemKeys.name] = "Ecks'av's Tribal Guardian",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211941] = {
            [itemKeys.name] = "Windwalker's Yari",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211944] = {
            [itemKeys.name] = "Tojara's Karma",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211953] = {
            [itemKeys.name] = "Scroll of Arcane Recovery I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211954] = {
            [itemKeys.name] = "Scroll of Arcane Accuracy I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211955] = {
            [itemKeys.name] = "Scroll of Arcane Protection - Fire I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211956] = {
            [itemKeys.name] = "Scroll of Arcane Protection - Frost I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [211957] = {
            [itemKeys.name] = "Scroll of Arcane Power I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212160] = {
            [itemKeys.name] = "Chronoboon Displacer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {958,983,1257,1275,1307,1308,1351,1463,1673,2805,3323,3335,3351,3490,3500,3542,3562,3700,3970,4083,4220,4562,4575,4878,5110,5151,8361,13476,14739,15175},
            [itemKeys.startQuest] = nil,
        },
        [212225] = {
            [itemKeys.name] = "Neverending Soul Vessel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212230] = {
            [itemKeys.name] = "Schematic: Soul Vessel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [212347] = {
            [itemKeys.name] = "Illari's Key",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423569},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212548] = {
            [itemKeys.name] = "Rune of Eclipse",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212549] = {
            [itemKeys.name] = "Rune of the Trapper",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212551] = {
            [itemKeys.name] = "Rune of Warfare",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212552] = {
            [itemKeys.name] = "Psychosophic Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212553] = {
            [itemKeys.name] = "Jewel-Encrusted Box",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212559] = {
            [itemKeys.name] = "Rune of the Poisoned Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212560] = {
            [itemKeys.name] = "Rune of Ancestral Awakening",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212561] = {
            [itemKeys.name] = "Rune of Burning Darkness",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212562] = {
            [itemKeys.name] = "Rune of Healing Rage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {212553},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212580] = {
            [itemKeys.name] = "Lorekeeper's Staff",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753},
            [itemKeys.startQuest] = nil,
        },
        [212581] = {
            [itemKeys.name] = "Outrunner's Bow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753},
            [itemKeys.startQuest] = nil,
        },
        [212582] = {
            [itemKeys.name] = "Protector's Sword",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753},
            [itemKeys.startQuest] = nil,
        },
        [212583] = {
            [itemKeys.name] = "Sentinel's Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14753},
            [itemKeys.startQuest] = nil,
        },
        [212584] = {
            [itemKeys.name] = "Advisor's Gnarled Staff",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14754},
            [itemKeys.startQuest] = nil,
        },
        [212585] = {
            [itemKeys.name] = "Outrider's Bow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14754},
            [itemKeys.startQuest] = nil,
        },
        [212586] = {
            [itemKeys.name] = "Legionnaire's Sword",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14754},
            [itemKeys.startQuest] = nil,
        },
        [212587] = {
            [itemKeys.name] = "Scout's Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {14754},
            [itemKeys.startQuest] = nil,
        },
        [212588] = {
            [itemKeys.name] = "Provisioner's Gloves",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [212589] = {
            [itemKeys.name] = "Courier Treads",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [212590] = {
            [itemKeys.name] = "Hoist Strap",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214096,214098,214099,214101},
            [itemKeys.startQuest] = nil,
        },
        [212693] = {
            [itemKeys.name] = "Rotten Seed",
            [itemKeys.npcDrops] = {216310},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79348,
        },
        [212723] = {
            [itemKeys.name] = "Rumbling Essence",
            [itemKeys.npcDrops] = {92,2735,2736},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212724] = {
            [itemKeys.name] = "Whirling Essence",
            [itemKeys.npcDrops] = {11576,11577,11578},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212726] = {
            [itemKeys.name] = "Rushing Essence",
            [itemKeys.npcDrops] = {4382,4385,4386,4387},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212748] = {
            [itemKeys.name] = "Tattered Note",
            [itemKeys.npcDrops] = {4421},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79358,
        },
        [212792] = {
            [itemKeys.name] = "Eye of the Tempest",
            [itemKeys.npcDrops] = {216474},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212982] = {
            [itemKeys.name] = "Squall-breakers Potion",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212994] = {
            [itemKeys.name] = "Stormchaser's Bindings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212996] = {
            [itemKeys.name] = "Lightningcaller's Bindings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [212997] = {
            [itemKeys.name] = "Tidecaller's Bindings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213036] = {
            [itemKeys.name] = "Water of Elune'ara",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213074] = {
            [itemKeys.name] = "Rune of Nourishing",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213086] = {
            [itemKeys.name] = "Rune of the Storm",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213087] = {
            [itemKeys.name] = "Sergeant's Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [213088] = {
            [itemKeys.name] = "Sergeant's Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [213092] = {
            [itemKeys.name] = "Rune of the Alpha",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {421568},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213093] = {
            [itemKeys.name] = "Rune of Power",
            [itemKeys.npcDrops] = {217302},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213094] = {
            [itemKeys.name] = "Rune of Fire Nova",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213096] = {
            [itemKeys.name] = "Rune of Decoys",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213098] = {
            [itemKeys.name] = "Rune of Invocation",
            [itemKeys.npcDrops] = {217605},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213101] = {
            [itemKeys.name] = "Rune of Shadowflames",
            [itemKeys.npcDrops] = {217580},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213102] = {
            [itemKeys.name] = "Rune of Wickedness",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213103] = {
            [itemKeys.name] = "Rune of Blood Surge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213104] = {
            [itemKeys.name] = "Rune of Ruthless Precision",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213105] = {
            [itemKeys.name] = "Shawarmageddon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213109] = {
            [itemKeys.name] = "Rune of Focused Rage",
            [itemKeys.npcDrops] = {218032},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213110] = {
            [itemKeys.name] = "Rune of the Commander",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424110},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213111] = {
            [itemKeys.name] = "Rune of Intervention",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423920},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213113] = {
            [itemKeys.name] = "Spell Notes: Hot Streak",
            [itemKeys.npcDrops] = {2760,217305},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213116] = {
            [itemKeys.name] = "Spell Notes: Chronostatic Preservation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213118] = {
            [itemKeys.name] = "Rune of the Jungle King",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213119] = {
            [itemKeys.name] = "Rune of Instinct",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213120] = {
            [itemKeys.name] = "Rune of the Dreamer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {404402},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213121] = {
            [itemKeys.name] = "Preserved Holly",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213443},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213122] = {
            [itemKeys.name] = "Rune of Steady Shot",
            [itemKeys.npcDrops] = {217836},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213124] = {
            [itemKeys.name] = "Rune of Close Combat",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213125] = {
            [itemKeys.name] = "Rune of Invigoration",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213126] = {
            [itemKeys.name] = "Rune of the Scrapper",
            [itemKeys.npcDrops] = {217783},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213127] = {
            [itemKeys.name] = "Spell Notes: PELFRB STOLLOTS",
            [itemKeys.npcDrops] = {780},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213128] = {
            [itemKeys.name] = "Rune of Piety",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213130] = {
            [itemKeys.name] = "Rune of Infusions",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213132] = {
            [itemKeys.name] = "Rune of the Guardian",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213136] = {
            [itemKeys.name] = "Rune of Subtlety",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {216618},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213137] = {
            [itemKeys.name] = "Rune of the Assailant",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213138] = {
            [itemKeys.name] = "Rune of the Southpaw",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215453},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213139] = {
            [itemKeys.name] = "Rune of the Assassin",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {428228},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213140] = {
            [itemKeys.name] = "Prophecy of the Quickened Path",
            [itemKeys.npcDrops] = {2252,2253,10758,10759,10760,10761},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213142] = {
            [itemKeys.name] = "Prophecy of Imprisoned Malice",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {428144},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213143] = {
            [itemKeys.name] = "Apocryphal Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424373},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213144] = {
            [itemKeys.name] = "Luminous Epiphany",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213165] = {
            [itemKeys.name] = "Basilisks: Should Petrification be Feared?",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {421526},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213168] = {
            [itemKeys.name] = "Copper Blood Coin",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {216971},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213169] = {
            [itemKeys.name] = "Silver Blood Coin",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213170] = {
            [itemKeys.name] = "Gold Blood Coin",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213278] = {
            [itemKeys.name] = "Bonk-Maestro's Handguards",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213279] = {
            [itemKeys.name] = "Reflective Skullcap",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213280] = {
            [itemKeys.name] = "Marksman's Scopevisor",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213281] = {
            [itemKeys.name] = "Electromagnetic Hyperflux Reactivator",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213282] = {
            [itemKeys.name] = "Hypercharged Gear of Innovation",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213283] = {
            [itemKeys.name] = "Hypercharged Gear of Conflagration",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213284] = {
            [itemKeys.name] = "Hypercharged Gear of Devastation",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213285] = {
            [itemKeys.name] = "Lev's Oil-Stained Bindings",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213286] = {
            [itemKeys.name] = "Electrocutioner's Needle",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213287] = {
            [itemKeys.name] = "Electrocutioner Hexnut",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213288] = {
            [itemKeys.name] = "Grubbis Grubby Gauntlets",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213289] = {
            [itemKeys.name] = "Hydrostaff",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213290] = {
            [itemKeys.name] = "Acidic Waders",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213291] = {
            [itemKeys.name] = "Toxic Revenger II",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213292] = {
            [itemKeys.name] = "Gizmotron Gigachopper",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213293] = {
            [itemKeys.name] = "Hi-tech Supergun Mk.VII",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213294] = {
            [itemKeys.name] = "Caverndeep Sabatons",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213295] = {
            [itemKeys.name] = "Ultrasonic Vibroblade",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213296] = {
            [itemKeys.name] = "Supercharged Headchopper",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213297] = {
            [itemKeys.name] = "Oscillating Blasthammer",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213298] = {
            [itemKeys.name] = "Mechbuilder's Overalls",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213299] = {
            [itemKeys.name] = "Petrolspill Pants",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213300] = {
            [itemKeys.name] = "Fighter Ace Gloves",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213301] = {
            [itemKeys.name] = "Synthetic Mantle",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213302] = {
            [itemKeys.name] = "Mantle of the Cunning Negotiator",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213303] = {
            [itemKeys.name] = "Lightning Rod Spaulders",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213304] = {
            [itemKeys.name] = "Troggslayer Pauldrons",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213305] = {
            [itemKeys.name] = "Machined Alloy Shoulderplates",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213306] = {
            [itemKeys.name] = "Ingenuity's Cover",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213307] = {
            [itemKeys.name] = "Drape of Dismantling",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213308] = {
            [itemKeys.name] = "Prototype Parachute Cloak",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213309] = {
            [itemKeys.name] = "Cloak of Invention",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213310] = {
            [itemKeys.name] = "Hyperconductive Shimmershirt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213311] = {
            [itemKeys.name] = "Hyperconductive Robe",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213312] = {
            [itemKeys.name] = "Insulated Apron",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213313] = {
            [itemKeys.name] = "Insulated Chestguard",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213314] = {
            [itemKeys.name] = "Electromantic Chainmail",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213315] = {
            [itemKeys.name] = "Electromantic Chainshirt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213316] = {
            [itemKeys.name] = "H.A.Z.A.R.D. Breastplate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213317] = {
            [itemKeys.name] = "Experimental Aim Stabilizers",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213318] = {
            [itemKeys.name] = "Ornate Dark Iron Bangles",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213319] = {
            [itemKeys.name] = "Machinist's Gloves",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213320] = {
            [itemKeys.name] = "Fingers of Arcane Accuracy",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213321] = {
            [itemKeys.name] = "Volatile Concoction Belt",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213322] = {
            [itemKeys.name] = "Skullduggery Waistband",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213323] = {
            [itemKeys.name] = "Cord of Deep Earth",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213324] = {
            [itemKeys.name] = "Electromagnetic Waistcord",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213325] = {
            [itemKeys.name] = "Darkvision Girdle",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213326] = {
            [itemKeys.name] = "Girdle of Reclamation",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213327] = {
            [itemKeys.name] = "Belt of the Trogg Berserker",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213328] = {
            [itemKeys.name] = "Hyperconductive Pantaloons",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213329] = {
            [itemKeys.name] = "Hyperconductive Skirt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213330] = {
            [itemKeys.name] = "H.A.Z.A.R.D. Legplates",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213331] = {
            [itemKeys.name] = "Insulated Leggings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213332] = {
            [itemKeys.name] = "Insulated Legguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213333] = {
            [itemKeys.name] = "Electromantic Chausses",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213334] = {
            [itemKeys.name] = "Electromantic Gambeson",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213335] = {
            [itemKeys.name] = "H.A.Z.A.R.D. Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213336] = {
            [itemKeys.name] = "Hyperconductive Walkers",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213337] = {
            [itemKeys.name] = "Hyperconductive Sandals",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213338] = {
            [itemKeys.name] = "Electromantic Grounding Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213339] = {
            [itemKeys.name] = "Electromantic Grounding Sabatons",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213340] = {
            [itemKeys.name] = "Gnomebot Operators Boots",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213341] = {
            [itemKeys.name] = "Insulated Workboots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213342] = {
            [itemKeys.name] = "Insulated Galoshes",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [213343] = {
            [itemKeys.name] = "Justice Badge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213344] = {
            [itemKeys.name] = "Gnomeregan Peace Officer's Torque",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213345] = {
            [itemKeys.name] = "Piston Pendant",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213346] = {
            [itemKeys.name] = "Pendant of Homecoming",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213347] = {
            [itemKeys.name] = "Miniaturized Combustion Chamber",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213348] = {
            [itemKeys.name] = "Gyromatic Experiment 420b",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213349] = {
            [itemKeys.name] = "Gniodine Pill Bottle",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213350] = {
            [itemKeys.name] = "Wirdal's Hardened Core",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213351] = {
            [itemKeys.name] = "Irradiated Tower Shield",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213352] = {
            [itemKeys.name] = "Gear-Mender's Grace",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213353] = {
            [itemKeys.name] = "Defibrillating Staff",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213354] = {
            [itemKeys.name] = "Staff of the Evil Genius",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213355] = {
            [itemKeys.name] = "Falco's Sting",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213356] = {
            [itemKeys.name] = "Thermaplugg's Custom Blaster",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213369] = {
            [itemKeys.name] = "Faintly Glowing Leather",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213370] = {
            [itemKeys.name] = "Irradiated Leather Scraps",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213371] = {
            [itemKeys.name] = "Crate of Tainted Gniodine Solution",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213372] = {
            [itemKeys.name] = "Insulating Gniodine",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213373] = {
            [itemKeys.name] = "Reflective Scrapmetal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213376] = {
            [itemKeys.name] = "Low-Background Truesilver Plates",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213378] = {
            [itemKeys.name] = "Unstable Microfilament",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213379] = {
            [itemKeys.name] = "Hyperconductive Arcano-Filament",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213381] = {
            [itemKeys.name] = "Pile of Tarnished Gears",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213383] = {
            [itemKeys.name] = "Polished Truesilver Gears",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213390] = {
            [itemKeys.name] = "Whirling Truesilver Gearwall",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213391] = {
            [itemKeys.name] = "UNUSED Electrified Mithril Gauntlets",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213408] = {
            [itemKeys.name] = "Gyromatic Macro-Adjustor",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213409] = {
            [itemKeys.name] = "Mekkatorque's Arcano-Shredder",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213410] = {
            [itemKeys.name] = "Glimmering Gizmoblade",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213411] = {
            [itemKeys.name] = "Izzleflick's Inextinguishable Igniter",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213412] = {
            [itemKeys.name] = "Dielectric Safety Shield",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213413] = {
            [itemKeys.name] = "Generously Padded Shoulderpads",
            [itemKeys.npcDrops] = {220007},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213414] = {
            [itemKeys.name] = "Mech-Mender's Sash",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213415] = {
            [itemKeys.name] = "Tinker's Wrist Wraps",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213416] = {
            [itemKeys.name] = "Thermaplugg's Rocket Cleaver",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213417] = {
            [itemKeys.name] = "Truesilver Filament Coif",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213418] = {
            [itemKeys.name] = "Welded Truesilver Ringlets",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213419] = {
            [itemKeys.name] = "9-60 Repair Manual",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213421] = {
            [itemKeys.name] = "Singed Note",
            [itemKeys.npcDrops] = {2318,2319},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213422] = {
            [itemKeys.name] = "Illegible Recipe",
            [itemKeys.npcDrops] = {2254,2255,2256,2287,2569,2570,2571},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79624,
        },
        [213426] = {
            [itemKeys.name] = "Ogre Lightning Rod",
            [itemKeys.npcDrops] = {2562,2564,2566},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213427] = {
            [itemKeys.name] = "Grime-Encrusted Salvage",
            [itemKeys.npcDrops] = {6206,6207,6211,6220,6226,6227,6228,6230,6232,6233,6234,6329,215728,217280,218242,218243,218244,218245,218537,220007,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213442] = {
            [itemKeys.name] = "Cogmaster's Claw",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213443] = {
            [itemKeys.name] = "Ticking Present",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213444] = {
            [itemKeys.name] = "Tarnished Prayer Bead I",
            [itemKeys.npcDrops] = {2553,2554,2555,2556,2557,2562,2564,2566,2572,2573,2574,2575,2586,2587,2588,2589,2590,2595,2618,2619,4062},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213445] = {
            [itemKeys.name] = "Tarnished Prayer Bead II",
            [itemKeys.npcDrops] = {2552,2553,2554,2555,2556,2557,2562,2564,2566,2567,2573,2574,2575,2577,2586,2587,2588,2589,2591,2618,2619,4062},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213446] = {
            [itemKeys.name] = "Tarnished Prayer Bead III",
            [itemKeys.npcDrops] = {2552,2554,2555,2556,2557,2562,2564,2572,2573,2574,2575,2586,2587,2588,2589,2590,2618,2619,4062},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213447] = {
            [itemKeys.name] = "Rosary of the Light",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213448] = {
            [itemKeys.name] = "Divine Prayer Bead I",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213449] = {
            [itemKeys.name] = "Divine Prayer Bead II",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213450] = {
            [itemKeys.name] = "Divine Prayer Bead III",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213451] = {
            [itemKeys.name] = "Corrupted Fire Totem",
            [itemKeys.npcDrops] = {217392},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213452] = {
            [itemKeys.name] = "Dormant Holy Rune",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423841},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213513] = {
            [itemKeys.name] = "Libram of Deliverance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423695},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213526] = {
            [itemKeys.name] = "Hybrid Haunch",
            [itemKeys.npcDrops] = {2347,2473,2474},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213527] = {
            [itemKeys.name] = "Smuggler's Spice Blend",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {422911},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213528] = {
            [itemKeys.name] = "Balmy Brew",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {422919},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213529] = {
            [itemKeys.name] = "Viscous Venom",
            [itemKeys.npcDrops] = {769},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213542] = {
            [itemKeys.name] = "The Necro-Gnomicon",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213543] = {
            [itemKeys.name] = "Scroll: UPDOG",
            [itemKeys.npcDrops] = {205,206,210,212,431,595,604,660,667,670,682,686,736,757,759,766,780,785,787,920,938,940,1034,1035,1069,1085,1094,1096,1110,1150,1158,1270,1487,2240,2241,2246,2251,2271,2368,2369,2375,2376,2377,2406,2408,2503,2555,2556,2557,2559,2560,2581,2587,2725,2728,2729,2761,3755,3757,3759,3771,3789,3804,3808,3833,4017,4041,4095,4097,4101,4107,4109,4111,4120,4124,4139,4140,4142,4143,4147,4151,4308,4355,4360,4376,4396,4541,4548,4632,4642,4645,4651,4666,4670,4692,4702,4715,4726,4834,5237,5333,5336,6219,6224,6575,7011,7078,7334,7346,11561,13019,216667},
            [itemKeys.objectDrops] = {2852,2855,2857,4096,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213544] = {
            [itemKeys.name] = "Scroll: TOPAZ YORAK",
            [itemKeys.npcDrops] = {205,206,210,315,511,533,570,588,595,597,604,660,669,686,687,696,760,762,765,766,767,769,780,782,785,910,937,940,1023,1034,1096,1110,1114,1142,1151,1270,1666,2248,2268,2269,2271,2305,2319,2356,2359,2371,2375,2377,2387,2406,2407,2408,2503,2554,2556,2559,2560,2564,2582,2619,2721,2735,2831,3752,3759,3767,3770,3771,4038,4095,4097,4100,4101,4109,4133,4139,4140,4142,4143,4147,4150,4308,4392,4396,4397,4632,4635,4664,4665,4670,4672,4726,5253,5292,6575,7334,7344,7346,7603,10759,11561,11562,216667,217305},
            [itemKeys.objectDrops] = {2852,2855,2857,4096,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213545] = {
            [itemKeys.name] = "Scroll: PEATCHY ATTAX",
            [itemKeys.npcDrops] = {205,206,570,587,604,667,674,684,686,691,694,696,697,701,736,740,760,766,767,768,772,780,891,898,978,1019,1022,1023,1034,1038,1097,1152,1157,1418,2241,2245,2247,2248,2251,2252,2268,2269,2319,2320,2358,2359,2368,2369,2370,2371,2373,2374,2375,2376,2406,2408,2555,2559,2563,2564,2567,2574,2586,2589,2729,2760,2764,2831,2894,3752,3755,3767,3770,3810,4100,4107,4109,4110,4113,4124,4140,4142,4143,4147,4151,4154,4308,4342,4345,4359,4360,4361,4378,4541,4633,4637,4641,4642,4664,4668,4674,4675,4679,4680,4685,4693,4696,5336,5618,6167,6575,7011,7333,7344,7346,7603,7726,10757,10761,11559,12677},
            [itemKeys.objectDrops] = {2852,2855,2857,4096,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213546] = {
            [itemKeys.name] = "Scroll: SHOOBEEDOOP",
            [itemKeys.npcDrops] = {205,206,210,533,570,588,597,604,667,676,683,685,686,687,696,736,741,759,765,766,772,785,856,920,937,1034,1036,1057,1082,1085,1096,1110,1157,1270,1557,1666,2240,2268,2269,2318,2319,2358,2370,2371,2372,2373,2374,2375,2376,2377,2406,2407,2408,2412,2556,2557,2565,2578,2589,2595,2618,2628,2728,2735,2760,2894,3752,3757,3767,3799,4093,4097,4101,4109,4130,4140,4142,4143,4147,4151,4154,4158,4248,4308,4362,4378,4397,4398,4504,4633,4635,4668,4671,4685,4692,4711,4712,5249,5336,6224,6575,7333,7344,7603,10896,11561,14123,217669},
            [itemKeys.objectDrops] = {2852,2855,2857,4096,105578},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213547] = {
            [itemKeys.name] = "Scroll: THAW WORDS",
            [itemKeys.npcDrops] = {205,206,212,533,587,682,689,766,780,785,787,889,898,905,920,937,938,940,941,1023,1034,1094,1096,1114,1152,1157,1251,1270,1663,2242,2245,2248,2249,2251,2252,2268,2318,2358,2369,2374,2375,2376,2377,2406,2407,2408,2554,2556,2559,2560,2563,2572,2574,2586,2587,2589,2619,2628,2727,2728,2729,2732,2735,2762,2893,3752,3802,3807,4027,4031,4042,4097,4100,4101,4104,4107,4116,4120,4124,4139,4140,4144,4150,4154,4308,4358,4363,4378,4379,4397,4504,4636,4663,4665,4668,4674,4696,4697,4711,4716,4726,5254,5255,6073,6218,6222,6224,6575,7344,7346,7603,10761,11561,11563,217305},
            [itemKeys.objectDrops] = {2852,2855,2857,4096,105578,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213548] = {
            [itemKeys.name] = "Scroll of Liminal Passage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213549] = {
            [itemKeys.name] = "Scroll of Arcane Angling",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213550] = {
            [itemKeys.name] = "Scroll of Polymorph: Odd Melon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213551] = {
            [itemKeys.name] = "Scroll of Controlled Displacement",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213552] = {
            [itemKeys.name] = "Tear of Theradras",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213553] = {
            [itemKeys.name] = "Tear of Theradras",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {422896},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213554] = {
            [itemKeys.name] = "Vial of Crystal Water",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {422902},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213558] = {
            [itemKeys.name] = "Elemental Salve",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213559] = {
            [itemKeys.name] = "Mechanostrider Gear Shifter",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213560] = {
            [itemKeys.name] = "Mechanostrider Muffler",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213562] = {
            [itemKeys.name] = "Bug Catching Net",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213565},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213563] = {
            [itemKeys.name] = "Field Guide",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213565},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213564] = {
            [itemKeys.name] = "Scroll of Minor Evocation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213565] = {
            [itemKeys.name] = "Entomology Starter Kit",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217412},
            [itemKeys.startQuest] = nil,
        },
        [213566] = {
            [itemKeys.name] = "Arbor Tarantula Specimen",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213567] = {
            [itemKeys.name] = "Flesh Picker Specimen",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213568] = {
            [itemKeys.name] = "Hay Weevil Specimen",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213572] = {
            [itemKeys.name] = "Soul of the Void",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213573] = {
            [itemKeys.name] = "Conjuror's Pendant",
            [itemKeys.npcDrops] = {2590},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213574] = {
            [itemKeys.name] = "Desiccated Seed Pod",
            [itemKeys.npcDrops] = {4632,4633,4634,4635,4636,4637,12976,12977},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213583] = {
            [itemKeys.name] = "Brimstone Carving",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213594] = {
            [itemKeys.name] = "Idol of the Heckler",
            [itemKeys.npcDrops] = {217418},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213599] = {
            [itemKeys.name] = "Unsettling Vision",
            [itemKeys.npcDrops] = {4711,4713,4714,4716,4718,4719,218249},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213632] = {
            [itemKeys.name] = "Partial Spell Notes",
            [itemKeys.npcDrops] = {217669},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213633] = {
            [itemKeys.name] = "Partial Spell Notes",
            [itemKeys.npcDrops] = {217703},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213634] = {
            [itemKeys.name] = "Partial Spell Notes",
            [itemKeys.npcDrops] = {217711},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213641] = {
            [itemKeys.name] = "Box of Gnomeregan Salvage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213701] = {
            [itemKeys.name] = "Strong Harpy Feather",
            [itemKeys.npcDrops] = {4100,4101,4104,4490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213709] = {
            [itemKeys.name] = "Cloud Serpent Fang",
            [itemKeys.npcDrops] = {4117,4118,4119},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213735] = {
            [itemKeys.name] = "Pristine G-7 C.O.R.E. Processor",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [213736] = {
            [itemKeys.name] = "Corroded G-7 C.O.R.E. Processor",
            [itemKeys.npcDrops] = {216666},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79981,
        },
        [213737] = {
            [itemKeys.name] = "Offering to the Wind Spirit",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [214435] = {
            [itemKeys.name] = "Cougar Cage Key",
            [itemKeys.npcDrops] = {4095,4097,4099},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215111] = {
            [itemKeys.name] = "Gneuro-Linked Arcano-Filament Monocle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215112] = {
            [itemKeys.name] = "UNUSED - Cloak of Arcane Insulation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215113] = {
            [itemKeys.name] = "UNUSED - Razor-Lined Shoulderpads",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215114] = {
            [itemKeys.name] = "Glowing Hyperconductive Scale Coif",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215115] = {
            [itemKeys.name] = "Hyperconductive Goldwrap",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215116] = {
            [itemKeys.name] = "UNUSED - Hyperconductive Speed Belt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215126] = {
            [itemKeys.name] = "UNUSED Flask of Merciless Pursuit",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215127] = {
            [itemKeys.name] = "High-Yield Radiation Bomb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215129] = {
            [itemKeys.name] = "Formula: Enchant Weapon - Dismantle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215138] = {
            [itemKeys.name] = "Formula: Enchant Chest - Retricutioner",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215141] = {
            [itemKeys.name] = "Enchanted Sigil: Innovation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215149] = {
            [itemKeys.name] = "Pattern: Gneuro-Linked Arcano-Filament Monocle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215152] = {
            [itemKeys.name] = "Pattern: Glowing Gneuro-Linked Cowl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215153] = {
            [itemKeys.name] = "Schematic: Whirling Truesilver Gearwall",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215155] = {
            [itemKeys.name] = "Plans: Temepered Interference-Negating Helmet",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215156] = {
            [itemKeys.name] = "Schematic: Hyperconductive Goldwrap",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215161] = {
            [itemKeys.name] = "Tempered Interference-Negating Helmet",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215162] = {
            [itemKeys.name] = "Mildly Irradiated Rejuvenation Potion",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215163] = {
            [itemKeys.name] = "Recipe: Mildly-Irradiated Rejuvenation Potion",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215166] = {
            [itemKeys.name] = "Glowing Gneuro-Linked Cowl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215167] = {
            [itemKeys.name] = "Reflective Truesilver Braincage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215168] = {
            [itemKeys.name] = "Ez-Thro Radiation Bomb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215235] = {
            [itemKeys.name] = "2hander Animation as 1hander test copy",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215257] = {
            [itemKeys.name] = "Scroll of Comprehension",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215367] = {
            [itemKeys.name] = "Pattern: Faintly Glowing Leather",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215368] = {
            [itemKeys.name] = "Pattern: Hyperconductive Arcano-Filament",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215369] = {
            [itemKeys.name] = "Pattern: Invoker's Cord",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3364,4168},
            [itemKeys.startQuest] = nil,
        },
        [215370] = {
            [itemKeys.name] = "Pattern: Invoker's Mantle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {3364,4168},
            [itemKeys.startQuest] = nil,
        },
        [215371] = {
            [itemKeys.name] = "Jani's Charm",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {216618},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215373] = {
            [itemKeys.name] = "Silver Hand Training Hammer",
            [itemKeys.npcDrops] = {4283,4286,4287,4288,4289,4290,4291,4292,4293,4294,4295,4296,4297,4298,4299,4300,4301,4302,4303,4306,4540},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215374] = {
            [itemKeys.name] = "Ancestral Sword",
            [itemKeys.npcDrops] = {4283,4286,4287,4288,4289,4290,4291,4292,4293,4294,4295,4296,4297,4298,4299,4300,4301,4302,4303,4306,4540},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215375] = {
            [itemKeys.name] = "Tactician's Staff",
            [itemKeys.npcDrops] = {4283,4286,4287,4288,4289,4290,4291,4292,4293,4294,4295,4296,4297,4298,4299,4300,4301,4302,4303,4306,4540},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215376] = {
            [itemKeys.name] = "Crusader's Mace",
            [itemKeys.npcDrops] = {4283,4286,4287,4288,4289,4290,4291,4292,4293,4294,4295,4296,4297,4298,4299,4300,4301,4302,4303,4306,4540},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215377] = {
            [itemKeys.name] = "Irradiated Robe",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215378] = {
            [itemKeys.name] = "Irradiated Boots",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215379] = {
            [itemKeys.name] = "Irradiated Trousers",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215380] = {
            [itemKeys.name] = "Power-Assisted Lifting Belt",
            [itemKeys.npcDrops] = {218242},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215381] = {
            [itemKeys.name] = "Gneuro-Conductive Channeler's Hood",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215382] = {
            [itemKeys.name] = "Rad-Resistant Scale Hood",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215383] = {
            [itemKeys.name] = "Plans: Reflective Truesilver Braincage",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215384] = {
            [itemKeys.name] = "Plans: Low-Background Truesilver Plates",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215385] = {
            [itemKeys.name] = "Waylaid Supplies: Gold Bars",
            [itemKeys.npcDrops] = {587,588,597,667,682,684,686,689,701,736,757,767,783,858,905,937,941,1062,1557,2240,2241,2248,2249,2250,2252,2369,2376,2377,2406,2407,2408,2556,2559,2562,2563,2582,2586,2595,2628,2650,2729,4130,4139,4140,4142,4147,4150,4151,4308,4329,4361,4378,4413,4632,4633,4634,4636,4638,4640,4641,4642,4648,4666,4670,4672,4673,4674,4696,4714,4726,4728,4834,5232,5255,6222,6224,7175,7725,12976,217305},
            [itemKeys.objectDrops] = {2855,105578},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215386] = {
            [itemKeys.name] = "Waylaid Supplies: Mithril Bars",
            [itemKeys.npcDrops] = {588,669,681,682,683,685,686,687,689,694,701,702,736,764,766,775,905,940,942,1096,1108,1150,1557,2240,2241,2243,2248,2358,2371,2377,2407,2440,2554,2555,2556,2557,2574,2582,2586,2589,2619,2725,2728,2831,2929,2932,4130,4133,4139,4140,4143,4144,4150,4151,4154,4308,4359,4397,4636,4638,4639,4640,4641,4647,4665,4666,4671,4673,4692,4696,4697,4700,4711,4728,4834,5229,5268,5292,5616,6224,7011,7344,7603,9377,11563,12977,218249},
            [itemKeys.objectDrops] = {2855,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215387] = {
            [itemKeys.name] = "Waylaid Supplies: Heavy Hide",
            [itemKeys.npcDrops] = {205,206,210,212,429,533,570,587,588,595,604,682,683,686,689,701,736,785,787,889,920,941,948,1019,1023,1034,1035,1038,1044,1057,1096,1108,1110,1158,1251,1270,1663,1666,2089,2248,2268,2269,2348,2358,2368,2369,2374,2375,2376,2387,2408,2552,2554,2559,2589,2727,3378,3696,3752,3755,3757,3765,3767,3770,3791,3801,3806,3808,3821,4023,4024,4027,4041,4062,4095,4097,4099,4100,4101,4107,4109,4114,4119,4124,4139,4140,4142,4143,4154,4248,4249,4273,4308,4376,4490,4541,4633,4636,4641,4663,4664,4666,4671,4672,4673,4675,4969,6575,7333,10758,10759,10760,10761,10882,11576,12977,13019},
            [itemKeys.objectDrops] = {2852},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215388] = {
            [itemKeys.name] = "Waylaid Supplies: Thick Leather",
            [itemKeys.npcDrops] = {588,595,660,681,682,683,686,689,699,702,764,766,941,1082,1094,1096,1754,2240,2241,2248,2249,2250,2251,2252,2253,2271,2359,2371,2376,2377,2406,2407,2408,2553,2554,2559,2561,2577,2578,2581,2582,2589,2618,2701,2729,2831,3941,4130,4133,4140,4142,4143,4144,4150,4151,4154,4308,4343,4377,4387,4490,4619,4634,4638,4639,4641,4664,4665,4670,4672,4674,4675,4676,4712,4714,4715,4728,5253,5268,5292,5300,5616,7078,7344,7397,7603,11562,12347,217305},
            [itemKeys.objectDrops] = {2855,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215389] = {
            [itemKeys.name] = "Waylaid Supplies: Fadeleaf",
            [itemKeys.npcDrops] = {205,206,210,397,429,533,570,595,604,681,682,683,694,697,736,785,787,891,898,920,937,940,948,1022,1023,1034,1035,1038,1057,1084,1085,1110,1150,1152,1157,1158,1160,1270,1418,1666,2089,2240,2248,2249,2252,2268,2269,2270,2271,2348,2356,2368,2369,2370,2371,2373,2374,2375,2376,2377,2406,2408,2503,2553,2559,2589,2628,3696,3752,3757,3765,3770,3771,3791,3804,3806,3821,3940,4023,4024,4041,4093,4095,4097,4099,4100,4101,4104,4107,4109,4117,4120,4124,4126,4142,4143,4147,4248,4249,4308,4541,4634,4635,4636,4641,4644,4663,4664,4692,4696,4728,4969,6575,7333,10757,10760,10882,11680,11681,11697},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215390] = {
            [itemKeys.name] = "Waylaid Supplies: Khadgar's Whisker",
            [itemKeys.npcDrops] = {587,588,667,671,683,686,687,694,697,702,736,937,938,940,1096,1108,1114,1550,2240,2241,2249,2251,2252,2271,2272,2371,2376,2377,2407,2408,2440,2554,2555,2556,2557,2559,2560,2561,2566,2619,2725,2729,2735,2906,3660,4131,4139,4140,4142,4143,4147,4150,4151,4158,4308,4324,4358,4458,4505,4619,4634,4635,4636,4641,4642,4648,4664,4666,4672,4675,4692,4700,4711,4714,5251,6224,7078,7333,7344,7346,7603},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215391] = {
            [itemKeys.name] = "Waylaid Supplies: Wintersbite",
            [itemKeys.npcDrops] = {205,206,210,397,533,570,604,681,683,694,701,703,740,785,891,892,898,920,941,948,1018,1019,1022,1023,1034,1035,1036,1038,1044,1057,1110,1150,1157,1158,1159,1251,1353,1418,1663,1666,2089,2240,2241,2248,2250,2252,2268,2269,2305,2337,2348,2369,2371,2372,2373,2374,2375,2376,2385,2404,2408,2554,2559,2560,2582,2586,3752,3755,3757,3759,3767,3791,3802,3804,3806,3807,3932,4023,4093,4095,4100,4101,4104,4107,4109,4114,4119,4124,4126,4131,4142,4144,4147,4150,4151,4248,4249,4308,4541,4632,4634,4640,4641,4664,4671,4674,4696,6115,6575,7175,7333,9377,10757,10759,10761,11562,11697,12976,217305,217703,217836},
            [itemKeys.objectDrops] = {2852},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215392] = {
            [itemKeys.name] = "Waylaid Supplies: Purple Lotus",
            [itemKeys.npcDrops] = {595,671,674,677,683,689,696,701,741,772,937,938,940,941,976,1084,1096,1144,1151,2240,2241,2242,2247,2248,2249,2250,2251,2253,2319,2358,2371,2376,2377,2407,2408,2554,2555,2556,2557,2559,2560,2562,2564,2589,2592,2619,2725,2728,2729,4133,4139,4140,4142,4143,4144,4147,4151,4154,4158,4308,4362,4378,4385,4396,4397,4632,4635,4640,4641,4644,4652,4666,4667,4670,4671,4674,4681,4685,4692,4693,4697,4718,4728,4834,5234,5251,5255,7175,7333,7725,11561,11576},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215393] = {
            [itemKeys.name] = "Waylaid Supplies: Greater Healing Potions",
            [itemKeys.npcDrops] = {588,595,674,683,685,689,691,701,745,767,780,781,856,941,1085,1096,1097,1487,2240,2242,2248,2249,2251,2252,2253,2271,2272,2359,2371,2376,2377,2407,2408,2556,2557,2559,2562,2564,2581,2586,2618,2650,2732,4140,4142,4143,4147,4150,4151,4154,4158,4308,4363,4632,4634,4639,4641,4666,4667,4670,4673,4700,4711,4713,4715,4728,5237,5336,6222,7078,7333,7346,7405,7603,7726,11563,11576,11577,12347,217305},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215394] = {
            [itemKeys.name] = "Waylaid Supplies: Lesser Stoneshield Potions",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215395] = {
            [itemKeys.name] = "Waylaid Supplies: Elixirs of Agility",
            [itemKeys.npcDrops] = {92,587,588,667,683,685,687,689,696,697,741,750,858,937,940,978,979,1082,1096,1144,1150,1151,1152,1562,2241,2242,2247,2248,2250,2251,2252,2253,2271,2272,2358,2359,2371,2376,2377,2407,2554,2555,2556,2559,2562,2563,2572,2582,2587,2628,2725,2727,4140,4142,4143,4144,4151,4154,4308,4360,4376,4635,4636,4644,4663,4664,4666,4670,4672,4673,4674,4675,4711,4714,4719,4834,5232,5237,5286,5295,7011,7175,7333,11562,11563,217305},
            [itemKeys.objectDrops] = {2855,105578,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215396] = {
            [itemKeys.name] = "Waylaid Supplies: Elixirs of Greater Defense",
            [itemKeys.npcDrops] = {92,597,660,667,670,671,674,684,687,690,691,696,759,761,764,766,767,783,858,939,942,1082,1096,1114,1142,1557,1561,1564,1907,2242,2243,2246,2247,2557,2561,2567,2572,2573,2574,2649,2650,2716,2725,2729,2732,2743,2760,2761,2893,4260,4342,4343,4344,4360,4361,4362,4363,4377,4379,4386,4387,4399,4411,4644,4645,4654,4676,4680,4681,4682,4684,4693,4727,4729,5229,5232,5234,5237,5268,5274,5278,5293,5295,5334,5335,5616,5618,5843,6219,6222,6224,7011,7334,7344,7346,7405,7603,7855,11559,11561,14123,216667},
            [itemKeys.objectDrops] = {2857},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215397] = {
            [itemKeys.name] = "Waylaid Supplies: Massive Iron Axes",
            [itemKeys.npcDrops] = {597,660,667,671,674,684,686,687,690,691,696,759,764,766,767,772,781,858,922,942,978,979,1082,1096,1151,1488,1491,1551,1557,1713,2246,2253,2423,2557,2561,2566,2572,2573,2574,2649,2651,2701,2725,2728,2729,2732,2734,2735,2736,4323,4324,4342,4343,4352,4355,4359,4361,4362,4363,4376,4377,4378,4379,4385,4386,4387,4396,4411,4668,4677,4679,4682,4685,4693,4694,4699,4719,5234,5251,5260,5268,5286,5292,5295,5300,5331,6218,6219,6222,6224,7011,7078,7334,7344,7346,7603,7725,7727,11561},
            [itemKeys.objectDrops] = {2857},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215398] = {
            [itemKeys.name] = "Waylaid Supplies: Green Iron Bracers",
            [itemKeys.npcDrops] = {205,206,210,212,431,522,533,570,604,681,682,686,689,702,703,736,740,785,787,898,937,948,1022,1023,1034,1035,1038,1096,1110,1157,1158,1159,1270,1418,1663,2248,2252,2268,2269,2319,2348,2358,2359,2369,2370,2371,2375,2376,2385,2404,2406,2407,2408,2450,2554,2557,2559,2564,2566,2582,2586,3752,3757,3758,3759,3763,3765,3767,3770,3782,3791,3799,3803,3804,3806,3807,3810,3811,3825,3833,3932,4024,4027,4038,4041,4042,4062,4093,4095,4097,4099,4100,4101,4104,4107,4109,4114,4116,4117,4118,4119,4120,4124,4126,4131,4133,4139,4140,4142,4143,4147,4150,4154,4249,4308,4376,4413,4541,4548,4632,4634,4635,4636,4637,4640,4648,4653,4666,4670,4674,4696,4713,4728,4834,6073,6575,7078,7404,10757,10758,10759,10761,10992,217711},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215399] = {
            [itemKeys.name] = "Waylaid Supplies: Heavy Mithril Gauntlets",
            [itemKeys.npcDrops] = {587,588,681,685,686,689,691,701,702,759,855,856,858,920,937,938,941,1085,1097,1152,2240,2241,2248,2250,2251,2252,2271,2319,2358,2369,2371,2376,2377,2406,2407,2408,2554,2555,2556,2567,2573,2578,2581,2589,2619,2628,2727,2829,4133,4140,4143,4151,4154,4308,4361,4363,4378,4634,4638,4640,4641,4642,4645,4663,4666,4673,4675,4689,4692,4693,4695,4696,4713,4715,4716,4718,5295,5334,7333,7344,7346,7603,11576,12347,217305},
            [itemKeys.objectDrops] = {2855,105578},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215400] = {
            [itemKeys.name] = "Waylaid Supplies: Solid Grinding Stones",
            [itemKeys.npcDrops] = {205,206,210,212,431,533,604,682,683,699,703,787,855,892,898,909,938,1022,1023,1034,1110,1150,1157,1158,1251,1270,1418,1663,2089,2248,2250,2251,2252,2268,2269,2270,2271,2337,2356,2358,2368,2369,2371,2373,2374,2375,2376,2377,2385,2387,2407,2408,2503,2554,2556,2559,2564,2572,2586,2618,3378,3752,3757,3758,3759,3767,3770,3789,3791,3802,3806,4023,4027,4028,4041,4062,4093,4099,4100,4101,4104,4107,4109,4111,4116,4117,4119,4124,4126,4140,4142,4147,4150,4151,4154,4248,4249,4308,4358,4541,4619,4632,4637,4641,4651,4664,4670,4671,4673,4675,6575,7078,10756,10758,10759,10760,10761,11680,12347,218249},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215401] = {
            [itemKeys.name] = "Waylaid Supplies: Compact Harvest Reaper Kits",
            [itemKeys.npcDrops] = {588,595,682,689,697,701,740,755,921,937,940,1096,1150,2240,2241,2242,2248,2249,2251,2252,2358,2359,2371,2377,2407,2408,2554,2555,2556,2559,2563,2567,2589,2717,2721,2732,4110,4133,4139,4140,4143,4144,4150,4151,4158,4308,4634,4636,4637,4639,4640,4649,4666,4674,4692,4696,4700,4713,4714,4726,5234,5251,5335,6219,7346,7603,11562,11563,14123},
            [itemKeys.objectDrops] = {2855,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215402] = {
            [itemKeys.name] = "Waylaid Supplies: Big Iron Bombs",
            [itemKeys.npcDrops] = {205,206,210,212,431,533,570,595,604,681,682,683,686,689,694,697,785,787,898,938,940,941,1018,1022,1023,1035,1036,1085,1096,1110,1150,1157,1158,1159,1251,1270,1418,1663,1666,2089,2248,2250,2251,2252,2253,2268,2269,2369,2370,2371,2372,2374,2375,2376,2377,2385,2387,2406,2407,2408,2503,2555,2556,2560,2563,2564,2566,2582,2586,2589,2619,3752,3757,3758,3759,3762,3763,3765,3767,3791,3801,3932,4023,4027,4038,4041,4097,4099,4100,4101,4104,4107,4109,4112,4114,4118,4119,4124,4140,4142,4147,4154,4158,4248,4249,4308,4480,4541,4632,4633,4641,4666,4673,4675,4696,4697,4712,6068,6115,6575,7333,10757,10758,10759,10761,11576,12759},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215403] = {
            [itemKeys.name] = "Waylaid Supplies: Deadly Scopes",
            [itemKeys.npcDrops] = {205,206,210,212,300,429,431,533,570,587,604,681,682,683,694,701,785,787,898,920,921,937,941,1019,1023,1034,1036,1038,1096,1110,1150,1158,1418,1663,1666,1720,2089,2248,2268,2269,2270,2271,2348,2356,2368,2370,2371,2372,2374,2375,2376,2377,2387,2408,2450,2554,2555,2556,2559,2582,2587,2589,2619,2628,3696,3752,3755,3757,3758,3770,3791,3797,3806,3807,3818,3932,4027,4041,4093,4097,4099,4100,4101,4104,4107,4109,4114,4118,4119,4120,4124,4133,4139,4140,4142,4143,4151,4154,4249,4308,4541,4632,4637,4640,4641,4663,4665,4671,4673,4674,6575,7333,10756,10757,10758,10759,10760,10761,10896,11562,11681,12347,12918,13019},
            [itemKeys.objectDrops] = {2852},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215404] = {
            [itemKeys.name] = "Waylaid Supplies: Mithril Blunderbuss",
            [itemKeys.npcDrops] = {92,597,660,667,671,674,684,686,691,696,750,751,757,760,762,765,766,767,769,772,780,782,858,939,942,978,1081,1082,1084,1096,1114,1152,1562,1653,1907,2245,2306,2561,2572,2573,2635,2649,2651,2652,2725,2727,2729,2732,2735,2742,2743,2760,4331,4334,4343,4351,4352,4358,4360,4362,4377,4379,4387,4391,4396,4397,4414,4504,4668,4676,4680,4684,4685,4695,4699,4716,5184,5232,5237,5240,5249,5251,5268,5286,5292,5293,5295,5300,5304,5333,5335,5336,5477,6219,6222,6224,7011,7334,7344,7346,7603,11561,14123,216667},
            [itemKeys.objectDrops] = {2857},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215405] = {
            [itemKeys.name] = "Waylaid Supplies: Gnomish Rocket Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215406] = {
            [itemKeys.name] = "Waylaid Supplies: Goblin Mortars",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215407] = {
            [itemKeys.name] = "Waylaid Supplies: Barbaric Shoulders",
            [itemKeys.npcDrops] = {522,587,595,682,683,685,694,701,736,741,782,920,938,940,941,942,1085,1096,1097,1114,1144,2240,2248,2249,2250,2252,2253,2358,2371,2376,2377,2406,2407,2408,2555,2556,2557,2559,2564,2573,2582,2589,2619,2649,2721,2761,2923,4130,4140,4142,4143,4144,4147,4150,4151,4308,4376,4377,4387,4634,4636,4637,4647,4649,4651,4664,4665,4666,4671,4673,4674,4696,4711,4718,4726,4728,4834,6219,6224,7175,7333,7346,11561,13019},
            [itemKeys.objectDrops] = {2855,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215408] = {
            [itemKeys.name] = "Waylaid Supplies: Guardian Gloves",
            [itemKeys.npcDrops] = {205,206,210,511,533,604,681,682,683,686,689,736,785,787,892,898,920,948,949,1022,1023,1034,1036,1096,1110,1150,1157,1158,1270,1418,1663,1666,2248,2250,2251,2268,2269,2356,2368,2369,2370,2371,2372,2374,2375,2376,2377,2387,2404,2408,2448,2503,2554,2556,2559,2560,2562,2581,2582,2586,3378,3752,3754,3757,3771,3806,3808,3810,3821,3834,4024,4027,4029,4038,4041,4042,4093,4095,4097,4099,4100,4101,4104,4107,4109,4114,4124,4126,4140,4142,4143,4151,4158,4248,4249,4308,4351,4541,4632,4633,4635,4663,4666,4675,4696,4711,4726,4834,6073,6575,7175,7333,10617,10756,10759,10760,11576},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215409] = {
            [itemKeys.name] = "Waylaid Supplies: Turtle Scale Bracers",
            [itemKeys.npcDrops] = {597,660,667,671,672,674,687,696,757,760,764,766,767,772,780,939,978,1082,1088,1096,1114,1142,1144,1151,1152,1564,1713,1907,2242,2246,2247,2253,2557,2561,2567,2572,2574,2649,2723,2725,2728,2732,2735,2742,2829,2831,2945,4324,4328,4342,4343,4359,4362,4363,4376,4378,4379,4396,4397,4400,4413,4414,4467,4668,4676,4679,4680,4682,4695,4702,4727,5045,5229,5232,5237,5240,5251,5253,5254,5255,5286,5292,5331,5333,5477,5858,6219,7011,7078,7344,7346,7603,11559,11561,11577,11937},
            [itemKeys.objectDrops] = {2857},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215410] = {
            [itemKeys.name] = "Waylaid Supplies: Dusky Belts",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215411] = {
            [itemKeys.name] = "Waylaid Supplies: Frost Leather Cloaks",
            [itemKeys.npcDrops] = {205,206,210,212,300,429,431,533,570,588,604,681,683,689,787,898,920,938,940,948,1014,1022,1023,1034,1035,1036,1038,1044,1085,1094,1110,1157,1158,1160,1270,1418,2089,2248,2251,2252,2268,2269,2348,2368,2369,2372,2373,2374,2375,2376,2377,2385,2406,2408,2503,2554,2555,2559,2564,2581,2582,2586,2587,2628,3752,3755,3757,3759,3763,3767,3770,3771,3789,3791,3802,3803,3807,3808,3833,3919,3932,3942,4023,4024,4027,4028,4038,4041,4095,4099,4100,4101,4104,4107,4109,4114,4116,4117,4119,4120,4124,4126,4130,4140,4142,4147,4154,4248,4249,4308,4457,4541,4548,4633,4635,4637,4641,4663,4664,4673,4675,4696,4711,4713,4718,6575,7175,7333,10756,10761,11578,12976},
            [itemKeys.objectDrops] = {2852},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215412] = {
            [itemKeys.name] = "Waylaid Supplies: Shadowskin Gloves",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215413] = {
            [itemKeys.name] = "Waylaid Supplies: Formal White Shirts",
            [itemKeys.npcDrops] = {205,206,210,212,533,570,595,604,682,683,688,689,785,787,855,905,948,1018,1022,1023,1034,1036,1044,1084,1110,1150,1157,1160,1258,1418,1663,1720,2089,2240,2241,2248,2253,2269,2270,2305,2319,2359,2368,2369,2370,2372,2374,2375,2377,2387,2406,2408,2450,2503,2556,2559,2563,2572,2579,2581,2582,2586,2587,2589,2628,2714,2727,3752,3757,3759,3762,3765,3789,3807,3808,3932,4024,4027,4041,4093,4095,4097,4100,4101,4104,4107,4109,4113,4114,4118,4119,4124,4133,4139,4142,4143,4147,4150,4151,4248,4308,4541,4634,4637,4638,4641,4647,4664,4665,4666,4674,4696,4711,4719,4728,5600,5827,6073,6575,7078,10756,10758,10759,11562,11576,217711},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215414] = {
            [itemKeys.name] = "Waylaid Supplies: Crimson Silk Pantaloons",
            [itemKeys.npcDrops] = {587,588,595,667,681,682,686,689,741,856,905,937,938,940,941,942,977,1096,1150,1551,2240,2241,2250,2251,2252,2359,2369,2371,2376,2377,2406,2407,2408,2554,2556,2563,2564,2567,2582,2586,2587,2618,2628,2650,2701,2728,2731,2830,3941,4139,4140,4143,4144,4147,4150,4151,4154,4308,4334,4377,4378,4490,4634,4640,4641,4644,4663,4666,4667,4671,4673,4674,4675,4692,4711,4716,5260,6218,7175,7333,7346,7848,9377,11559,11561,11562,11576},
            [itemKeys.objectDrops] = {2855,105578,105579},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215415] = {
            [itemKeys.name] = "Waylaid Supplies: Rich Purple Silk Shirts",
            [itemKeys.npcDrops] = {522,587,597,667,670,674,681,682,683,685,689,694,697,701,736,766,767,780,858,937,940,1085,1094,1096,1144,1152,2248,2249,2250,2251,2252,2253,2271,2358,2359,2376,2377,2406,2407,2408,2554,2555,2556,2560,2563,2564,2566,2587,2721,2725,2831,4140,4143,4150,4151,4308,4413,4481,4635,4636,4638,4640,4641,4645,4666,4667,4668,4670,4672,4673,4684,4696,4711,4719,4726,4728,5229,5236,5600,7333,7346,7603,11559,11562,11576,217305},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215416] = {
            [itemKeys.name] = "Waylaid Supplies: White Bandit Masks",
            [itemKeys.npcDrops] = {92,597,660,667,669,670,671,684,686,687,691,696,741,744,759,760,762,764,765,766,767,769,772,780,781,782,861,922,939,942,1082,1087,1096,1114,1550,1563,2243,2246,2247,2253,2320,2557,2561,2565,2572,2573,2574,2595,2701,2728,2729,2732,2734,2743,2762,2831,2907,2923,4331,4351,4357,4360,4361,4362,4376,4377,4379,4397,4645,4662,4668,4679,4681,4684,4685,4693,4695,4715,4727,5229,5234,5237,5240,5253,5254,5278,5286,5292,5295,5304,5331,5334,5335,6218,6222,6224,7011,7346,7603,7725,11559,11561,11937,14123,216667},
            [itemKeys.objectDrops] = {2857},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215417] = {
            [itemKeys.name] = "Waylaid Supplies: Soothing Turtle Bisque",
            [itemKeys.npcDrops] = {587,588,681,682,685,687,689,701,736,740,765,769,940,943,1150,2240,2241,2248,2251,2252,2253,2271,2304,2318,2319,2359,2369,2371,2376,2377,2406,2555,2556,2559,2560,2561,2563,2565,2586,2595,2619,2628,2651,2714,2732,2740,4130,4140,4143,4151,4154,4158,4308,4329,4358,4634,4635,4638,4639,4640,4648,4663,4665,4670,4673,4679,4700,4702,4711,4712,4713,4714,4715,5292,7344,7848,11563,11576},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215418] = {
            [itemKeys.name] = "Waylaid Supplies: Spider Sausages",
            [itemKeys.npcDrops] = {587,682,683,685,686,689,766,769,920,939,940,941,2240,2248,2252,2253,2271,2272,2359,2369,2376,2377,2406,2407,2408,2553,2554,2555,2556,2557,2564,2587,2717,2727,2830,4062,4131,4133,4140,4143,4151,4154,4158,4308,4359,4362,4379,4480,4634,4636,4640,4644,4666,4670,4674,4680,4700,4712,4714,5236,5295,7346,7603,11576,13019},
            [itemKeys.objectDrops] = {2855},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215419] = {
            [itemKeys.name] = "Waylaid Supplies: Heavy Silk Bandages",
            [itemKeys.npcDrops] = {588,595,681,683,685,689,691,697,701,736,740,741,755,765,905,921,937,938,940,1096,1108,1152,1557,1907,2240,2250,2251,2252,2272,2304,2358,2376,2377,2407,2440,2554,2556,2557,2560,2566,2574,2587,2628,2727,2765,2830,4133,4139,4140,4143,4147,4151,4158,4308,4376,4415,4481,4635,4636,4641,4642,4663,4664,4666,4670,4671,4692,4696,4714,4719,5232,5292,7175,7333,7346,7603,11562,11576,12347,217305,218249},
            [itemKeys.objectDrops] = {2855,105578},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215420] = {
            [itemKeys.name] = "Waylaid Supplies: Rockscale Cod",
            [itemKeys.npcDrops] = {205,206,210,212,533,570,588,595,604,681,683,701,785,787,856,892,920,940,1019,1022,1023,1034,1035,1038,1057,1096,1097,1110,1150,1157,1251,1270,1418,1663,1666,2240,2241,2248,2250,2251,2268,2269,2270,2335,2358,2368,2369,2370,2372,2374,2375,2376,2377,2387,2408,2503,2553,2556,2557,2559,2560,2564,2586,2619,3378,3752,3755,3758,3763,3765,3770,3771,3789,3803,3804,3806,3808,3833,3932,4027,4093,4095,4097,4099,4100,4101,4104,4107,4109,4119,4124,4140,4142,4143,4147,4151,4154,4248,4249,4308,4541,4633,4637,4641,4664,4665,4666,4674,4675,4692,4712,4714,6575,7333,10759,10761,10882,11562,11576},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215421] = {
            [itemKeys.name] = "Waylaid Supplies: Fire Oil",
            [itemKeys.npcDrops] = {205,206,210,397,429,431,533,570,587,588,604,681,682,683,689,694,702,736,785,787,855,898,920,948,1022,1023,1034,1035,1038,1044,1057,1096,1110,1157,1158,1251,1418,1663,1666,2241,2248,2249,2253,2268,2269,2319,2335,2337,2348,2356,2358,2359,2368,2369,2372,2373,2374,2375,2376,2377,2406,2407,2408,2448,2450,2503,2552,2555,2556,2559,2560,2564,2582,2586,2739,2829,3752,3757,3762,3763,3765,3767,3770,3771,3799,3803,3806,3808,3932,4023,4024,4027,4041,4093,4095,4097,4099,4100,4101,4104,4107,4109,4119,4124,4126,4140,4142,4147,4150,4151,4154,4248,4249,4308,4541,4619,4632,4633,4634,4639,4640,4641,4642,4666,4670,4672,4673,4675,4696,4711,4712,4969,6575,7078,10756,10759,10761,11680},
            [itemKeys.objectDrops] = {2852,4096},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215422] = {
            [itemKeys.name] = "Pattern: Glowing Hyperconductive Scale Coif",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215423] = {
            [itemKeys.name] = "Pattern: Gneuro-Conductive Channeler's Hood",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215424] = {
            [itemKeys.name] = "Pattern: Rad-Resistant Scale Hood",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215425] = {
            [itemKeys.name] = "Swamp Echo",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424265},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215426] = {
            [itemKeys.name] = "Graveyard Echo",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424266},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215427] = {
            [itemKeys.name] = "Arathi Echo",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424267},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215428] = {
            [itemKeys.name] = "Theramore Echo",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {424264},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215429] = {
            [itemKeys.name] = "Schematic: Polished Truesilver Gears",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215430] = {
            [itemKeys.name] = "Gnomeregan Fallout",
            [itemKeys.npcDrops] = {6206,6207,6211,6220,6226,6227,6230,6232,6233,6234,6329,215728,216668,217280,218242,218537,220007,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215431] = {
            [itemKeys.name] = "Schematic: High-Yield Radiation Bomb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215432] = {
            [itemKeys.name] = "Schematic: Ez-Thro Radiation Bomb",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215433] = {
            [itemKeys.name] = "Recipe: Insulating Gniodine",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [215434] = {
            [itemKeys.name] = "UNUSED",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215435] = {
            [itemKeys.name] = "Libram of Benediction",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215436] = {
            [itemKeys.name] = "Totem of Invigorating Flame",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215437] = {
            [itemKeys.name] = "Trogg Transfigurator 3000",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215441] = {
            [itemKeys.name] = "Broken Hammer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423703},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79939,
        },
        [215443] = {
            [itemKeys.name] = "Shattered Orb Fragment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215444] = {
            [itemKeys.name] = "Monster - Mace - Broken Pipes",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215449] = {
            [itemKeys.name] = "World Shrinker",
            [itemKeys.npcDrops] = {215728},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215450] = {
            [itemKeys.name] = "Tabard Project (Aidan)",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215451] = {
            [itemKeys.name] = "Large Strongbox",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215452] = {
            [itemKeys.name] = "Medium Strongbox",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215451},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215453] = {
            [itemKeys.name] = "Small Strongbox",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215452},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215454] = {
            [itemKeys.name] = "Tiny Strongbox",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215453},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215456] = {
            [itemKeys.name] = "Singed Message",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215465},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215458] = {
            [itemKeys.name] = "Enclosed Message",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215453},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215459] = {
            [itemKeys.name] = "Enclosed Message",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215452},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215460] = {
            [itemKeys.name] = "Enclosed Message",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {215451},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215461] = {
            [itemKeys.name] = "Domesticated Attack Chicken",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215465] = {
            [itemKeys.name] = "Exploded Strongbox",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215468] = {
            [itemKeys.name] = "Orders from the Grand Crusader",
            [itemKeys.npcDrops] = {218908},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79945,
        },
        [215683] = {
            [itemKeys.name] = "Geomancy: The Stone-Cold Truth",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423895},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215815] = {
            [itemKeys.name] = "Defensive Magics 101",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423896},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215816] = {
            [itemKeys.name] = "A Web of Lies: Debunking Myths and Legends",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423897},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215817] = {
            [itemKeys.name] = "Demons and You",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423898},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215820] = {
            [itemKeys.name] = "Mummies: A Guide to the Unsavory Undead",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423899},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215822] = {
            [itemKeys.name] = "RwlRwlRwlRwl!",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423900},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [215824] = {
            [itemKeys.name] = "A Ludite's Guide to Caring for Your Demonic Pet",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {423901},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216483] = {
            [itemKeys.name] = "Witherbark Mallet",
            [itemKeys.npcDrops] = {2552,2553,2554,2555,2556,2557},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216484] = {
            [itemKeys.name] = "Shockforged Battleboots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [216485] = {
            [itemKeys.name] = "Shockforged Breastplate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [216486] = {
            [itemKeys.name] = "Shockforged Legplates",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [216490] = {
            [itemKeys.name] = "Idol of Wrath",
            [itemKeys.npcDrops] = {217280},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216491] = {
            [itemKeys.name] = "Shipment of Stranglethorn Lumber",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216492] = {
            [itemKeys.name] = "Whistle of the Mottled Blood Raptor",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216494] = {
            [itemKeys.name] = "Aragriar's Whimsical World Warper",
            [itemKeys.npcDrops] = {220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216495] = {
            [itemKeys.name] = "Sanguine Crusher",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216496] = {
            [itemKeys.name] = "Sanguine Skullcrusher",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216497] = {
            [itemKeys.name] = "Exsanguinar",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216498] = {
            [itemKeys.name] = "Enchanted Sanguine Grimoire",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216499] = {
            [itemKeys.name] = "Bloodbark Crusher",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216500] = {
            [itemKeys.name] = "Bloodbonded Grove Talisman",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216501] = {
            [itemKeys.name] = "Bloodstorm Barrier",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216502] = {
            [itemKeys.name] = "Bloodstorm War Totem",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216503] = {
            [itemKeys.name] = "Bloodstorm Jewel",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216504] = {
            [itemKeys.name] = "Eclipsed Bloodlight Saber",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216505] = {
            [itemKeys.name] = "Bloodlight Crusader's Radiance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216506] = {
            [itemKeys.name] = "Bloodlight Avenger's Edge",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216507] = {
            [itemKeys.name] = "Umbral Bloodseal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216508] = {
            [itemKeys.name] = "Infernal Bloodcoil Band",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216509] = {
            [itemKeys.name] = "Infernal Pact Essence",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216510] = {
            [itemKeys.name] = "Blood Resonance Circle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216511] = {
            [itemKeys.name] = "Emberblood Seal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216512] = {
            [itemKeys.name] = "Loop of Chilled Veins",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216513] = {
            [itemKeys.name] = "Tigerblood Talisman",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216514] = {
            [itemKeys.name] = "Sanguine Quiver",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216515] = {
            [itemKeys.name] = "Sanguine Ammo Pouch",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216516] = {
            [itemKeys.name] = "Bloodlash Bow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216517] = {
            [itemKeys.name] = "Sanguine Sanctuary",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216518] = {
            [itemKeys.name] = "Blood Covenant Seal",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216519] = {
            [itemKeys.name] = "Sanguine Shadow Band",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216520] = {
            [itemKeys.name] = "Bloodharvest Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216521] = {
            [itemKeys.name] = "Swift Sanguine Strikers",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216522] = {
            [itemKeys.name] = "Blood Spattered Stiletto",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216523] = {
            [itemKeys.name] = "Rusted Cage Key",
            [itemKeys.npcDrops] = {759,762},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216549] = {
            [itemKeys.name] = "Reins of the Bengal Tiger",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216570] = {
            [itemKeys.name] = "Reins of the Golden Sabercat",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216606] = {
            [itemKeys.name] = "Rune of Two-Handed Mastery",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {217014},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216607] = {
            [itemKeys.name] = "Bloodlight Offering",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216608] = {
            [itemKeys.name] = "Radiant Ray Reflectors",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216610] = {
            [itemKeys.name] = "Orders from the Grand Crusader",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216615] = {
            [itemKeys.name] = "Ancestral Bloodstorm Beacon",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216616] = {
            [itemKeys.name] = "Compound Cage Key",
            [itemKeys.npcDrops] = {939,978},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216617] = {
            [itemKeys.name] = "Kurzen Fighter's Uniform",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216618] = {
            [itemKeys.name] = "Captain Aransas' Reward",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216619] = {
            [itemKeys.name] = "Student Fodder",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216620] = {
            [itemKeys.name] = "Bloodrot Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216621] = {
            [itemKeys.name] = "Blooddrenched Drape",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216622] = {
            [itemKeys.name] = "Coagulated Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216623] = {
            [itemKeys.name] = "Cape of Hemostasis",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216633] = {
            [itemKeys.name] = "Hydraulic Truesilver Strut",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216634] = {
            [itemKeys.name] = "GG12-082 Cartridge Fuse",
            [itemKeys.npcDrops] = {6224,6225,6226,6227,6230,6232,6233,6234,215728,218242,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216635] = {
            [itemKeys.name] = "Spent Voidcore",
            [itemKeys.npcDrops] = {5335,5336,5337},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216636] = {
            [itemKeys.name] = "Charged Voidcore",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216637] = {
            [itemKeys.name] = "A Pile of Random Parts",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {213641},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216645] = {
            [itemKeys.name] = "Mote of Darkness",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216646] = {
            [itemKeys.name] = "Ziri's Mystery Crate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {217689},
            [itemKeys.startQuest] = nil,
        },
        [216661] = {
            [itemKeys.name] = "Grime-Encrusted Ring",
            [itemKeys.npcDrops] = {6212,6228},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79986,
        },
        [216662] = {
            [itemKeys.name] = "Brilliant Gold Ring",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216673] = {
            [itemKeys.name] = "Talvash's Brilliant Gold Ring",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216674] = {
            [itemKeys.name] = "Nogg's Brilliant Gold Ring",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216675] = {
            [itemKeys.name] = "Pristine Civinad Robes",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216676] = {
            [itemKeys.name] = "Nimble Triprunner Dungarees",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216678] = {
            [itemKeys.name] = "Triple Reinforced Leggings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216679] = {
            [itemKeys.name] = "Shoni's Dismantling Tool",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216680] = {
            [itemKeys.name] = "Shilly Mittens",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216698] = {
            [itemKeys.name] = "Sergeant Major's Dragonhide Armsplints",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216699] = {
            [itemKeys.name] = "Sergeant Major's Chain Armguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216700] = {
            [itemKeys.name] = "Sergeant Major's Leather Armsplints",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216701] = {
            [itemKeys.name] = "Sergeant Major's Plate Wristguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216702] = {
            [itemKeys.name] = "Sergeant Major's Silk Cuffs",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216703] = {
            [itemKeys.name] = "First Sergeant's Mail Wristguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216704] = {
            [itemKeys.name] = "First Sergeant's Dragonhide Armguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216705] = {
            [itemKeys.name] = "First Sergeant's Plate Bracers",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216706] = {
            [itemKeys.name] = "First Sergeant's Leather Armguards",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216707] = {
            [itemKeys.name] = "First Sergeant's Silk Cuffs",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216738] = {
            [itemKeys.name] = "Manual of Redirect",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4542,4543,6487,6488,6489,6490,7355},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216740] = {
            [itemKeys.name] = "Tome of Expanded Intellect",
            [itemKeys.npcDrops] = {3974,3975,3983,4543,6487,6488,6489,6490,7355},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216744] = {
            [itemKeys.name] = "Scroll of Increased Fortitude",
            [itemKeys.npcDrops] = {3974,3975,3983,4542,4543,6487,6488,6489,6490,6906,7356,7357},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216745] = {
            [itemKeys.name] = "Scroll of Shadowfiend",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4542,4543,6487,6488,6489,6490,7358},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216746] = {
            [itemKeys.name] = "Handbook of Commanding Shout",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4542,4543,6487,6488,6489,6490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216747] = {
            [itemKeys.name] = "Grimoire of Soul Harvesting",
            [itemKeys.npcDrops] = {3974,3975,3983,4543,6487,6488,6489,6490,7357,7358,8567},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216748] = {
            [itemKeys.name] = "Grimoire of Portal of Summoning",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4543,6487,6488,6489,6490,7228},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216764] = {
            [itemKeys.name] = "Leaflet of Deeper Wilds",
            [itemKeys.npcDrops] = {3974,3975,3983,4542,4543,6487,6488,6489,6490,7355,7357,8567},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216767] = {
            [itemKeys.name] = "Leaflet of Revive",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4542,4543,6487,6488,6489,6490,8567},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216768] = {
            [itemKeys.name] = "Testament of Enhanced Blessings",
            [itemKeys.npcDrops] = {3974,3976,3977,3983,4543,6487,6488,6489,6490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216769] = {
            [itemKeys.name] = "Revelation of Totemic Projection",
            [itemKeys.npcDrops] = {3974,3975,3976,3983,4543,6487,6488,6489,6490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216770] = {
            [itemKeys.name] = "Treatise on Aspect of the Viper",
            [itemKeys.npcDrops] = {3974,3975,3976,3977,3983,4542,4543,6487,6488,6489,6490,7357},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216771] = {
            [itemKeys.name] = "Leaflet of Enhanced Restoration",
            [itemKeys.npcDrops] = {3974,3975,3976,3977,3983,4542,4543,6487,6488,6489,6490,6910,7228,7358,8567},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216879] = {
            [itemKeys.name] = "Mysterious Troll Scroll",
            [itemKeys.npcDrops] = {587,588,595,667,671,696,697,699,702,780},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216880] = {
            [itemKeys.name] = "Deciphered Troll Scroll",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 79731,
        },
        [216884] = {
            [itemKeys.name] = "Bloodthirst Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216914] = {
            [itemKeys.name] = "Bloodstained Commendation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216938] = {
            [itemKeys.name] = "Greater Insignia of the Alliance",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12805},
            [itemKeys.startQuest] = nil,
        },
        [216939] = {
            [itemKeys.name] = "Greater Insignia of the Horde",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {12799},
            [itemKeys.startQuest] = nil,
        },
        [216941] = {
            [itemKeys.name] = "Ariden's Sigil",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216945] = {
            [itemKeys.name] = "Curious Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216946] = {
            [itemKeys.name] = "Glittering Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216947] = {
            [itemKeys.name] = "Whirring Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216948] = {
            [itemKeys.name] = "Odd Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216949] = {
            [itemKeys.name] = "Heavy Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216950] = {
            [itemKeys.name] = "Creepy Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216951] = {
            [itemKeys.name] = "Slippery Dalaran Relic",
            [itemKeys.npcDrops] = {218931},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216956] = {
            [itemKeys.name] = "Torn Letter",
            [itemKeys.npcDrops] = {4668,4705},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [216971] = {
            [itemKeys.name] = "Satchel of Copper Blood Coins",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [216972] = {
            [itemKeys.name] = "Satchel of Silver Blood Coins",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {218115},
            [itemKeys.startQuest] = nil,
        },
        [217005] = {
            [itemKeys.name] = "Repairman's Cape",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217006] = {
            [itemKeys.name] = "Mechanic's Pipehammer",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217007] = {
            [itemKeys.name] = "Power Depleted Boots",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217008] = {
            [itemKeys.name] = "Power Depleted Chest",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217009] = {
            [itemKeys.name] = "Power Depleted Legs",
            [itemKeys.npcDrops] = {218242,218537,220072},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217014] = {
            [itemKeys.name] = "Supply Bag",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217017] = {
            [itemKeys.name] = "Fishing Tournament!",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217161] = {
            [itemKeys.name] = "Spell Notes: TROFF IRESTBOL",
            [itemKeys.npcDrops] = {780},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217244] = {
            [itemKeys.name] = "Pattern: Crimson Silk Robe",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {6568},
            [itemKeys.startQuest] = nil,
        },
        [217245] = {
            [itemKeys.name] = "Crimson Silk Robe",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217246] = {
            [itemKeys.name] = "Black Mageweave Vest",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217247] = {
            [itemKeys.name] = "Black Mageweave Leggings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217248] = {
            [itemKeys.name] = "Earthen Silk Belt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217249] = {
            [itemKeys.name] = "Pattern: Earthen Silk Belt",
            [itemKeys.npcDrops] = {595,2588,3974,3983,4143,4286,4287,4288,4289,4290,4291,4292,4293,4294,4295,4296,4298,4299,4300,4301,4302,4304,4308,4540,4860,5253,6426,6487,7342,7349,7351,11561,14123},
            [itemKeys.objectDrops] = {75296,75300},
            [itemKeys.itemDrops] = {4634},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217250] = {
            [itemKeys.name] = "Crimson Silk Shoulders",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217251] = {
            [itemKeys.name] = "Pattern: Crimson Silk Shoulders",
            [itemKeys.npcDrops] = {667,772,2649,3975,4286,4287,4290,4291,4294,4295,4296,4298,4299,4300,4301,4302,4304,4308,4359,4540,4674,4712,4850,4851,6230,6427,6489,7328,7346,7348,11561,216668},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = {4636},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217252] = {
            [itemKeys.name] = "Long Silken Cloak",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217253] = {
            [itemKeys.name] = "Boots of the Enchanter",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217254] = {
            [itemKeys.name] = "Pattern: Boots of the Enchanter",
            [itemKeys.npcDrops] = {940,1097,1715,2371,2377,2387,2406,2565,3378,3974,4107,4283,4286,4287,4290,4291,4292,4293,4296,4298,4299,4302,4304,4308,4436,4540,4543,4711,6426,6427,7332,7335,7342,7873},
            [itemKeys.objectDrops] = {2855,75296,75299,75300},
            [itemKeys.itemDrops] = {4634},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217255] = {
            [itemKeys.name] = "Crimson Silk Belt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217256] = {
            [itemKeys.name] = "Earthen Vest",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217257] = {
            [itemKeys.name] = "Enchanter's Cowl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217258] = {
            [itemKeys.name] = "Pattern: Enchanter's Cowl",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {2670,12246},
            [itemKeys.startQuest] = nil,
        },
        [217259] = {
            [itemKeys.name] = "Big Voodoo Mask",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217260] = {
            [itemKeys.name] = "Pattern: Big Voodoo Mask",
            [itemKeys.npcDrops] = {782,4286,4292,4295,4298,4301,4540,4716,6212},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217261] = {
            [itemKeys.name] = "Big Voodoo Robe",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217262] = {
            [itemKeys.name] = "Pattern: Big Voodoo Robe",
            [itemKeys.npcDrops] = {4286,4289,4295,4299,4301,4540,4850,7342},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217263] = {
            [itemKeys.name] = "Guardian Leather Bracers",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217264] = {
            [itemKeys.name] = "Pattern: Guardian Leather Bracers",
            [itemKeys.npcDrops] = {2553,4286,4287,4289,4290,4291,4295,4298,4299,4301,4302,4304,4393,4532,4851,6206,6426,6427,7603},
            [itemKeys.objectDrops] = {2857,75299},
            [itemKeys.itemDrops] = {4636},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217265] = {
            [itemKeys.name] = "Guardian Belt",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217266] = {
            [itemKeys.name] = "Pattern: Guardian Belt",
            [itemKeys.npcDrops] = {1715,4296},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217267] = {
            [itemKeys.name] = "Guardian Pants",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217268] = {
            [itemKeys.name] = "Turtle Scale Breastplate",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217270] = {
            [itemKeys.name] = "Turtle Scale Gloves",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217271] = {
            [itemKeys.name] = "Pattern: Turtle Scale Gloves",
            [itemKeys.npcDrops] = {2907,4292,4540},
            [itemKeys.objectDrops] = {2560},
            [itemKeys.itemDrops] = {21228},
            [itemKeys.vendors] = {7852,7854},
            [itemKeys.startQuest] = nil,
        },
        [217273] = {
            [itemKeys.name] = "Golden Scale Gauntlets",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217274] = {
            [itemKeys.name] = "Plans: Golden Scale Gauntlets",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217275] = {
            [itemKeys.name] = "Golden Scale Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217276] = {
            [itemKeys.name] = "Plans: Golden Scale Boots",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217277] = {
            [itemKeys.name] = "Golden Scale Cuirass",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217278] = {
            [itemKeys.name] = "Plans: Golden Scale Cuirass",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = {2857,75300},
            [itemKeys.itemDrops] = {4637},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217279] = {
            [itemKeys.name] = "Golden Scale Coif",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217280] = {
            [itemKeys.name] = "Plans: Golden Scale Coif",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {5411},
            [itemKeys.startQuest] = nil,
        },
        [217281] = {
            [itemKeys.name] = "Moonsteel Broadsword",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217282] = {
            [itemKeys.name] = "Plans: Moonsteel Broadsword",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {2482},
            [itemKeys.startQuest] = nil,
        },
        [217283] = {
            [itemKeys.name] = "Golden Scale Shoulders",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217284] = {
            [itemKeys.name] = "Plans: Golden Scale Shoulders",
            [itemKeys.npcDrops] = {938,2251,2565,4287,4288,4291,4299,4304,4306,4308,4540,4851,6426,6427},
            [itemKeys.objectDrops] = {74448,75296,75299},
            [itemKeys.itemDrops] = {4634},
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217285] = {
            [itemKeys.name] = "Golden Scale Leggings",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217286] = {
            [itemKeys.name] = "Plans: Golden Scale Leggings",
            [itemKeys.npcDrops] = {667,694,736,941,2241,2369,2407,3802,4283,4286,4287,4288,4289,4291,4293,4294,4296,4297,4299,4302,4303,4304,4306,4308,4421,4436,4440,4515,4539,4540,4541,4623,4700,6426,6427},
            [itemKeys.objectDrops] = {75299},
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217287] = {
            [itemKeys.name] = "Greater Mystic Wand",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217288] = {
            [itemKeys.name] = "Robes of the Lich",
            [itemKeys.npcDrops] = {7358},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217289] = {
            [itemKeys.name] = "Deathchill Armor",
            [itemKeys.npcDrops] = {7358},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217290] = {
            [itemKeys.name] = "Glowing Eye of Mordresh",
            [itemKeys.npcDrops] = {7357},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217291] = {
            [itemKeys.name] = "Mordresh's Lifeless Skull",
            [itemKeys.npcDrops] = {7357},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217292] = {
            [itemKeys.name] = "Deathmage Sash",
            [itemKeys.npcDrops] = {7357},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217293] = {
            [itemKeys.name] = "Silky Spider Cape",
            [itemKeys.npcDrops] = {7355},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217294] = {
            [itemKeys.name] = "Embalmed Shroud",
            [itemKeys.npcDrops] = {6488},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217295] = {
            [itemKeys.name] = "Necrotic Wand",
            [itemKeys.npcDrops] = {6490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217296] = {
            [itemKeys.name] = "Ghostshard Talisman",
            [itemKeys.npcDrops] = {6490},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217297] = {
            [itemKeys.name] = "Robe of Doan",
            [itemKeys.npcDrops] = {6487},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217298] = {
            [itemKeys.name] = "Mantle of Doan",
            [itemKeys.npcDrops] = {6487},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217299] = {
            [itemKeys.name] = "Illusionary Rod",
            [itemKeys.npcDrops] = {6487},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217300] = {
            [itemKeys.name] = "Whitemane's Chapeau",
            [itemKeys.npcDrops] = {3977},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217301] = {
            [itemKeys.name] = "Triune Amulet",
            [itemKeys.npcDrops] = {3977},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217302] = {
            [itemKeys.name] = "Mograine's Might",
            [itemKeys.npcDrops] = {3976},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217303] = {
            [itemKeys.name] = "Stoneweaver Leggings",
            [itemKeys.npcDrops] = {7228},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217304] = {
            [itemKeys.name] = "Revelosh's Gloves",
            [itemKeys.npcDrops] = {6910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217305] = {
            [itemKeys.name] = "Revelosh's Armguards",
            [itemKeys.npcDrops] = {6910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217306] = {
            [itemKeys.name] = "Revelosh's Boots",
            [itemKeys.npcDrops] = {6910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217307] = {
            [itemKeys.name] = "Revelosh's Spaulders",
            [itemKeys.npcDrops] = {6910},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217308] = {
            [itemKeys.name] = "Enchanted Sigil: Innovation",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217314] = {
            [itemKeys.name] = "Moonsight Rifle",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217315] = {
            [itemKeys.name] = "Precision Bow",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217336] = {
            [itemKeys.name] = "Monster - Incubus Whip Mace1H",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217337] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217338] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217339] = {
            [itemKeys.name] = "Supply Shipment",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217350] = {
            [itemKeys.name] = "Thermaplugg's Engineering Notes",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 80324,
        },
        [217351] = {
            [itemKeys.name] = "Thermaplugg's Engineering Notes",
            [itemKeys.npcDrops] = {218537},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = 80325,
        },
        [217398] = {
            [itemKeys.name] = "Lesser Arcane Elixir",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217399] = {
            [itemKeys.name] = "Recipe: Lesser Arcane Elixir",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = {213077,214070,214099},
            [itemKeys.startQuest] = nil,
        },
        [217495] = {
            [itemKeys.name] = "Scroll of Cryoblast",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217496] = {
            [itemKeys.name] = "Scroll of the Blade",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217497] = {
            [itemKeys.name] = "Narpas Sword",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217693] = {
            [itemKeys.name] = "Invisibility Potion",
            [itemKeys.npcDrops] = nil,
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
        [217704] = {
            [itemKeys.name] = "Ironshod Bludgeon",
            [itemKeys.npcDrops] = {7228},
            [itemKeys.objectDrops] = nil,
            [itemKeys.itemDrops] = nil,
            [itemKeys.vendors] = nil,
            [itemKeys.startQuest] = nil,
        },
    }
end
