---@type SeasonOfDiscovery
local SeasonOfDiscovery = QuestieLoader:ImportModule("SeasonOfDiscovery")
---@type QuestieDB
local QuestieDB = QuestieLoader:ImportModule("QuestieDB")
---@type ZoneDB
local ZoneDB = QuestieLoader:ImportModule("ZoneDB")

--- Load the base NPCs for Season of Discovery
--- These are generated, do NOT EDIT the data entries here.
--- If you want to edit an NPC, do so in sodNPCFixes.lua
function SeasonOfDiscovery:LoadBaseNPCs()
    local npcKeys = QuestieDB.npcKeys
    local zoneIDs = ZoneDB.zoneIDs
    local sortKeys = QuestieDB.sortKeys

    return {
        [162539] = {
            [npcKeys.name] = "World Talent Master",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [169106] = {
            [npcKeys.name] = "Shea Dorrisist",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [173338] = {
            [npcKeys.name] = "Invisible Bunny",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [176552] = {
            [npcKeys.name] = "Orgrimmar",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [176553] = {
            [npcKeys.name] = "Stormwind",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [176663] = {
            [npcKeys.name] = "Call for Help Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [176721] = {
            [npcKeys.name] = "Dummy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [177063] = {
            [npcKeys.name] = "Spellcaster Dummy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [183709] = {
            [npcKeys.name] = "Season 01 Controller - Era",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [183713] = {
            [npcKeys.name] = "Season 02 Controller - Fresh Start",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [184314] = {
            [npcKeys.name] = "Beta Vendor and Raid Teleporter",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [184383] = {
            [npcKeys.name] = "Sulfuron Harbinger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185273] = {
            [npcKeys.name] = "Firemaw Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185320] = {
            [npcKeys.name] = "Unstable Chromatic Drake",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 2677,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185331] = {
            [npcKeys.name] = "Avelina Lilly Projection",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185332] = {
            [npcKeys.name] = "Isaac Pearson Projection",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185336] = {
            [npcKeys.name] = "Jezelle's Incubus",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{77.4, 13.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185342] = {
            [npcKeys.name] = "Ruby Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185371] = {
            [npcKeys.name] = "Emerald Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185378] = {
            [npcKeys.name] = "Azure Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185379] = {
            [npcKeys.name] = "Obsidian Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185380] = {
            [npcKeys.name] = "Bronze Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185536] = {
            [npcKeys.name] = "Suppression Device Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185585] = {
            [npcKeys.name] = "Flame Patch",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185590] = {
            [npcKeys.name] = "Dream Fog",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185605] = {
            [npcKeys.name] = "Chromatic Drakonid Spawner",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185631] = {
            [npcKeys.name] = "Corrupted Weapon",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185650] = {
            [npcKeys.name] = "Dream Breath Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185658] = {
            [npcKeys.name] = "Corrupted Weapon",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185746] = {
            [npcKeys.name] = "Bouncing Bolt",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185760] = {
            [npcKeys.name] = "Nightmare Brambles",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185859] = {
            [npcKeys.name] = "Wing Buffet Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [185868] = {
            [npcKeys.name] = "[DNT] Electricity Visual Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [186130] = {
            [npcKeys.name] = "[DNT] Ebonroc and Flamegor Controller",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [186348] = {
            [npcKeys.name] = "The Prophet Skeram",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [186506] = {
            [npcKeys.name] = "[DNT] Frost Tomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187162] = {
            [npcKeys.name] = "Eye of C'Thun",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187237] = {
            [npcKeys.name] = "Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187274] = {
            [npcKeys.name] = "Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187309] = {
            [npcKeys.name] = "Eye of C'Thun",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187327] = {
            [npcKeys.name] = "Claw Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187345] = {
            [npcKeys.name] = "Eye Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187346] = {
            [npcKeys.name] = "Giant Claw Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187383] = {
            [npcKeys.name] = "AQ40 Bonus Difficulty Trigger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187564] = {
            [npcKeys.name] = "AQ40 Bonus Difficulty Indicator",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187664] = {
            [npcKeys.name] = "Fungal Spore Cluster",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187724] = {
            [npcKeys.name] = "Mad Scientist",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187725] = {
            [npcKeys.name] = "Deathknight Captain",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187728] = {
            [npcKeys.name] = "Necropolis Acolyte",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187729] = {
            [npcKeys.name] = "Patchwork Golem",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187798] = {
            [npcKeys.name] = "Corrupted Weapon",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187801] = {
            [npcKeys.name] = "Corrupted Weapon",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187832] = {
            [npcKeys.name] = "Skeletal Hitching Post",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187978] = {
            [npcKeys.name] = "Thaddius Mechanic Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [187993] = {
            [npcKeys.name] = "Plagued Ghoul",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188018] = {
            [npcKeys.name] = "Dread Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188109] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188110] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188111] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188119] = {
            [npcKeys.name] = "Claw Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188131] = {
            [npcKeys.name] = "Giant Claw Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188134] = {
            [npcKeys.name] = "Eye Tentacle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188147] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188148] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188170] = {
            [npcKeys.name] = "Giant Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188178] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188182] = {
            [npcKeys.name] = "Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [188183] = {
            [npcKeys.name] = "Giant Tentacle Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [189047] = {
            [npcKeys.name] = "[DNT] Quick Sand",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [200571] = {
            [npcKeys.name] = "Artificer Terothordius",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [201722] = {
            [npcKeys.name] = "Ghamoo-ra",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [201854] = {
            [npcKeys.name] = "Cutty",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{50, 52},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [201933] = {
            [npcKeys.name] = "Spellfrost Orb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202060] = {
            [npcKeys.name] = "Frozen Murloc",
            [npcKeys.minLevel] = 9,
            [npcKeys.maxLevel] = 9,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{76.8, 51.4},{77, 51.8},},
                [85] = {{66.2, 40.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202079] = {
            [npcKeys.name] = "Shadowfiend",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202093] = {
            [npcKeys.name] = "Polymorphed Apprentice",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{28.2, 82.8},{28.4, 82.4},{28.6, 83.2},{28.8, 82.4},{29.4, 84},{29.6, 83.2},{29.6, 83.8},{29.6, 84.6},{30.4, 90.4},{30.4, 90.6},{30.8, 90.2},{31, 90.6},{31.2, 68.4},{31.2, 68.8},{31.2, 87},{31.6, 90},{31.8, 90.6},{34.4, 83.2},{34.8, 82.2},{34.8, 83},{34.8, 84},{37.2, 77.6},{37.4, 77.2},{37.8, 77.6},{38, 76.8},{38.2, 76.4},{38.6, 76},{40.4, 82.8},{40.4, 89.4},{40.4, 90},{40.6, 87.6},{40.6, 89.4},{40.8, 89.8},{44.4, 56.4},{44.4, 57},{44.4, 57.6},{44.6, 56.8},{44.6, 57.6},{44.8, 56.2},{46, 86.4},{46, 86.6},{46.2, 70.8},{46.4, 69.4},{46.4, 70.2},{46.6, 70.4},{46.8, 71},{48.4, 52.4},{48.4, 52.8},{48.4, 54.2},{48.4, 54.8},{48.4, 81.4},{48.6, 51.4},{48.6, 52.6},{48.6, 53.6},{48.6, 55},{48.8, 51.8},{48.8, 55.6},{48.8, 81.2},{49, 81.6},{49.2, 72.8},{49.4, 73.6},{49.8, 73.4},{49.8, 73.6},{51.4, 84},{56.2, 81},{56.2, 81.6},{56.4, 80.2},{56.6, 80.4},{56.6, 81},{58.4, 60},{59, 59.4},{59.2, 60},{61.4, 76.8},{61.6, 77},{61.8, 77.6},{62.4, 63.2},{62.4, 63.6},{62.8, 63.8},{63.2, 63},{63.4, 62.4},{63.6, 62.8},{65, 41.2},{65.4, 42},{65.6, 41.6},{65.6, 42.8},{65.8, 41.2},{66.8, 82.2},{67, 82.6},{67.2, 81.2},{68.8, 74.6},{69.4, 76.2},{69.8, 75.8},{70, 75.4},{70.2, 63.6},{70.4, 63.4},{70.6, 63.2},{70.6, 63.6},{73.4, 41.2},{73.8, 40.4},{74, 41},{77.4, 39.4},{77.4, 40},{77.6, 39.4},{77.6, 39.8},{78.4, 75.2},{79.2, 79.8},{79.4, 64.2},{79.4, 65.6},{79.4, 78.4},{79.4, 78.8},{79.6, 64.2},{79.6, 79},{79.6, 80.2},{79.8, 64.6},{80, 65.8},{80.4, 49.8},{80.4, 50.6},{80.6, 50.4},{80.8, 49.4},{80.8, 50.6},{82.4, 70},{82.4, 70.8},{82.4, 86},{82.4, 86.8},{82.6, 86.2},{82.8, 70.6},{83, 70.4},{83.4, 83.4},{83.4, 83.8},{83.4, 86.8},{83.6, 82.8},{83.8, 84},{83.8, 84.6},{85.4, 65.4},{85.4, 66},{85.8, 65.2},{85.8, 66},{85.8, 67.2},{86.6, 65.6},{86.8, 82.6},{87, 82.2},{87.6, 82.2},{90.2, 77.2},{90.4, 77.6},{90.6, 77.2},{90.6, 77.8},{90.8, 76.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202116] = {
            [npcKeys.name] = "Cut-throat Mugger",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{57.4, 28.8},{61.4, 29.2},{61.8, 29.2},{62.8, 29},{63.2, 29.6},},
                [1537] = {{44, 11},{47.4, 12.6},{49.2, 12.6},{50.4, 11.6},{51.2, 12.6},{51.4, 10.6},{51.4, 11.8},{51.8, 12.4},{51.8, 12.8},{52, 10.6},{52.6, 9.6},{52.8, 9},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202387] = {
            [npcKeys.name] = "Eye of the Void",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202390] = {
            [npcKeys.name] = "Homunculus",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202391] = {
            [npcKeys.name] = "Homunculus",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202392] = {
            [npcKeys.name] = "Homunculus",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202699] = {
            [npcKeys.name] = "Baron Aquanis",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202838] = {
            [npcKeys.name] = "Fathom Elemental",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202839] = {
            [npcKeys.name] = "Riptide Elemental",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [202840] = {
            [npcKeys.name] = "Torrent Elemental",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203079] = {
            [npcKeys.name] = "Wandering Swordsman",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{50.2, 49},{51.8, 48.8},{52.6, 48.8},{53.4, 47.4},{53.4, 47.6},{53.6, 47.4},{53.6, 47.6},},
                [12] = {{22.2, 73.2},{22.2, 73.6},{22.4, 72.4},{22.8, 73.6},{23, 73.2},{23.4, 74.8},{23.6, 74},{24, 75.2},{24.2, 70.6},{24.4, 69.4},{24.6, 75.2},{25, 70.8},{25.2, 70},{25.4, 69.2},{25.6, 70},{25.6, 74.2},{26.2, 72.2},{28.8, 72.2},{29.8, 74.8},{29.8, 76.4},{30.2, 73.2},{30.2, 73.6},{30.6, 73.4},{30.6, 74.2},{35.6, 79.2},{35.6, 83.6},{36, 80.4},{36, 80.6},{36, 84.6},{36.2, 82.4},{36.6, 79.8},{36.8, 83.4},{37, 80.8},{37.4, 74.6},{37.6, 78.6},{38.4, 75.4},{38.4, 75.8},{38.4, 76.8},{38.6, 75.4},{38.6, 75.6},{38.8, 74.4},{39.8, 75.2},{40, 72.6},{40.4, 74.4},{40.6, 74.4},{40.8, 74.6},{41.4, 71},{41.4, 75.8},{41.6, 74.6},{41.6, 76.6},},
                [14] = {{36, 47.4},{36, 47.8},{37, 46.8},{40.6, 50.6},{40.8, 49.4},{41.2, 48.4},{41.2, 49.6},{41.8, 48.8},{54.4, 40.4},{55, 39.4},{55.2, 38.4},{55.8, 27.8},{55.8, 38.4},{56, 38.6},{56.4, 21.4},{56.4, 21.8},{56.4, 27},{56.6, 21.4},{56.6, 21.6},{56.6, 26.6},{56.6, 37.4},},
                [85] = {{78.2, 63.4},{78.4, 65},{79.4, 64.4},{79.4, 64.8},{79.6, 64.4},{79.8, 65.4},},
                [141] = {{39.6, 37.6},{39.8, 37.4},{39.8, 69.4},{39.8, 69.6},{43.8, 76.8},{54.8, 66},{62.4, 71.8},{62.6, 71.8},},
                [215] = {{38, 55},{40.4, 53},{40.6, 53},{40.6, 53.8},{45.4, 36.4},{45.4, 37},{45.6, 36.4},{46, 37.2},{52, 39.2},{52.4, 40.4},{60.2, 67.4},{60.2, 67.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203138] = {
            [npcKeys.name] = "Anvilrage Overseer",
            [npcKeys.minLevel] = 48,
            [npcKeys.maxLevel] = 49,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203139] = {
            [npcKeys.name] = "Son of Arugal",
            [npcKeys.minLevel] = 24,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{35.2, 16.2},{35.4, 19.2},{35.6, 18.8},{36, 16.4},{36, 18.2},{36.2, 19.8},{36.2, 22.2},{36.2, 28.6},{36.4, 15.2},{36.4, 20.6},{36.4, 24.8},{36.6, 15.6},{36.8, 27.8},{37.4, 15.4},{37.4, 16.8},{37.4, 24},{37.6, 15.8},{37.8, 25.8},{38, 14.2},{38, 24},{38, 28},{38.2, 16.8},{38.4, 14.8},{39, 17.2},{39.2, 15.2},{39.2, 16.4},{39.2, 26},{39.2, 27.2},{39.8, 15.4},{39.8, 18},{39.8, 27},{40, 16.4},{40, 28.2},{40.2, 30.8},{40.4, 16.6},{40.4, 29.2},{40.8, 17.2},{41, 19.4},{41, 19.6},{41.2, 29.4},{41.4, 29.6},{41.6, 18.2},{41.6, 21.6},{41.8, 20},{42.2, 20.8},{42.4, 30.2},{42.6, 21.4},{42.6, 28.6},{43, 22},{43, 29.6},{43.6, 19},{43.6, 21.4},{43.6, 21.8},{43.6, 28},{43.6, 29.6},{44.2, 78.4},{44.4, 18.2},{44.4, 19.8},{44.4, 29.2},{44.8, 17.8},{44.8, 20.8},{45, 29},{45, 29.8},{45, 81.6},{45.2, 17},{45.2, 81.2},{45.4, 19},{45.4, 28.2},{45.4, 34.2},{45.4, 83.2},{45.6, 16.8},{45.6, 29.2},{45.6, 82},{45.6, 83.4},{45.8, 21.4},{46, 27.4},{46, 79},{46, 83.6},{46.2, 18.8},{46.4, 26.2},{46.4, 27.6},{46.4, 32.2},{46.4, 33.6},{46.6, 19.4},{46.6, 25.2},{46.6, 81.2},{46.8, 33.4},{47, 17.8},{47, 20.6},{47, 26.8},{47, 83.8},{47.2, 32.2},{47.2, 83.4},{47.4, 20.2},{47.4, 26.2},{47.4, 34},{47.4, 34.6},{47.6, 31.8},{47.6, 33.4},{47.6, 33.6},{47.8, 19},{47.8, 31.4},{48, 19.6},{48.2, 26},{48.2, 34.6},{48.2, 38.4},{48.4, 20.8},{48.4, 25.4},{48.6, 31.6},{48.6, 34.8},{48.8, 26.6},{48.8, 31},{48.8, 72.4},{49, 18.8},{49, 26.2},{49, 75.6},{49.2, 20.8},{49.2, 22.6},{49.2, 25.4},{49.2, 33.2},{49.4, 36.4},{49.6, 23.4},{49.6, 33.4},{49.6, 82.4},{49.8, 17.2},{49.8, 33.6},{49.8, 76.4},{49.8, 83.2},{50, 25},{50.2, 79},{50.4, 36.6},{50.6, 34.8},{50.6, 35.8},{50.8, 79.2},{51, 78.4},{51.4, 36.8},{51.6, 76.4},{51.6, 77.6},{52, 37.8},{52, 72},{52.2, 75},{52.2, 77.4},{52.6, 76.2},{52.8, 74.2},{53.4, 76.6},{53.6, 75.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203147] = {
            [npcKeys.name] = "Whirlpool",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203218] = {
            [npcKeys.name] = "Giant Whirlpool",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203226] = {
            [npcKeys.name] = "Viktoria Woods",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{69.4, 50},{69.6, 50.6},{69.8, 50},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203279] = {
            [npcKeys.name] = "Riptide Essence",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203475] = {
            [npcKeys.name] = "Liv Bradford",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{22.4, 64.4},{22.4, 64.6},{22.6, 64.2},{22.6, 64.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203478] = {
            [npcKeys.name] = "Stuart",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{18.4, 56.8},{19.8, 57.4},{20.6, 57.6},{21.2, 61},{21.4, 57.4},{21.4, 58.6},{21.4, 60.4},{21.4, 61.8},{21.4, 62.8},{21.4, 63.6},{21.6, 62.6},{21.8, 60.2},{21.8, 62.4},{21.8, 64.6},{22, 53.8},{22, 54.6},{22, 56.6},{22, 58},{22, 58.6},{22.2, 51},{22.2, 52.2},{22.2, 60.8},{22.2, 63.6},{22.4, 55.6},{22.6, 54.2},{22.6, 56.8},{23, 56.4},{23, 63},{23.2, 53.2},{29.4, 51},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [203694] = {
            [npcKeys.name] = "Riding Lobster (Dead)",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204068] = {
            [npcKeys.name] = "Lady Sarevess",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204070] = {
            [npcKeys.name] = "Soboz",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{41.2, 36.2},{42, 36.6},{42.2, 35.4},{42.2, 35.6},{42.8, 35.8},},
                [14] = {{67.4, 87.8},{67.4, 88.6},{67.6, 87.4},{67.6, 87.8},{67.8, 88.6},},
                [1497] = {{21.6, 44.6},{22.2, 42.6},{22.6, 43.2},{23, 38.4},{23, 43.8},{23.2, 42.2},{23.4, 41.4},{23.6, 37.4},{23.6, 38.4},{23.6, 40.4},{23.8, 39.4},{24, 41.4},{24, 41.6},{24.6, 40.6},},
                [1519] = {{25, 77.6},{25.2, 77.4},{25.6, 78.2},{25.8, 76.2},{25.8, 76.8},{25.8, 78.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204091] = {
            [npcKeys.name] = "Blackfathom Elite",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204253] = {
            [npcKeys.name] = "Blackfathom Elite Spawner",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204256] = {
            [npcKeys.name] = "Damien Kane",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{44, 66.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204346] = {
            [npcKeys.name] = "Treant",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204503] = {
            [npcKeys.name] = "Dead Acolyte",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{56.4, 57.4},{56.4, 57.8},{56.6, 57.4},{56.6, 57.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204545] = {
            [npcKeys.name] = "Tim's Stalker 01",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204645] = {
            [npcKeys.name] = "Blackfathom Elite",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204675] = {
            [npcKeys.name] = "Rune Scrying Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204758] = {
            [npcKeys.name] = "Exploded Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204827] = {
            [npcKeys.name] = "Adventurer's Remains",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{43, 49.4},{43, 49.6},},
                [12] = {{52.2, 84.4},{52.2, 84.6},},
                [14] = {{48, 79.4},{48, 79.6},},
                [141] = {{33.6, 35.4},{33.6, 35.6},},
                [215] = {{60.4, 33.4},{60.4, 33.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204921] = {
            [npcKeys.name] = "Gelihast",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204937] = {
            [npcKeys.name] = "Adventurer's Spirit",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{43, 49.6},},
                [12] = {{52.2, 84.4},{52.2, 84.6},},
                [14] = {{48, 79.4},{48, 79.6},},
                [141] = {{33.4, 35.6},{33.6, 35.6},},
                [215] = {{60.2, 33.4},{60.4, 33.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204940] = {
            [npcKeys.name] = "Puddle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204960] = {
            [npcKeys.name] = "Chomp",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204961] = {
            [npcKeys.name] = "Snarly",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204962] = {
            [npcKeys.name] = "Tidelurker Mrrglrlr",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [204989] = {
            [npcKeys.name] = "Wounded Adventurer",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{61.8, 47},{61.8, 47.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205153] = {
            [npcKeys.name] = "Ada Gelhardt",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 10,
            [npcKeys.spawns] = {
                [10] = {{4.4, 28.2},{4.4, 28.6},{4.6, 28.2},{4.6, 28.6},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205278] = {
            [npcKeys.name] = "Brother Romulus",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{38.2, 28.2},{38.2, 29},{38.4, 27.4},{38.6, 27.4},{38.6, 28.4},{38.6, 28.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205382] = {
            [npcKeys.name] = "Mokwa",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{35, 57.6},{35.2, 57.2},{35.6, 57.6},{35.8, 57.2},{36, 56.2},{36.6, 56.6},{37.8, 55.6},{37.8, 56.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205383] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205422] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205423] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205635] = {
            [npcKeys.name] = "Takoda Sunmane",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{46.2, 60.2},{46.2, 60.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205692] = {
            [npcKeys.name] = "Rustling Bush",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{59, 54.4},{59, 54.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205700] = {
            [npcKeys.name] = "Venture Co. Poacher",
            [npcKeys.minLevel] = 6,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{58.2, 54},{58.8, 54.2},{59, 54.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205724] = {
            [npcKeys.name] = "Blackfathom Tendril",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205729] = {
            [npcKeys.name] = "Boarton Shadetotem",
            [npcKeys.minLevel] = 4,
            [npcKeys.maxLevel] = 4,
            [npcKeys.zoneID] = 1638,
            [npcKeys.spawns] = {
                [1638] = {{39.4, 64.4},{39.4, 65.2},{39.4, 65.6},{39.4, 66.6},{39.6, 64.4},{39.6, 65.4},{39.6, 65.6},{39.6, 66.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {76156,76160,76240},
            [npcKeys.questEnds] = {76156,76160,76240},
        },
        [205733] = {
            [npcKeys.name] = "Blackfathom Pearl",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205765] = {
            [npcKeys.name] = "Murloc Egg",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205767] = {
            [npcKeys.name] = "Void Murloc",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [205805] = {
            [npcKeys.name] = "Water Elemental",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206152] = {
            [npcKeys.name] = "SoD Test Dummy 25",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206153] = {
            [npcKeys.name] = "SoD Test Dummy 26",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206154] = {
            [npcKeys.name] = "SoD Test Dummy 27",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206155] = {
            [npcKeys.name] = "SoD Test Dummy 28",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206217] = {
            [npcKeys.name] = "Bubble Beam Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206245] = {
            [npcKeys.name] = "Unleashed Nature Spirit",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{66.4, 57.6},{66.6, 57.2},{67, 58},{67, 58.6},},
                [215] = {{37.2, 49.8},{37.4, 49.4},{37.6, 49.6},{37.8, 49.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [206248] = {
            [npcKeys.name] = "Wooden Effigy",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{66.8, 58},},
                [215] = {{37.4, 49.4},{37.4, 49.6},{37.8, 49.4},{37.8, 49.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207010] = {
            [npcKeys.name] = "Telemachus",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207356] = {
            [npcKeys.name] = "Lorgus Jett",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207358] = {
            [npcKeys.name] = "Blackfathom Tide Priestess",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207359] = {
            [npcKeys.name] = "Blackfathom Tide Priestess",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207367] = {
            [npcKeys.name] = "Blackfathom Tide Priestess",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207397] = {
            [npcKeys.name] = "Corrupted Windfury Totem",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207457] = {
            [npcKeys.name] = "Corrupted Lightning Shield Totem",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207515] = {
            [npcKeys.name] = "Lurkmane",
            [npcKeys.minLevel] = 6,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{30.6, 61.4},{30.8, 61.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207520] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207527] = {
            [npcKeys.name] = "Corrupted Molten Fury",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207576] = {
            [npcKeys.name] = "Corrupted Molten Fury Totem",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207577] = {
            [npcKeys.name] = "Lunar Stone",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{52.8, 79.8},},
                [215] = {{35, 69.4},{35.2, 69.6},{35.4, 68.2},{35.4, 70.6},{35.6, 70.4},{35.6, 70.6},{35.8, 68.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207588] = {
            [npcKeys.name] = "Mulgore Lunar Stone Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207637] = {
            [npcKeys.name] = "Vateya Timberhoof",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 215,
            [npcKeys.spawns] = {
                [215] = {{46.4, 61.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207743] = {
            [npcKeys.name] = "Netali Proudwind",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1638,
            [npcKeys.spawns] = {
                [1638] = {{28.4, 18.2},{28.4, 19.2},{28.6, 18.2},{28.6, 19.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207754] = {
            [npcKeys.name] = "Mooart",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1638,
            [npcKeys.spawns] = {
                [1638] = {{26.4, 18.4},{26.4, 18.8},{26.4, 19.8},{26.6, 19},{26.6, 19.8},{27, 18.4},{27.2, 14.4},{28, 21.4},{28.2, 22.4},{28.6, 23.2},{29.4, 21.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [207957] = {
            [npcKeys.name] = "Vahi Bonesplitter",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{53, 43.4},{53, 43.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208023] = {
            [npcKeys.name] = "Gru'ark",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{49.6, 65},{53.6, 57.4},{55.8, 57},{56.4, 58},{57, 44.6},{57, 56},{57.6, 57},{58, 48.4},{58, 48.8},{58, 55.2},{58.2, 51.2},{58.2, 54.4},{58.4, 50.2},{58.4, 52},{58.4, 52.6},{58.4, 56.2},{58.6, 51.4},{58.6, 51.8},{58.8, 53.4},{58.8, 53.6},{61.4, 44.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208036] = {
            [npcKeys.name] = "Ba'so",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{51.8, 58.6},{52, 58.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208071] = {
            [npcKeys.name] = "Back-alley Mugger",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{55.8, 44.6},{56.2, 42.8},{56.4, 43.6},{56.6, 44.2},{58.8, 50.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208124] = {
            [npcKeys.name] = "Raluk",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{67.6, 73},{67.8, 68.8},{68.4, 71.4},{68.4, 71.6},{68.4, 74.2},{68.6, 71.4},{68.6, 71.6},{71, 72},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208127] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208128] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208158] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208179] = {
            [npcKeys.name] = "Rustling Bush",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{40.4, 52},{40.6, 52},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208180] = {
            [npcKeys.name] = "Razormane Poacher",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{28.8, 49.8},{29, 49.4},{29.6, 50.6},},
                [14] = {{40.4, 51.4},{40.4, 52},{40.6, 51.4},{40.6, 52},{40.6, 52.6},{41, 50.4},{41.6, 51},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208184] = {
            [npcKeys.name] = "Razzil",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{52.2, 44},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208196] = {
            [npcKeys.name] = "Gillgar",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 85,
            [npcKeys.spawns] = {
                [85] = {{25.4, 47.4},{25.4, 48.4},{25.4, 48.6},{25.6, 48.2},{25.6, 48.6},{26.2, 47.4},{26.4, 46.4},{26.6, 46.8},{27.2, 46.4},{27.6, 46.4},{28.4, 46.8},{29, 46.6},{29.6, 46.6},{29.8, 46},{30.4, 49.4},{31, 48},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208226] = {
            [npcKeys.name] = "Darmak Bloodhowl",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{54.6, 41.4},{54.6, 41.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208275] = {
            [npcKeys.name] = "Frozen Makrura",
            [npcKeys.minLevel] = 9,
            [npcKeys.maxLevel] = 9,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{58.6, 45.4},{58.6, 45.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208307] = {
            [npcKeys.name] = "Serpent Loa",
            [npcKeys.minLevel] = 60,
            [npcKeys.maxLevel] = 60,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{55.4, 72.4},{55.4, 72.6},},
                [17] = {{51.4, 29.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208309] = {
            [npcKeys.name] = "Loa Altar",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 14,
            [npcKeys.spawns] = {
                [14] = {{55.4, 72.4},{55.4, 72.6},},
                [17] = {{51.4, 29.4},{51.6, 29.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208518] = {
            [npcKeys.name] = "Gaeriyan",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 440,
            [npcKeys.spawns] = {
                [440] = {{54, 23.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208546] = {
            [npcKeys.name] = "Brazier of Embersight",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208565] = {
            [npcKeys.name] = "Altar of the Light",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{28.8, 66.4},{28.8, 66.6},},
                [38] = {{37.4, 46.2},{37.6, 46.2},},
                [40] = {{52.6, 52.4},{52.8, 52.8},},
                [1537] = {{31.2, 21.4},{31.4, 21.6},{32.2, 20.6},{32.2, 21.8},{32.6, 21.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208619] = {
            [npcKeys.name] = "Dorac Graves",
            [npcKeys.minLevel] = 50,
            [npcKeys.maxLevel] = 50,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{47.2, 70.4},{47.2, 70.8},{47.8, 70.2},{47.8, 71.6},{48, 70.6},{48.4, 69.4},{48.6, 70.6},{48.6, 71.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208638] = {
            [npcKeys.name] = "Fyodi",
            [npcKeys.minLevel] = 7,
            [npcKeys.maxLevel] = 7,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{28.4, 38.6},{28.4, 42.8},{28.4, 43.6},{28.4, 44.6},{28.6, 43.8},{28.8, 40.2},{28.8, 42.4},{28.8, 44.8},{29.2, 38.4},{29.2, 38.8},{29.4, 42.6},{29.6, 38.4},{29.6, 42.2},{29.6, 42.8},{29.6, 45.6},{30, 39},{30.2, 43.6},{30.4, 36.6},{30.4, 40.4},{30.4, 41.2},{30.6, 37.4},{30.8, 39.4},{30.8, 39.6},{31.6, 38.6},{31.6, 40},{32.2, 36},{32.2, 37.6},{32.4, 37.2},{32.6, 36.8},{33.2, 35.6},{33.4, 34},{33.4, 35.2},{33.4, 37.6},{33.6, 35},{33.6, 36},{33.8, 34},{33.8, 37.2},{35.2, 32.8},{35.2, 34.2},{35.2, 35},{35.2, 36.4},{35.2, 36.6},{35.4, 38},{35.6, 33.2},{35.8, 33.8},{35.8, 36.4},{35.8, 37},{36.6, 36.6},{37.6, 35.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208652] = {
            [npcKeys.name] = "Junni Steelpass",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{46.4, 53.2},{46.6, 53.4},{46.6, 53.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208682] = {
            [npcKeys.name] = "Denton Bleakway",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{83.4, 25.4},{83.4, 25.8},{83.4, 26.6},{83.4, 27.6},{83.6, 24.8},{83.6, 26.2},{83.6, 27.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208711] = {
            [npcKeys.name] = "Toby",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{63.4, 50.2},{63.6, 50.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208712] = {
            [npcKeys.name] = "Odd Melon",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 85,
            [npcKeys.spawns] = {
                [85] = {{30.2, 47.2},{33.4, 49.4},{33.4, 49.8},{33.6, 49.4},{33.6, 49.8},{34.8, 51.2},{35.4, 49},{35.6, 49},{36.4, 50.8},{36.8, 51},{40.2, 42},{44, 38.6},{44.2, 38.4},{44.8, 28.4},{44.8, 28.6},{45.2, 33.4},{47.2, 28.8},{47.2, 50.2},{47.2, 50.8},{49.4, 46.2},{49.4, 46.8},{49.6, 46.4},{49.6, 46.6},{49.8, 50.4},{49.8, 59.4},{50, 59.6},{50.2, 50.8},{50.4, 31},{50.4, 61.8},{50.6, 31},{51.4, 57.2},{51.6, 57.2},{51.6, 57.6},{52.2, 30.2},{52.6, 30},{52.8, 57.6},{53.8, 28},{53.8, 58.6},{53.8, 59.8},{54, 56.4},{54, 56.8},{54, 58.2},{57.4, 40.4},{57.4, 40.6},{57.6, 40.2},{58.2, 35.4},{58.4, 35.6},{58.6, 35.2},{58.8, 58.4},{58.8, 58.6},{59.8, 33},{60, 37},{65.4, 62.8},{65.6, 63},{71.4, 34},{71.8, 50.6},{72.2, 50.4},{74.8, 61.6},{75, 61.4},{76, 51.4},{76, 59.4},{76, 59.6},{76.2, 51.8},{76.4, 61.4},{76.4, 61.6},{76.6, 61.4},{76.6, 61.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208752] = {
            [npcKeys.name] = "Frozen Trogg",
            [npcKeys.minLevel] = 9,
            [npcKeys.maxLevel] = 9,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{69.2, 58.2},{69.4, 58.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208758] = {
            [npcKeys.name] = "Junior Apothecary",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208802] = {
            [npcKeys.name] = "Wounded Adventurer",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{25.4, 43},{25.4, 43.6},{25.6, 43.4},{25.6, 43.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208809] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208810] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208811] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208812] = {
            [npcKeys.name] = "Jorul",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{35.4, 43.8},{35.8, 42.6},{36.4, 38.8},{36.4, 40.4},{36.6, 40.4},{37, 41},{37.2, 42.4},{37.4, 42.6},{37.6, 41.4},{37.8, 42.4},{38.4, 43.4},{38.4, 43.6},{38.6, 42},{38.6, 43.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208841] = {
            [npcKeys.name] = "Reverent Blacksmith",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208842] = {
            [npcKeys.name] = "The Gardener",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208843] = {
            [npcKeys.name] = "Abominable Snowman",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208844] = {
            [npcKeys.name] = "Crimson Emissary",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208845] = {
            [npcKeys.name] = "Par'kourc",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208846] = {
            [npcKeys.name] = "Sun Admirer",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208847] = {
            [npcKeys.name] = "Bloodrend Bandagethief",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208875] = {
            [npcKeys.name] = "Back-alley Mugger",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{55.8, 44.6},{56, 43.8},{56.2, 42.8},{57, 44.2},{58.4, 49.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208876] = {
            [npcKeys.name] = "Cut-throat Mugger",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{59, 27.8},{61.4, 29.4},{61.4, 29.6},{61.8, 29.2},{62.4, 29.6},{62.8, 29.6},{63.2, 29.2},},
                [1537] = {{50.2, 11.6},{50.6, 10.2},{51, 12.2},{51.4, 11},{51.8, 12.4},{52, 12.6},{52.8, 9.2},{52.8, 10.4},{57, 12},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208886] = {
            [npcKeys.name] = "Blackrat",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{57.2, 45.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208919] = {
            [npcKeys.name] = "Blueheart",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 85,
            [npcKeys.spawns] = {
                [85] = {{59.8, 51.8},{60.6, 52.2},{61, 53.2},{61.2, 53.6},{61.6, 51.4},{61.6, 52.8},{61.8, 51.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208920] = {
            [npcKeys.name] = "Penny Hawkins",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 85,
            [npcKeys.spawns] = {
                [85] = {{61.8, 52},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208927] = {
            [npcKeys.name] = "Dead Acolyte",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 85,
            [npcKeys.spawns] = {
                [85] = {{76.4, 44.8},{76.6, 44.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208974] = {
            [npcKeys.name] = "Dark Iron Poacher",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [208975] = {
            [npcKeys.name] = "Rustling Bush",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{28.8, 49.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209002] = {
            [npcKeys.name] = "Gaklik Voidtwist",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{47.2, 53.4},{47.2, 53.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209004] = {
            [npcKeys.name] = "Bruart",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 1537,
            [npcKeys.spawns] = {
                [1537] = {{59.4, 75.8},{60.8, 70.4},{63.6, 73},{65, 70.4},{65.2, 74.2},{65.4, 77},{66.4, 69.6},{66.6, 71.2},{66.6, 73.4},{66.6, 77},{66.8, 75.2},{67, 71.6},{67.2, 77.8},{67.6, 73},{67.6, 74.8},{68, 67.4},{68.8, 71.8},{69, 71},{69.2, 69.2},{69.4, 68.4},{69.8, 72},{70, 74.4},{70.2, 72.8},{70.8, 72.4},{71.2, 73.2},{71.2, 74.8},{71.2, 75.8},{72, 73.4},{72, 75.2},{72, 75.6},{72.2, 74},{72.2, 76.8},{72.6, 73.8},{72.6, 76},{73, 74.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209209] = {
            [npcKeys.name] = "Blackfathom Elite",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209213] = {
            [npcKeys.name] = "Murloc Spawner",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209214] = {
            [npcKeys.name] = "Blackfathom Murloc",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209511] = {
            [npcKeys.name] = "Oasis Snapjaw Hatchling",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{48.2, 40.2},{55.8, 42.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209524] = {
            [npcKeys.name] = "Patrolling Cheetah",
            [npcKeys.minLevel] = 18,
            [npcKeys.maxLevel] = 18,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{43.2, 56.6},{43.4, 56.4},{43.6, 56.6},{44, 56},{44.4, 55.4},{44.6, 55},{44.6, 55.6},{50.2, 44.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209548] = {
            [npcKeys.name] = "Secret Agent",
            [npcKeys.minLevel] = 77,
            [npcKeys.maxLevel] = 77,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{50.4, 79.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209607] = {
            [npcKeys.name] = "Lieutenant Stonebrew",
            [npcKeys.minLevel] = 17,
            [npcKeys.maxLevel] = 17,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{61.4, 54.8},{62.4, 56.2},{62.6, 56.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209608] = {
            [npcKeys.name] = "Delwynna",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{63.2, 23.2},{63.4, 20.6},{63.4, 22},{63.6, 22},{64, 21.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209678] = {
            [npcKeys.name] = "Twilight Lord Kelris",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209742] = {
            [npcKeys.name] = "Desert Mirage",
            [npcKeys.minLevel] = 15,
            [npcKeys.maxLevel] = 15,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{54.4, 36},{54.8, 35.8},{55, 35.4},{55.8, 34},{57.4, 35.8},{57.4, 38.4},{57.4, 38.6},{57.6, 35.4},{57.6, 35.8},{57.6, 38.4},{57.6, 38.6},{58.8, 37.4},{58.8, 37.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209758] = {
            [npcKeys.name] = "Invading Nightmare",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209773] = {
            [npcKeys.name] = "Dream Copy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209797] = {
            [npcKeys.name] = "Bruuz",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{63.2, 38.6},{63.6, 38.4},{64.4, 39},{64.4, 39.6},{65, 37.2},{65.2, 39.4},{65.2, 39.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209806] = {
            [npcKeys.name] = "Waterfall Test",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209811] = {
            [npcKeys.name] = "Rustling Bush",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{46.4, 46.2},{46.6, 46.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209812] = {
            [npcKeys.name] = "Fallenroot Poacher",
            [npcKeys.minLevel] = 9,
            [npcKeys.maxLevel] = 9,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{43.2, 42.6},{45.2, 46.4},{45.2, 47.2},{45.4, 49.4},{45.4, 49.6},{46, 49.6},{46.2, 45.4},{46.2, 46.6},{46.2, 47.8},{46.2, 49},{46.4, 44.2},{46.4, 46.2},{46.6, 45.4},{46.6, 46.2},{46.6, 46.6},{46.6, 49},{46.8, 43.6},{47, 49.6},{47.6, 44.4},{47.6, 50.2},{48, 51},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209815] = {
            [npcKeys.name] = "Phantasmal Priestess",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209830] = {
            [npcKeys.name] = "Phantasmal Statue",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209872] = {
            [npcKeys.name] = "Syllart",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{56.2, 59.4},{56.2, 59.6},{56.4, 58.4},{56.6, 57.8},{56.8, 58.8},{57.2, 60.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209889] = {
            [npcKeys.name] = "Barebones",
            [npcKeys.minLevel] = 63,
            [npcKeys.maxLevel] = 63,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{56, 60.4},{56, 61.6},{56.2, 60.8},{56.6, 61.2},},
                [1637] = {{51.6, 76.8},{51.8, 75.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209908] = {
            [npcKeys.name] = "Heretic Idol",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 38,
            [npcKeys.spawns] = {
                [38] = {{71.4, 27.2},{71.8, 27},{71.8, 27.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209928] = {
            [npcKeys.name] = "Mowgh",
            [npcKeys.minLevel] = 8,
            [npcKeys.maxLevel] = 8,
            [npcKeys.zoneID] = 141,
            [npcKeys.spawns] = {
                [141] = {{46.4, 33.8},{47.4, 32.4},{47.4, 32.8},{47.4, 33.6},{47.6, 32.6},{48, 31.6},{48.2, 31.4},{48.6, 31.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209939] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209940] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209941] = {
            [npcKeys.name] = "Hunter's Vision",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209948] = {
            [npcKeys.name] = "Relaeron",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{39.2, 9},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209949] = {
            [npcKeys.name] = "Sickly Deer",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{39.2, 8.4},{39.2, 9},{39.4, 10.2},{39.6, 9.2},{39.6, 9.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209954] = {
            [npcKeys.name] = "Demonic Remains",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 38,
            [npcKeys.spawns] = {
                [38] = {{72.4, 68.4},{72.4, 68.6},{72.6, 68.4},{72.6, 68.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [209958] = {
            [npcKeys.name] = "Graix",
            [npcKeys.minLevel] = 18,
            [npcKeys.maxLevel] = 18,
            [npcKeys.zoneID] = 38,
            [npcKeys.spawns] = {
                [38] = {{71.2, 68.2},{72.4, 68.4},{72.4, 68.8},{72.6, 67.8},{72.6, 68.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210006] = {
            [npcKeys.name] = "Battle Totem",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210107] = {
            [npcKeys.name] = "Kackle",
            [npcKeys.minLevel] = 15,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 38,
            [npcKeys.spawns] = {
                [38] = {{50.2, 53.8},{52.4, 54.4},{52.4, 55.8},{52.8, 56.8},{53, 54},{53.2, 55.4},{53.4, 53.4},{53.4, 56.2},{53.8, 55.8},{54.2, 53.2},{54.2, 54},{54.2, 55.2},{54.2, 56.8},{54.6, 52.2},{54.6, 55.6},{54.8, 53.6},{54.8, 54.6},{54.8, 56.6},{55, 52.6},{55.6, 52.2},{55.6, 54.4},{55.6, 55},{55.6, 57},{55.8, 53.2},{55.8, 57.6},{56.4, 55.6},{57.2, 64.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210451] = {
            [npcKeys.name] = "Lady Sedorax",
            [npcKeys.minLevel] = 18,
            [npcKeys.maxLevel] = 18,
            [npcKeys.zoneID] = 148,
            [npcKeys.spawns] = {
                [148] = {{54.8, 31.6},{55, 32.6},{55.2, 33.8},{55.2, 35.2},{55.4, 36.2},{55.4, 36.8},{55.6, 35.2},{55.6, 36.4},{55.6, 36.6},{55.8, 34.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210482] = {
            [npcKeys.name] = "Paxnozz",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 148,
            [npcKeys.spawns] = {
                [148] = {{46.4, 14.4},{46.4, 17},{46.6, 13.8},{46.6, 17},{47, 15.6},{47.4, 14.6},{47.8, 14.8},{48, 13.4},{48, 14},{48, 16},{48, 18},{48.4, 16.6},{48.6, 16.6},{48.8, 15.6},{49, 11.8},{49, 13.2},{49, 14.8},{49.4, 14.2},{49.6, 14.4},{50, 11.8},{50, 14.6},{50.2, 12.6},{50.4, 11.4},{50.8, 12.4},{51, 13.6},{51.2, 13.4},{51.8, 13.2},{51.8, 15.6},{52.4, 17.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210483] = {
            [npcKeys.name] = "Aggressive Squashling",
            [npcKeys.minLevel] = 16,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{36.6, 48.8},{36.6, 52.8},{39.2, 55.2},{43.2, 34.2},{43.2, 35.2},{43.6, 33.8},{43.8, 33.4},{43.8, 38},{44.2, 32.4},{44.8, 31.8},{48.8, 22},{49, 22.6},{49.8, 20.2},{50.2, 24},{50.6, 18.8},{51.4, 24.8},{51.4, 26},{52.4, 30.4},{54.8, 35.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210487] = {
            [npcKeys.name] = "Horror of the Deep",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{25.2, 65.6},{25.2, 69},{26, 69.4},{26, 69.6},{26.2, 66.6},{26.2, 68},{26.4, 66},{26.8, 66.2},{26.8, 68},{26.8, 69.8},{27.2, 68.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210501] = {
            [npcKeys.name] = "Harvest Reaper Prototype",
            [npcKeys.minLevel] = 18,
            [npcKeys.maxLevel] = 18,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{34.4, 52.6},{35.2, 51.4},{35.4, 51.6},{35.6, 51.4},{35.6, 52.2},{44.4, 39.2},{45, 40.8},{45.2, 39.6},{45.4, 38.8},{45.6, 39},{49.6, 36.6},{50, 18.8},{50.2, 17.4},{50.4, 18.4},{50.6, 18.4},{50.6, 18.8},{50.8, 20.4},{51.4, 33.4},{51.8, 34},{52.2, 32.4},{52.4, 32.6},{52.8, 33},{52.8, 33.8},{58.8, 54},{61.2, 57.4},{61.2, 58.2},{61.4, 58.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210528] = {
            [npcKeys.name] = "Terrified Soul",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210533] = {
            [npcKeys.name] = "Silverspur",
            [npcKeys.minLevel] = 14,
            [npcKeys.maxLevel] = 14,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{33.6, 40.8},{34.4, 37.2},{34.8, 38.6},{35, 38.4},{35, 39.6},{35.6, 38.4},{36, 38.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210537] = {
            [npcKeys.name] = "Undying Laborer",
            [npcKeys.minLevel] = 14,
            [npcKeys.maxLevel] = 15,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{28.8, 49.4},{29, 50.6},{29.2, 41.4},{29.2, 50.2},{29.4, 46.4},{29.6, 50.6},{29.8, 46},{29.8, 46.6},{29.8, 49.8},{30.2, 49.2},{30.4, 42.6},{30.4, 45.4},{30.4, 47.8},{30.6, 46},{30.6, 48.4},{30.8, 43},{31.2, 44.8},{31.4, 44.2},{31.6, 43.8},{31.8, 43.4},{31.8, 45},{32.2, 42.4},{32.6, 43.2},{34.8, 41.6},{35.2, 40.8},{36.2, 44},{37.8, 45.4},{42.2, 32.4},{43.4, 26.8},{43.8, 25.2},{44.4, 21.2},{44.4, 24},{44.4, 25.6},{44.4, 27},{44.4, 29.4},{44.6, 19.4},{44.6, 20.2},{44.6, 24.2},{44.6, 24.8},{44.8, 29},{45, 22.8},{45, 26},{45, 26.6},{45.2, 31.2},{45.4, 18.2},{45.4, 21.2},{45.4, 21.6},{45.4, 28},{45.6, 20.4},{45.6, 21.4},{45.6, 21.8},{45.8, 27.4},{45.8, 36.4},{46, 18.4},{46, 26.4},{46.2, 18.8},{46.2, 28.2},{46.6, 19.4},{46.6, 27.4},{46.8, 29.6},{47.6, 33.6},{50.4, 28.2},{50.6, 28},{50.6, 29.2},{53.4, 27.6},{54.4, 29.6},{55.2, 31.4},{55.8, 47.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210549] = {
            [npcKeys.name] = "Defias Scout",
            [npcKeys.minLevel] = 14,
            [npcKeys.maxLevel] = 15,
            [npcKeys.zoneID] = 40,
            [npcKeys.spawns] = {
                [40] = {{49.4, 47.4},{50.2, 48.6},{50.4, 47.4},{50.4, 47.6},{50.4, 56.2},{50.6, 47.6},{50.8, 54.4},{51.2, 46.8},{51.4, 55.4},{51.4, 55.6},{51.6, 55.4},{51.6, 55.6},{51.8, 46.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210697] = {
            [npcKeys.name] = "Captain Namier",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{36.6, 26.6},{37.4, 27.6},{37.4, 28.6},{38, 28.8},{38, 29.6},{38.2, 28.4},{38.4, 30.6},{38.6, 28.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210750] = {
            [npcKeys.name] = "M'grrgl",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210802] = {
            [npcKeys.name] = "Webbed Victim",
            [npcKeys.minLevel] = 16,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{34.8, 7.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210845] = {
            [npcKeys.name] = "Jixo Madrocket",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 406,
            [npcKeys.spawns] = {
                [406] = {{59.2, 62.4},{59.2, 62.6},{60.6, 62.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {78114,78121},
            [npcKeys.questEnds] = {78114,78121},
        },
        [210887] = {
            [npcKeys.name] = "Unsuspecting Pridewing",
            [npcKeys.minLevel] = 19,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 406,
            [npcKeys.spawns] = {
                [406] = {{60.4, 62.2},{60.6, 62.4},{60.6, 62.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210901] = {
            [npcKeys.name] = "Stonetalon Wyvern stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [210995] = {
            [npcKeys.name] = "Alonso",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{42, 69.2},{42.4, 70},{43.4, 70.4},{43.4, 70.6},{43.6, 70.4},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {78132,78133,78134,78144},
            [npcKeys.questEnds] = {78132,78133,78134,78144},
        },
        [211022] = {
            [npcKeys.name] = "Owen Thadd",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{73.2, 33.6},{73.4, 32.4},{73.4, 33},{73.6, 33},{73.6, 33.6},{74, 32.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79094,79095,79096},
            [npcKeys.questEnds] = {78124,78127,78142,78143,78145,78146,78147,78148,78149,78150,79094,79095,79096,79097,79535,79536,79947,79948,79949,79950,79951,79952,79953},
        },
        [211033] = {
            [npcKeys.name] = "Garion Wendell",
            [npcKeys.minLevel] = 60,
            [npcKeys.maxLevel] = 60,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{37.8, 80.2},{37.8, 80.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78124,78127,78142,78143,78145,78146,78147,78148,78149,78150,79091,79092,79093,79097,79535,79536,79905,79947,79948,79949,79950,79951,79952,79953},
            [npcKeys.questEnds] = {78124,78127,78142,78143,78145,78146,78147,78148,78149,78150,79091,79092,79093,79097,79535,79536,79905,79947,79948,79949,79950,79951,79952,79953},
        },
        [211042] = {
            [npcKeys.name] = "Green Dragon Whelp",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{42, 69.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211043] = {
            [npcKeys.name] = "Alonso",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211146] = {
            [npcKeys.name] = "Lost Adventurer",
            [npcKeys.minLevel] = 16,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{34.2, 9.4},{34.4, 8.4},{35, 7.8},{35.4, 11.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211183] = {
            [npcKeys.name] = "Secret Agent",
            [npcKeys.minLevel] = 77,
            [npcKeys.maxLevel] = 77,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{50.2, 79.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211188] = {
            [npcKeys.name] = "Maethra Slagheart",
            [npcKeys.minLevel] = 50,
            [npcKeys.maxLevel] = 50,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{37, 88.8},{38, 88},{38, 89.2},{38, 89.6},{38.6, 88.8},{38.6, 89.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78192,78193},
            [npcKeys.questEnds] = {78192},
        },
        [211200] = {
            [npcKeys.name] = "Agon",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 10,
            [npcKeys.spawns] = {
                [10] = {{60.2, 31.8},{60.2, 33},{60.4, 30.8},{60.6, 25.8},{60.6, 32.6},{60.8, 32.4},{61, 30.6},{61.2, 30},{61.4, 29.2},{61.6, 28.2},{61.6, 29},{61.6, 30.4},{61.6, 31.8},{62, 31.2},{62.2, 25.4},{62.6, 25.8},{62.6, 31.8},{62.8, 35.8},{63, 31.2},{63, 34.6},{63.4, 25},{63.4, 29},{63.6, 31.4},{64, 32.4},{64, 34},{64, 35},{64.2, 33},{64.2, 37.8},{64.4, 23.6},{64.4, 24.6},{64.4, 26.4},{64.4, 35.6},{64.4, 36.8},{64.6, 36.8},{64.8, 24.8},{64.8, 34.4},{64.8, 35.6},{65, 23.8},{65, 32.4},{65.2, 21.4},{65.2, 21.6},{65.2, 23.2},{65.2, 27.8},{65.2, 34.8},{65.4, 26},{65.4, 26.8},{65.4, 30},{65.6, 24},{65.6, 25.2},{65.6, 26.4},{65.6, 34.8},{66, 27},{66, 32.6},{66.4, 23.4},{66.4, 27.8},{66.4, 29.2},{66.4, 29.8},{66.6, 30.6},{66.6, 33},{66.8, 23.4},{66.8, 28.4},{66.8, 29.8},{67.2, 29.4},{67.4, 27.2},{67.6, 30.2},{67.6, 30.6},{67.6, 32},{67.8, 32.8},{68.6, 28},{68.8, 32.4},{69.4, 32.6},{69.6, 29.4},{69.6, 30.2},{70.2, 33.4},{70.8, 31.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211225] = {
            [npcKeys.name] = "Baj'ura",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{46, 20},{46.4, 19.4},{46.4, 20.6},{47, 19.4},{47, 19.6},{47.8, 19.2},{48, 19.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78198,78199},
            [npcKeys.questEnds] = {78198},
        },
        [211229] = {
            [npcKeys.name] = "Dietrich Praice",
            [npcKeys.minLevel] = 60,
            [npcKeys.maxLevel] = 60,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{35.4, 88},{35.6, 88},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78196,78197},
            [npcKeys.questEnds] = {78196},
        },
        [211269] = {
            [npcKeys.name] = "Summoned Wisp",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{86.8, 43.4},{86.8, 43.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211274] = {
            [npcKeys.name] = "Ashenvale Wisp Target",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211279] = {
            [npcKeys.name] = "Corrupt Spirit Beast",
            [npcKeys.minLevel] = 23,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{86.6, 43.6},{86.8, 43.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211298] = {
            [npcKeys.name] = "Syrnoya",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 148,
            [npcKeys.spawns] = {
                [148] = {{37.6, 43.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {78195},
        },
        [211330] = {
            [npcKeys.name] = "Suspicious Target",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{65.8, 11.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211338] = {
            [npcKeys.name] = "Vile Fin Hunter",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{65.8, 11.2},{65.8, 11.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211653] = {
            [npcKeys.name] = "Grizzby",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{61.8, 39.4},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {78265,78266,78267,78287,78288},
            [npcKeys.questEnds] = {78265,78266,78267,78297,78304},
        },
        [211736] = {
            [npcKeys.name] = "Grizzled Protector",
            [npcKeys.minLevel] = 16,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{36.6, 16},{36.6, 17.2},{37.8, 15.6},{39.4, 15.2},{40.4, 16.8},{40.6, 18},{41.6, 19.6},{42, 24},{42.2, 21.8},{42.4, 18.6},{44, 51.6},{44.2, 16.6},{44.6, 48.8},{45, 24.6},{47.2, 19},{47.2, 20.6},{47.4, 32.6},{48, 22.6},{48.2, 20.8},{48.6, 21.4},{48.6, 33.4},{48.8, 37.8},{49.4, 21.6},{49.6, 25.8},{49.8, 19},{49.8, 22.4},{50.4, 16.2},{50.4, 23.4},{50.4, 38.2},{50.6, 36.6},{50.6, 67},{50.8, 35.4},{51, 24.4},{51, 38.2},{51, 38.6},{51.2, 19.2},{51.2, 41.2},{51.2, 45.2},{51.4, 16},{51.6, 37.8},{51.6, 38.8},{51.8, 17.6},{51.8, 19.6},{52, 51.2},{52, 57.8},{52.6, 47.4},{52.6, 48},{53, 20.8},{53.2, 20.2},{53.4, 44.4},{53.4, 59},{53.8, 45.8},{54, 39.8},{54, 44.2},{54.6, 38.6},{54.6, 40.4},{55, 41.2},{55.8, 43},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211764] = {
            [npcKeys.name] = "Gefell",
            [npcKeys.minLevel] = 21,
            [npcKeys.maxLevel] = 22,
            [npcKeys.zoneID] = 209,
            [npcKeys.spawns] = {
                [209] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211765] = {
            [npcKeys.name] = "Gemela",
            [npcKeys.minLevel] = 21,
            [npcKeys.maxLevel] = 22,
            [npcKeys.zoneID] = 209,
            [npcKeys.spawns] = {
                [209] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211839] = {
            [npcKeys.name] = "Test Boss",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211875] = {
            [npcKeys.name] = "Decayed Elemental",
            [npcKeys.minLevel] = 15,
            [npcKeys.maxLevel] = 15,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{43.2, 20.6},{44.4, 21.4},{44.4, 21.8},{44.4, 42},{45, 21.6},{45, 22.6},{45.2, 21},{45.4, 20},{45.6, 22.6},{45.8, 21.2},{46, 19.8},{46, 22.2},{46.2, 19.4},{47, 19.2},{47.2, 20.4},{55.4, 15.6},{67.2, 24.6},},
                [1637] = {{37, 76.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211941] = {
            [npcKeys.name] = "Dream Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211951] = {
            [npcKeys.name] = "Koartul",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 267,
            [npcKeys.spawns] = {
                [267] = {{50.6, 50.2},{51.2, 41},{52.4, 39},{53.6, 43},{55.8, 59.6},{57.2, 22.8},{58.8, 26.4},{59, 23.4},{59, 34.6},{59.2, 38.6},{59.4, 32.8},{59.4, 40.2},{59.8, 28.8},{59.8, 36.2},{60.2, 33},{60.2, 33.8},{60.2, 35.2},{60.4, 20.4},{60.4, 26},{60.4, 28},{60.6, 27.8},{60.8, 25},{60.8, 25.6},{60.8, 27.4},{60.8, 35.2},{61, 30},{61, 33.2},{61.4, 30.6},{61.4, 34},{61.6, 33.6},{61.8, 33.4},{62, 31.6},{62.2, 31.4},{62.2, 34.6},{62.4, 27},{62.6, 34},{64, 33.8},{64.6, 32.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211952] = {
            [npcKeys.name] = "Omni Diffusion Armor",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211953] = {
            [npcKeys.name] = "Bulky Protective Ironweave",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211954] = {
            [npcKeys.name] = "Electrostatic Discharge Shielding",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211955] = {
            [npcKeys.name] = "Auxiliary Anti-Tamper Mechanism",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211956] = {
            [npcKeys.name] = "Scarimous the Wandering",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211965] = {
            [npcKeys.name] = "Carrodin",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 11,
            [npcKeys.spawns] = {
                [11] = {{46.4, 64.4},{46.4, 65},{46.4, 65.6},{46.8, 63.4},{47, 65.6},{47.2, 64.4},{47.2, 64.8},{47.6, 63.2},{47.6, 64},{47.6, 64.8},{47.6, 66},{48.4, 60.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [211967] = {
            [npcKeys.name] = "Leech Stalker Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212159] = {
            [npcKeys.name] = "Old Serra'kis",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212186] = {
            [npcKeys.name] = "Grugimdern",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 11,
            [npcKeys.spawns] = {
                [11] = {{36.8, 15.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212209] = {
            [npcKeys.name] = "Vodyanoi",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 11,
            [npcKeys.spawns] = {
                [11] = {{31.2, 18.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212252] = {
            [npcKeys.name] = "Harvest Golem V000-A",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212261] = {
            [npcKeys.name] = "Awakened Lich",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 10,
            [npcKeys.spawns] = {
                [10] = {{14.8, 37.2},{16, 37.4},{16.2, 38.6},{17.4, 37.4},{17.4, 37.6},{17.6, 37.4},{18, 38},{18, 38.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212330] = {
            [npcKeys.name] = "Explrgl",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212333] = {
            [npcKeys.name] = "Blackfathom Tendril",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212334] = {
            [npcKeys.name] = "Dead Twilight Cultist",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{17.2, 26.4},{17.2, 26.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212443] = {
            [npcKeys.name] = "Nightmare Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212458] = {
            [npcKeys.name] = "Phantasmal Crustacean",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212461] = {
            [npcKeys.name] = "Phantasmal Servant",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212462] = {
            [npcKeys.name] = "Phantasmal Softshell",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212463] = {
            [npcKeys.name] = "Phantasmal Snapjaw",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212468] = {
            [npcKeys.name] = "Invading Nightmare",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212598] = {
            [npcKeys.name] = "Recke Grinnes",
            [npcKeys.minLevel] = 16,
            [npcKeys.maxLevel] = 16,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{66.2, 31.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212667] = {
            [npcKeys.name] = "Fettered Elemental",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212674] = {
            [npcKeys.name] = "Death's Head Shaman",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212678] = {
            [npcKeys.name] = "Earth Elemental",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212692] = {
            [npcKeys.name] = "Bound Elemental",
            [npcKeys.minLevel] = 21,
            [npcKeys.maxLevel] = 21,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{41.4, 79.8},{42.2, 80.8},{42.4, 79.4},{43, 78.6},{43, 82.2},{43.2, 78.4},{43.2, 81.4},{43.6, 78.4},{43.6, 78.8},{43.8, 82.2},{44, 91.2},{44.2, 80.2},{44.4, 88.8},{44.8, 76.8},{44.8, 77.8},{44.8, 78.6},{44.8, 85.8},{45, 79.6},{45.2, 63.8},{45.2, 74.2},{45.8, 80.4},{45.8, 82.8},{46, 69},{46, 74},{46, 75.2},{46, 76.4},{46, 76.6},{47.6, 81.6},{47.8, 83.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212694] = {
            [npcKeys.name] = "Hirzek",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{41.8, 79.4},{41.8, 80.8},{43, 78.6},{43.2, 78.4},{43.6, 78.4},{43.6, 78.8},{44.2, 80},{44.2, 91},{44.4, 90},{44.6, 75.8},{44.8, 78.8},{45.4, 79.6},{45.4, 83.8},{46, 74.2},{46, 75.4},{46, 81.6},{46.2, 61.6},{46.2, 76.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212699] = {
            [npcKeys.name] = "Silverwing Archer",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{27.4, 27.6},{28, 26.4},{28.2, 27.4},{28.2, 28.8},{28.4, 28.4},{28.6, 28.2},{28.6, 28.8},{28.6, 29.6},{51.2, 55.8},{51.2, 56.6},{51.4, 54.4},{51.4, 54.8},{51.6, 54.6},{51.6, 55.6},{51.6, 57.2},{51.8, 54.4},{53, 54.6},{58.8, 72.6},{59.4, 72.2},{59.6, 72.6},{59.8, 71.6},{60, 71.4},{60.2, 70.4},{60.6, 71.4},{72.4, 72.2},{72.4, 73.4},{72.8, 72.4},{72.8, 73},{73, 74.8},{73.2, 73.6},{73.6, 73.8},{73.8, 73.4},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212703] = {
            [npcKeys.name] = "Silverwing Dryad",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{28.2, 27.2},{28.4, 28.2},{28.4, 28.6},{28.6, 26.8},{28.6, 28.2},{28.6, 28.8},{51.2, 55.6},{51.4, 54.4},{51.4, 55.4},{51.4, 57},{51.6, 55.6},{51.8, 55},{52, 54.4},{59.4, 72.6},{60, 70.4},{60, 71},{60, 71.8},{60, 72.6},{60.6, 71.4},{72.4, 72},{72.8, 72.4},{73, 73.2},{73.4, 73.6},{73.6, 73.6},{74, 73.4},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212706] = {
            [npcKeys.name] = "Silverwing Druid",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{28, 26.8},{28.4, 28.4},{28.4, 28.6},{28.6, 28.2},{28.6, 28.8},{29, 27},{51.2, 56},{51.4, 54.4},{51.4, 55.4},{51.4, 56.6},{51.6, 54.4},{51.6, 54.6},{59.4, 72},{59.8, 72.6},{60, 70.4},{60, 71},{60, 71.8},{72.4, 72.2},{72.4, 73},{72.6, 72.4},{72.8, 73},{73.2, 73.6},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212707] = {
            [npcKeys.name] = "Larodar",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{46, 56},{46.8, 56},{48.6, 56.6},{49.6, 56.6},{50, 52.8},{50, 57.6},{50.2, 49.2},{50.4, 51.2},{50.4, 53.8},{50.4, 54.8},{50.4, 55.8},{50.8, 50.6},{51, 51.6},{51, 60.2},{51.2, 53},{51.2, 54.4},{51.2, 58.8},{51.2, 61.6},{51.4, 55.2},{51.4, 55.6},{51.4, 56.6},{51.4, 57.6},{51.6, 54.4},{51.6, 54.8},{51.6, 55.6},{51.6, 57.4},{51.6, 60.2},{51.8, 53.4},{51.8, 57.6},{52.6, 54.4},{52.6, 58.2},{53.2, 54.6},{53.2, 58.6},{53.6, 54.6},{53.6, 58.4},{53.8, 54.4},{54.2, 53.4},{54.2, 59},{54.4, 51},{54.4, 51.8},{54.6, 53.4},{54.6, 59.2},{54.8, 54.4},{55, 54.8},{55, 59.6},{55.2, 55.6},{55.6, 55.6},{56, 55.2},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212727] = {
            [npcKeys.name] = "Warsong Grunt",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{20.4, 36.8},{21.2, 36.2},{21.2, 36.6},{21.2, 37.8},{21.2, 38.6},{21.6, 36.4},{21.8, 37.4},{21.8, 37.6},{21.8, 38.6},{38.4, 67.4},{38.4, 67.6},{38.4, 68.6},{38.6, 67},{38.8, 66.4},{39, 67.6},{39, 69.2},{39.6, 65.2},{39.6, 66.4},{39.6, 66.6},{39.6, 67.6},{51.6, 54.6},{52.4, 54.2},{53, 54.6},{53.2, 54.4},{54, 54.8},{54.2, 53.2},{54.2, 54},{54.6, 54.4},{54.8, 55},{55.4, 55.6},{55.6, 55.4},{55.6, 55.6},{68.4, 63.4},{68.4, 63.8},{68.8, 63.6},{69.4, 63},{69.6, 63},{69.6, 63.6},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212728] = {
            [npcKeys.name] = "Warsong Raider",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{21.4, 36.4},{21.4, 36.6},{21.4, 38.2},{21.6, 36.4},{21.8, 37.4},{21.8, 37.6},{21.8, 38.6},{37.4, 67.2},{38.4, 67.2},{38.4, 68},{38.6, 66.8},{38.8, 66.4},{38.8, 69},{39, 67.6},{39.6, 66.4},{39.6, 66.6},{51.6, 54},{53, 54.4},{53.4, 54.6},{54.2, 54.2},{54.4, 54.8},{54.6, 54},{54.8, 55.6},{55, 55},{55.6, 55.2},{68.4, 63.6},{68.6, 63.6},{69.4, 63},{69.6, 63},{69.6, 63.6},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212729] = {
            [npcKeys.name] = "Warsong Shaman",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{20, 35.4},{20.2, 37.2},{20.4, 36.2},{20.4, 37.6},{21.2, 36.6},{21.4, 36.4},{21.4, 38},{21.4, 38.6},{21.8, 36.2},{21.8, 37.4},{21.8, 37.6},{22, 38.6},{22.6, 38.2},{36, 66.4},{37, 66.2},{37, 67.4},{38, 66.4},{38.2, 67.8},{38.4, 67.2},{39, 66.4},{39, 67.6},{39.2, 67},{39.2, 68.8},{39.4, 65.4},{39.6, 64.4},{39.6, 65.4},{39.6, 66.4},{39.6, 67.4},{39.6, 67.6},{50.4, 54.8},{50.4, 56.2},{50.6, 57.8},{51.4, 54},{51.4, 54.6},{51.4, 56.6},{51.8, 54.4},{52.4, 54.6},{53, 54.4},{53, 54.6},{54.2, 53.2},{54.2, 54.4},{54.2, 54.6},{54.6, 53.2},{54.6, 54.4},{54.8, 55},{55, 55.6},{55.6, 51.8},{55.6, 55.4},{55.6, 55.6},{55.8, 59.4},{56.8, 56},{58.2, 56},{68, 62.4},{68.4, 62.6},{68.4, 63.8},{69, 63.6},{69.4, 62.4},{69.4, 62.8},{69.6, 63.6},{69.8, 63.2},{70, 62.4},{70.2, 59.6},{70.6, 63.2},{70.6, 63.6},{70.8, 62.4},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212730] = {
            [npcKeys.name] = "Tojara",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{49, 56.2},{49.4, 57},{49.6, 56.8},{49.8, 57.8},{50, 52.4},{50, 55.2},{50.2, 50.6},{50.2, 54.2},{50.2, 56},{50.4, 49.8},{50.4, 53.4},{50.6, 50.4},{50.8, 50.6},{51, 52.4},{51, 55.6},{51, 58.6},{51.2, 56.6},{51.2, 58.4},{51.4, 53.2},{51.4, 54.2},{51.4, 54.6},{51.6, 54},{51.6, 54.6},{51.6, 57},{51.8, 53.4},{52, 55.8},{52, 57.8},{53, 54.4},{53.2, 58.4},{53.4, 54.6},{53.6, 54.6},{54.2, 51.2},{54.2, 52.2},{54.2, 53.2},{54.2, 54},{54.4, 55.6},{54.6, 51.4},{54.6, 53.4},{54.6, 54.4},{54.8, 55},{54.8, 55.6},{55, 59.6},{55.6, 53.2},{55.6, 55.4},{55.6, 55.6},{55.8, 54},{56.2, 59.4},{56.2, 60.4},{56.4, 56.6},{56.6, 53},{56.8, 55.4},{57, 55.8},{57.8, 56},{58.6, 56.4},{58.8, 57.4},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212753] = {
            [npcKeys.name] = "Tortured Soul",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{45, 58.6},{63, 37.6},{63, 56.8},},
                [44] = {{76.4, 84.2},},
                [130] = {{44.2, 40.2},{44.4, 42},{44.4, 44},{44.8, 42.4},{45, 40.6},{46, 39.6},{46.4, 83.8},{47.4, 83.8},{49, 38.2},{52.6, 55},{53.8, 71.6},{56.8, 70.6},{57.6, 71.4},{58, 71.8},{58.6, 72},{59.4, 70.2},{59.4, 70.8},{59.8, 75.4},{60.4, 72.4},{60.4, 73.4},{60.4, 73.6},{60.6, 72.4},{60.6, 72.6},{62, 72.6},{62.6, 73.8},{63.6, 75.2},{65.8, 80},},
                [1497] = {{54.6, 72},{62.4, 50},{66.8, 43.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212763] = {
            [npcKeys.name] = "Sadistic Fiend",
            [npcKeys.minLevel] = 19,
            [npcKeys.maxLevel] = 19,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{57, 45.8},{57.4, 45},{58.2, 45.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212801] = {
            [npcKeys.name] = "Jubei",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{19.2, 38.6},{20, 35.8},{20.2, 35.4},{20.2, 38.2},{20.4, 37},{20.4, 38.8},{20.6, 35.4},{21.2, 36.4},{21.2, 38.6},{21.4, 37.4},{21.4, 37.6},{21.4, 39.6},{21.6, 36.4},{21.6, 37.4},{21.6, 37.6},{21.8, 38.6},{21.8, 39.6},{22, 35.4},{22.6, 36},{22.6, 38.4},{22.6, 38.6},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212802] = {
            [npcKeys.name] = "Moogul the Sly",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{68.4, 63.4},{68.4, 63.8},{69, 63.6},{69.4, 63.2},{69.6, 63.2},{69.6, 63.6},{70.2, 62},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212803] = {
            [npcKeys.name] = "Ceredwyn",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{72.4, 72.2},{72.4, 73},{72.6, 72.4},{72.8, 73},{73.2, 73.6},{73.6, 73.6},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212804] = {
            [npcKeys.name] = "Centrius",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{27.4, 26},{27.4, 26.8},{27.4, 27.6},{28, 26.4},{28.2, 27.2},{28.4, 28.4},{28.4, 28.6},{28.4, 29.6},{28.6, 26.6},{28.6, 28.2},{28.6, 28.8},{28.6, 30},{28.8, 26.2},{29.6, 26.4},{29.6, 27.2},{29.6, 28.2},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212809] = {
            [npcKeys.name] = "Wailing Spirit",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 20,
            [npcKeys.zoneID] = 130,
            [npcKeys.spawns] = {
                [130] = {{57.2, 70.4},{57.4, 71},{57.6, 71},{58, 71.8},{59.2, 70.6},{59.6, 71.4},{60.2, 74.4},{60.4, 72.2},{60.4, 72.8},{60.4, 74.8},{60.6, 72.4},{60.6, 73.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212837] = {
            [npcKeys.name] = "Primordial Anomaly",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 406,
            [npcKeys.spawns] = {
                [406] = {{27, 64},{27.2, 64.8},{27.4, 67.2},{27.6, 63},{28, 68.2},{28.2, 65.2},{28.2, 71.8},{28.4, 63.6},{28.4, 65.6},{28.6, 62},{28.6, 63.2},{28.6, 64.4},{28.6, 65},{28.6, 66},{28.8, 72.8},{29, 71},{29, 72.2},{29, 73.8},{29.6, 63.6},{29.6, 70.4},{29.6, 72},{31.2, 66.8},{31.8, 67.4},{32.6, 67.6},{33.2, 66.6},{33.2, 69.2},{33.2, 69.6},{33.2, 71.2},{33.6, 69.2},{33.6, 69.8},{35, 72.2},{36.4, 71.6},{36.4, 72.6},{36.6, 73},{36.8, 71.4},{37, 72.2},{37.6, 72},{37.6, 72.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212934] = {
            [npcKeys.name] = "Kazragore",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212953] = {
            [npcKeys.name] = "Felore Moonray",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [212969] = {
            [npcKeys.name] = "Kazragore",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{37.8, 66.4},{38.4, 66.8},{38.4, 67.6},{38.8, 66.4},{39, 67.2},{39, 68.6},{39.2, 67.8},{39.6, 66.4},{39.6, 66.6},{39.6, 67.6},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {79098},
        },
        [212970] = {
            [npcKeys.name] = "Felore Moonray",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{60, 70.4},{60, 71.4},{60, 71.8},{60, 72.6},},
            },
            [npcKeys.friendlyToFaction] = "A",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {79090},
        },
        [213077] = {
            [npcKeys.name] = "Elaine Compton",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{54.4, 60.8},{54.6, 61.2},{54.8, 62},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308},
        },
        [213334] = {
            [npcKeys.name] = "Aku'mai",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213444] = {
            [npcKeys.name] = "Mysterious Traveler",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{79, 80.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {78684},
            [npcKeys.questEnds] = nil,
        },
        [213445] = {
            [npcKeys.name] = "Raszel Ander",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{49.2, 57.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78914,78994},
            [npcKeys.questEnds] = {78994},
        },
        [213450] = {
            [npcKeys.name] = "Felguard",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213451] = {
            [npcKeys.name] = "Cleansing Elemental",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213452] = {
            [npcKeys.name] = "Corrosion Elemental",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213540] = {
            [npcKeys.name] = "Mysterious Traveler",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{49.2, 57.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213595] = {
            [npcKeys.name] = "Void Elemental",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213659] = {
            [npcKeys.name] = "Dark Ritual Portal",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213708] = {
            [npcKeys.name] = "Legion Hound",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{79, 80.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213709] = {
            [npcKeys.name] = "Mannoroc Lasher",
            [npcKeys.minLevel] = 24,
            [npcKeys.maxLevel] = 24,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{79, 80.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213710] = {
            [npcKeys.name] = "Felguard",
            [npcKeys.minLevel] = 24,
            [npcKeys.maxLevel] = 24,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{79, 80.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213711] = {
            [npcKeys.name] = "Searing Infernal",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{79, 80.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213795] = {
            [npcKeys.name] = "Gharrik",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 3,
            [npcKeys.spawns] = {
                [3] = {{22.4, 67.4},{22.4, 67.8},{23, 66.8},{23.2, 66.4},{26, 63.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [213994] = {
            [npcKeys.name] = "Corrosive Droplet",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214000] = {
            [npcKeys.name] = "Corrosive Pool Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214063] = {
            [npcKeys.name] = "Temporal Anomaly",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214070] = {
            [npcKeys.name] = "Jornah",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{51.4, 63.8},{51.6, 63.8},{51.6, 64.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308},
        },
        [214096] = {
            [npcKeys.name] = "Dokimi",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1638,
            [npcKeys.spawns] = {
                [1638] = {{39.4, 53.4},{39.4, 53.6},{39.6, 53.4},{39.6, 53.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308,80309},
        },
        [214098] = {
            [npcKeys.name] = "Gishah",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{64, 39.2},{64.4, 38.4},{64.6, 38.6},{64.8, 38.2},{65.6, 38.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308},
        },
        [214099] = {
            [npcKeys.name] = "Tamelyn Aldridge",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1537,
            [npcKeys.spawns] = {
                [1537] = {{24, 65.8},{24.4, 67.4},{24.4, 67.6},{24.6, 67.4},{24.6, 67.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {80309},
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308,80309},
        },
        [214101] = {
            [npcKeys.name] = "Marcy Baker",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{59.2, 56.6},{59.8, 56.4},{59.8, 56.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {78611,78612,78872,79100,79101,79102,79103,80307,80308},
            [npcKeys.questEnds] = {78611,78612,78872,79100,79101,79102,79103,80307,80308},
        },
        [214129] = {
            [npcKeys.name] = "Venture Co. Light Shredder",
            [npcKeys.minLevel] = 20,
            [npcKeys.maxLevel] = 21,
            [npcKeys.zoneID] = 406,
            [npcKeys.spawns] = {
                [406] = {{59, 49.4},{59, 53.8},{59.2, 54.6},{59.4, 50.6},{59.4, 52.8},{59.8, 51.4},{59.8, 51.6},{59.8, 52.8},{60, 54.4},{60, 55},{60.2, 50.4},{60.4, 55.8},{60.6, 51},{60.6, 55.8},{60.8, 49.8},{60.8, 54.6},{61.4, 52.4},{61.4, 52.8},{61.4, 53.6},{61.8, 52.4},{62, 51.4},{62, 53},{62.2, 53.6},{62.4, 55.2},{62.6, 52},{62.6, 52.8},{65.6, 48.6},{65.8, 46.4},{66.2, 57},{66.4, 47.4},{66.4, 47.6},{66.4, 58},{66.6, 47.6},{66.6, 54.8},{66.8, 47.4},{66.8, 48.6},{67.2, 46.4},{67.4, 56.2},{67.4, 57.4},{67.4, 57.6},{67.6, 57.4},{67.6, 57.6},{67.8, 56.4},{68.2, 47.4},{68.2, 47.6},{68.4, 48.6},{68.6, 47.4},{68.6, 47.8},{68.6, 54.4},{68.6, 57.8},{68.8, 48.6},{68.8, 57.4},{69.2, 49.8},{69.4, 55.2},{69.4, 56.4},{69.6, 47.4},{69.8, 54.4},{70, 48.2},{70, 53.4},{70, 55.6},{70.2, 55.2},{70.4, 41.2},{70.4, 41.6},{70.4, 42.6},{70.4, 49},{70.4, 49.6},{70.4, 51.2},{70.6, 41.4},{70.6, 42.8},{70.6, 55.2},{70.6, 55.6},{70.8, 42},{70.8, 48.2},{70.8, 49.2},{71, 49.8},{71, 50.8},{71.6, 48.2},{71.6, 49},{72, 56.6},{72.2, 50.2},{72.2, 52.4},{72.4, 53.2},{72.4, 53.6},{72.4, 55.8},{72.6, 53},{72.8, 52.4},{72.8, 53.6},{73, 44.8},{73, 51.4},{73.2, 55.6},{73.6, 51.6},{73.6, 56},{74.8, 52.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214133] = {
            [npcKeys.name] = "Breath Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214190] = {
            [npcKeys.name] = "Blackfathom Pearl",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214208] = {
            [npcKeys.name] = "N'ora Anyheart",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 17,
            [npcKeys.spawns] = {
                [17] = {{62, 39.4},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {78909},
            [npcKeys.questEnds] = {78909},
        },
        [214212] = {
            [npcKeys.name] = "Shadowy Figure",
            [npcKeys.minLevel] = 63,
            [npcKeys.maxLevel] = 63,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{88.4, 73.4},{88.6, 73.4},{88.6, 73.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214254] = {
            [npcKeys.name] = "Pool of Shadow Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214440] = {
            [npcKeys.name] = "Demolition Explosives",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214456] = {
            [npcKeys.name] = "Dro'zem the Blasphemous",
            [npcKeys.minLevel] = 23,
            [npcKeys.maxLevel] = 23,
            [npcKeys.zoneID] = 44,
            [npcKeys.spawns] = {
                [44] = {{32.4, 7},{32.6, 6.8},{33.4, 7.8},{33.6, 7},{34, 7.8},{34.2, 8.6},{34.6, 7.4},{34.6, 7.6},{34.8, 10},{35.4, 8.6},{35.6, 8.2},{35.8, 6.8},{35.8, 9},{35.8, 9.8},{36, 12},{36.4, 11.4},{36.6, 7.6},{36.8, 11.2},{36.8, 13.6},{37, 12},{38.6, 13.8},{39.6, 14.4},{57.4, 41.4},{58.8, 46.6},{59.8, 40.4},{59.8, 46.4},{60.8, 44},{61.2, 41},{61.2, 46},{61.4, 45.2},{61.4, 48},{62.2, 41.4},{62.2, 42.2},{62.4, 43.4},{62.4, 43.6},{62.4, 44.8},{62.6, 42},{62.6, 43.2},{62.6, 44.2},{62.6, 44.6},{63.4, 41.4},{63.6, 42.8},{64, 42},{64, 44.2},{64.2, 45.8},{64.4, 45},{64.6, 46.6},{64.8, 40.2},{65, 48.4},{65.2, 42.6},{69.8, 82.6},{71.6, 79.2},{71.8, 80.8},{72, 82},{72.6, 81},{73, 84.2},{73.6, 79.6},{73.8, 76},{74, 81},{74, 83.4},{74.2, 83.6},{74.4, 82.2},{74.6, 73.6},{74.6, 76.8},{74.6, 78.6},{75, 71.6},{75, 85},{75.2, 73.2},{75.2, 82},{75.2, 82.8},{75.2, 83.6},{75.4, 77.8},{75.4, 80.4},{75.4, 81},{75.6, 79.2},{75.6, 84.6},{75.8, 78.2},{75.8, 79.6},{75.8, 81.2},{75.8, 83.6},{76, 77.4},{76.2, 72},{76.2, 82},{76.2, 83.4},{76.6, 82.8},{77, 70},{77, 73.8},{77.2, 79.8},{77.4, 69.4},{77.4, 71.4},{77.4, 71.6},{77.4, 75},{77.6, 70.6},{77.6, 74},{77.8, 67},{78, 69.6},{78, 72.4},{78.2, 63.6},{78.2, 66.4},{78.2, 68.2},{78.4, 69.2},{78.4, 72.8},{78.6, 68},{78.6, 69.2},{78.6, 69.6},{78.6, 70.6},{78.6, 72},{78.8, 67.4},{79.4, 66.4},{79.4, 72.6},{79.8, 70.6},{79.8, 72.4},{80, 70},{80.2, 67.6},{80.2, 68.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214458] = {
            [npcKeys.name] = "Baron Aquanis",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214468] = {
            [npcKeys.name] = "Ghamoo-ra",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214469] = {
            [npcKeys.name] = "Lady Sarevess",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214519] = {
            [npcKeys.name] = "Incinerator Gar'im",
            [npcKeys.minLevel] = 23,
            [npcKeys.maxLevel] = 23,
            [npcKeys.zoneID] = 44,
            [npcKeys.spawns] = {
                [44] = {{74, 82.2},{74.4, 84.8},{74.6, 83.4},{75, 77.6},{75.2, 84.4},{75.8, 82},{75.8, 84.6},{76, 74.8},{76, 84.4},{76.4, 83.4},{76.6, 79.8},{76.6, 84.2},{76.8, 82.2},{77, 83.2},{77.2, 76.4},{77.4, 85.4},{77.4, 85.8},{77.4, 86.6},{77.8, 86.2},{77.8, 86.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214523] = {
            [npcKeys.name] = "Blackfathom Elite",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214524] = {
            [npcKeys.name] = "Gelihast",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214525] = {
            [npcKeys.name] = "Lorgus Jett",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214526] = {
            [npcKeys.name] = "Twilight Lord Kelris",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214527] = {
            [npcKeys.name] = "Aku'mai",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214528] = {
            [npcKeys.name] = "Boss Dummy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214529] = {
            [npcKeys.name] = "Brave Stonetorch",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 267,
            [npcKeys.spawns] = {
                [267] = {{65.8, 19.6},{66, 19.2},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214530] = {
            [npcKeys.name] = "Ardelle Dralta",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 267,
            [npcKeys.spawns] = {
                [267] = {{51, 58.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214589] = {
            [npcKeys.name] = "Cozy Sleeping Bag",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214603] = {
            [npcKeys.name] = "Blackfathom Tide Priestess",
            [npcKeys.minLevel] = 26,
            [npcKeys.maxLevel] = 26,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214612] = {
            [npcKeys.name] = "Aku'mai Fisher",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214658] = {
            [npcKeys.name] = "Ashenvale Arcane Shard Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214695] = {
            [npcKeys.name] = "Peon",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214830] = {
            [npcKeys.name] = "Wisp",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214837] = {
            [npcKeys.name] = "Cheddar",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214876] = {
            [npcKeys.name] = "Davius Voidstar",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 148,
            [npcKeys.spawns] = {
                [148] = {{36.8, 43.4},{36.8, 43.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79099},
            [npcKeys.questEnds] = {79099},
        },
        [214914] = {
            [npcKeys.name] = "Blackfathom Pearl",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [214954] = {
            [npcKeys.name] = "Rix Xizzix",
            [npcKeys.minLevel] = 45,
            [npcKeys.maxLevel] = 45,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{28.4, 75.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215062] = {
            [npcKeys.name] = "Supplicant",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{39.4, 28.4},{39.4, 29},{39.4, 30.4},{39.6, 28.4},{39.6, 29.2},{40, 29.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215068] = {
            [npcKeys.name] = "[DNT] Kill Credit - Priest - Meditation Sharing Tutorial",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215072] = {
            [npcKeys.name] = "Loa Altar",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{79.8, 67},{80, 67.8},},
                [331] = {{11.8, 35.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215081] = {
            [npcKeys.name] = "Bat Loa",
            [npcKeys.minLevel] = 60,
            [npcKeys.maxLevel] = 60,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{79.8, 67.2},},
                [331] = {{11.8, 35.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215095] = {
            [npcKeys.name] = "Supplicant",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{39.4, 87.4},{39.4, 87.8},{39.8, 86.2},{40, 86.8},{40, 88.6},{40.4, 87.8},{40.6, 88.4},{40.8, 86.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215096] = {
            [npcKeys.name] = "Supplicant",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{35.4, 86.4},{35.4, 86.8},{35.4, 87.6},{35.6, 87.6},{35.8, 86.2},{35.8, 87},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215097] = {
            [npcKeys.name] = "Supplicant",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215098] = {
            [npcKeys.name] = "Supplicant",
            [npcKeys.minLevel] = 5,
            [npcKeys.maxLevel] = 5,
            [npcKeys.zoneID] = 1497,
            [npcKeys.spawns] = {
                [1497] = {{48.2, 14.6},{48.4, 14},{49.4, 14.4},{49.4, 15},{49.4, 16},{49.6, 16},{50.2, 14.4},{50.2, 14.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215108] = {
            [npcKeys.name] = "Aku'mai Snapjaw",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 27,
            [npcKeys.zoneID] = 719,
            [npcKeys.spawns] = {
                [719] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215367] = {
            [npcKeys.name] = "Dawnwatcher Selgorm",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1657,
            [npcKeys.spawns] = {
                [1657] = {{55.6, 23.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215369] = {
            [npcKeys.name] = "Bashana Runetotem",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1638,
            [npcKeys.spawns] = {
                [1638] = {{70, 33.4},{70, 33.6},{70.8, 32.4},{70.8, 32.6},{70.8, 33.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215643] = {
            [npcKeys.name] = "Tokal",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{27, 77.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {79236,79242},
            [npcKeys.questEnds] = {79235,79236},
        },
        [215655] = {
            [npcKeys.name] = "Illari Duskfeather",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{93.8, 71.4},{93.8, 71.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215688] = {
            [npcKeys.name] = "Unkillable Test Dummy 25 Warrior",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215709] = {
            [npcKeys.name] = "Void Zone",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215728] = {
            [npcKeys.name] = "Crowd Pummeler 9-60",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215849] = {
            [npcKeys.name] = "Doan Karhan",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215850] = {
            [npcKeys.name] = "Raszel Ander",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{51.2, 82.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215854] = {
            [npcKeys.name] = "Shadowbinder Acolyte",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215855] = {
            [npcKeys.name] = "Shadowbinder Acolyte",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215890] = {
            [npcKeys.name] = "Gear Toss Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [215974] = {
            [npcKeys.name] = "Des'Altek",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216098] = {
            [npcKeys.name] = "Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216126] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216145] = {
            [npcKeys.name] = "Gnomeregan Smash Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216289] = {
            [npcKeys.name] = "Orokai",
            [npcKeys.minLevel] = 62,
            [npcKeys.maxLevel] = 62,
            [npcKeys.zoneID] = 493,
            [npcKeys.spawns] = {
                [493] = {{41.2, 43.6},{41.4, 43.4},{41.6, 43.4},{41.6, 43.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79377},
            [npcKeys.questEnds] = {79348,79377},
        },
        [216310] = {
            [npcKeys.name] = "Rotting Ancient",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 15,
            [npcKeys.spawns] = {
                [15] = {{41.2, 42},{41.4, 42.6},{41.8, 42.4},{42, 44},{42.4, 40.8},{42.4, 43.2},{43.6, 41},{44, 42},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216437] = {
            [npcKeys.name] = "Ancient Sapling",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{58.4, 44},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216445] = {
            [npcKeys.name] = "Ancient Sapling",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 8,
            [npcKeys.spawns] = {
                [8] = {{11.2, 38.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216448] = {
            [npcKeys.name] = "Ancient Sapling",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{32.8, 64.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216451] = {
            [npcKeys.name] = "Ancient Sapling",
            [npcKeys.minLevel] = 10,
            [npcKeys.maxLevel] = 10,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{46.8, 72},{47, 71.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216463] = {
            [npcKeys.name] = "Ailgrha Splittusk",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{55, 42},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216474] = {
            [npcKeys.name] = "Ravaging Tempest",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{53.8, 41},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216620] = {
            [npcKeys.name] = "Unkillable Test Dummy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216659] = {
            [npcKeys.name] = "Fallenroot Satyr",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{13.4, 10},{13.4, 12.2},{13.4, 13.2},{13.6, 11.8},{13.6, 13.4},{13.8, 10},{13.8, 11.2},{14.4, 9.4},{14.6, 9.8},{15, 10.6},{15.8, 11.4},{16.2, 11.6},{16.6, 11},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216660] = {
            [npcKeys.name] = "Fallenroot Rogue",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{14.4, 9.4},{14.4, 9.6},{14.8, 10.4},{15.2, 10.8},{15.8, 11.4},{16.4, 11.6},{16.6, 11},{16.6, 11.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216661] = {
            [npcKeys.name] = "Blackfathom Tide Priestess",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{12.8, 10},{13.4, 9.4},{13.4, 12.2},{13.4, 13.2},{13.6, 11.8},{13.6, 13.4},{13.8, 10},{13.8, 11.2},{14.2, 9.2},{14.6, 8.6},{14.8, 10.4},{15, 10.6},{15.8, 11.4},{16.4, 11.6},{16.6, 11},{17.6, 11},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216662] = {
            [npcKeys.name] = "Blackfathom Oracle",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{13.6, 9.6},{14.4, 9.4},{14.6, 9.8},{15.2, 10.8},{15.6, 11.2},{16.4, 11.6},{16.6, 11},{16.6, 11.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216664] = {
            [npcKeys.name] = "Sprok",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216665] = {
            [npcKeys.name] = "Gnomeregan Evacuee",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{24.4, 40.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216666] = {
            [npcKeys.name] = "Techbot",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216667] = {
            [npcKeys.name] = "Addled Leper",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 1,
            [npcKeys.spawns] = {
                [1] = {{24.4, 39.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216668] = {
            [npcKeys.name] = "Irradiated Invader",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216669] = {
            [npcKeys.name] = "Caverndeep Pillager",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216670] = {
            [npcKeys.name] = "Caverndeep Looter",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216671] = {
            [npcKeys.name] = "Caverndeep Invader",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216672] = {
            [npcKeys.name] = "Namdo Bizzfizzle",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216810] = {
            [npcKeys.name] = "Desiccated Fallout",
            [npcKeys.minLevel] = 41,
            [npcKeys.maxLevel] = 41,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216854] = {
            [npcKeys.name] = "Freewind Post Wyvern",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{53.2, 41.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216902] = {
            [npcKeys.name] = "Wulmort Jinglepocket",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1537,
            [npcKeys.spawns] = {
                [1537] = {{33.4, 65.6},{33.4, 67},{33.4, 67.6},{33.6, 66.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {7022,79482,79486,79492},
            [npcKeys.questEnds] = {79484,79492},
        },
        [216915] = {
            [npcKeys.name] = "Strange Snowman",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{35.4, 72.4},{35.4, 72.6},{35.6, 72.4},{35.6, 72.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {79484,79485},
            [npcKeys.questEnds] = {79482,79483},
        },
        [216924] = {
            [npcKeys.name] = "Kaymard Copperpinch",
            [npcKeys.minLevel] = 30,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{53.2, 66.4},{53.2, 66.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {6961,79483,79487,79495},
            [npcKeys.questEnds] = {79485,79495},
        },
        [216931] = {
            [npcKeys.name] = "The Abominable Greench",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{28.4, 62.6},{28.6, 51.6},{28.8, 56.2},{28.8, 61.4},{29, 55.4},{29.2, 57},{29.2, 63.6},{29.4, 62.6},{29.6, 59.8},{29.8, 63.4},{30, 51.4},{30, 55.6},{30, 62},{30.4, 46.6},{30.4, 54.6},{30.4, 64.4},{30.4, 70.8},{30.6, 63.6},{30.8, 47.2},{30.8, 63},{30.8, 65.6},{30.8, 66.8},{31, 65.4},{31, 70.6},{31.4, 48.8},{31.4, 53.4},{31.6, 51.4},{31.6, 66.2},{31.6, 71.8},{31.8, 61.4},{31.8, 70.6},{32, 66.8},{32, 70},{32.2, 48.6},{32.2, 52.8},{32.4, 50},{32.4, 67.8},{32.6, 47.8},{32.6, 49},{32.6, 50.6},{32.8, 50},{32.8, 71},{32.8, 71.8},{33, 70.4},{33.6, 72.2},{34, 70.2},{34.2, 69.4},{34.2, 70.8},{34.6, 46.6},{34.6, 71},{35.2, 74.8},{36.8, 70.4},{37, 72.4},{37.6, 69.4},{37.8, 72.2},{38.2, 66.4},{38.2, 74},{38.4, 72.8},{38.6, 72.4},{39, 70.2},{39.2, 66},{39.2, 70.6},{39.4, 68.6},{39.6, 64.4},{39.6, 64.8},{39.8, 81.8},{40.2, 73.8},{40.2, 74.6},{40.4, 71.8},{40.4, 73.4},{40.6, 62.6},{40.6, 69.8},{40.6, 74.8},{40.6, 80},{40.8, 74.2},{41, 72},{41, 77.6},{41, 79.2},{41.4, 69.4},{41.4, 76.6},{41.6, 79.6},{41.8, 69.2},{42, 77},{43, 67.4},{43.8, 62.8},{43.8, 66},{44, 65.2},{44.4, 64},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [216937] = {
            [npcKeys.name] = "Blood Moon Listener Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217049] = {
            [npcKeys.name] = "Mirror Image",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{20, 36.2},{20.2, 35.2},{20.4, 37},{20.4, 37.6},{21.2, 38},{21.2, 38.6},{21.2, 39.8},{21.4, 36.4},{21.4, 36.8},{21.6, 36.2},{21.6, 37.4},{21.6, 38.6},{21.8, 37.6},{21.8, 39.8},{22.4, 34.8},{22.6, 38.2},{22.6, 38.8},{22.8, 35.8},{24, 38.4},{25.2, 37.4},{25.4, 39.4},{25.6, 39.4},{25.6, 40.4},{25.8, 41},{26, 34.4},{26, 41.8},{26.4, 45.2},{26.8, 43.2},{26.8, 44},{48.2, 56.4},{48.4, 56.6},{48.6, 56.4},{49.4, 56.8},{49.8, 56.6},{50, 52.4},{50.2, 49.6},{50.2, 53},{50.2, 56.2},{50.4, 48.6},{50.4, 54.4},{50.4, 54.6},{50.8, 51.2},{51, 51.8},{51, 57},{51.2, 52.8},{51.2, 54.6},{51.4, 54.2},{51.4, 56},{51.6, 53.2},{51.6, 54.4},{51.6, 54.6},{51.6, 56},{51.6, 57.2},{52.4, 57.8},{53.2, 54.4},{53.2, 54.6},{53.8, 54.6},{54.2, 53.8},{54.4, 52.4},{54.4, 53.4},{54.4, 55.6},{54.6, 53.4},{54.6, 54.2},{54.8, 54.8},{55, 50.8},{55, 55.6},{55.6, 53.6},{55.6, 55},{56.4, 49.4},{56.4, 55.6},{56.4, 57.4},{56.4, 57.6},{56.6, 53},{56.8, 55.6},{58.4, 56.6},{58.8, 56.2},{59.2, 54.8},{59.2, 68.4},{59.6, 56.2},{59.8, 63.6},{59.8, 68.4},{60, 55},{60, 68.6},{60, 72.2},{60.2, 69.6},{60.6, 68.4},{60.8, 70.2},{61, 55.6},{61.6, 58},{62.2, 54.6},{62.6, 68.4},{62.8, 54.2},{63, 60},{63.4, 54.6},{63.6, 55},{63.8, 67.2},{64.2, 56.4},{64.2, 57.4},{64.4, 57.8},{64.4, 59.6},{64.4, 66.4},{64.8, 59.4},{64.8, 59.6},{64.8, 60.6},{64.8, 65.2},{64.8, 66.8},{65.2, 64.4},{65.6, 63.8},{65.6, 64.8},{65.8, 60.6},{66, 63},{66.4, 62},{66.6, 63.8},{66.6, 64.8},{67, 63.4},{67.2, 62.2},{67.6, 64.6},{68.2, 60.8},{68.4, 62.4},{68.4, 62.8},{68.4, 63.8},{68.6, 62.4},{68.6, 64.6},{68.8, 63.6},{69.2, 63},{69.6, 60.4},{69.6, 63},{69.6, 63.6},{69.8, 61.4},{70.2, 62.4},{70.6, 62.4},{70.6, 63},},
            },
            [npcKeys.friendlyToFaction] = "H",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217215] = {
            [npcKeys.name] = "Irradiated Cloud",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217216] = {
            [npcKeys.name] = "Grubbis Controller",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217221] = {
            [npcKeys.name] = "Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217280] = {
            [npcKeys.name] = "Grubbis",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217290] = {
            [npcKeys.name] = "Ogre Lightning Rod",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{33.4, 44.4},{33.4, 44.6},{33.6, 44.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217300] = {
            [npcKeys.name] = "Skonk",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{57.4, 74.4},{57.6, 74.4},{57.6, 74.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79677,79678},
            [npcKeys.questEnds] = {79624,79677,79678},
        },
        [217302] = {
            [npcKeys.name] = "Tam'kar",
            [npcKeys.minLevel] = 37,
            [npcKeys.maxLevel] = 37,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{32.4, 43.4},{33, 44.8},{33.4, 44.4},{33.6, 44.4},{33.8, 45},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217305] = {
            [npcKeys.name] = "Ancient Fire Elemental",
            [npcKeys.minLevel] = 34,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 36,
            [npcKeys.spawns] = {
                [36] = {{59.4, 45.2},{59.4, 46.4},{59.4, 46.6},{59.4, 47.6},{59.6, 46.6},{59.8, 45},{60, 45.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217308] = {
            [npcKeys.name] = "Irradiated Goo",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217387] = {
            [npcKeys.name] = "Brother Atticus",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{27.2, 57},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217392] = {
            [npcKeys.name] = "Flameseer Dubelen",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{56.4, 21.2},{56.4, 21.8},{56.4, 22.8},{56.6, 21.4},{56.6, 21.8},{58.2, 21.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217412] = {
            [npcKeys.name] = "Amaryllis Webb",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 8,
            [npcKeys.spawns] = {
                [8] = {{25, 54.2},{25.2, 54.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217418] = {
            [npcKeys.name] = "Zai'enki",
            [npcKeys.minLevel] = 28,
            [npcKeys.maxLevel] = 28,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{68.2, 55.6},{68.4, 55.4},{68.6, 55.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217580] = {
            [npcKeys.name] = "Seductress Ceeyna",
            [npcKeys.minLevel] = 38,
            [npcKeys.maxLevel] = 38,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{81.4, 80.2},{81.8, 80.4},{81.8, 80.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217582] = {
            [npcKeys.name] = "Caverndeep Ambusher",
            [npcKeys.minLevel] = 41,
            [npcKeys.maxLevel] = 41,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217588] = {
            [npcKeys.name] = "Arbor Tarantula",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{43.8, 18.4},{43.8, 18.6},{44.2, 22.2},{45.2, 19.4},{45.2, 19.6},{45.4, 22.2},{45.4, 23.2},{45.6, 22.2},{45.6, 23},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217589] = {
            [npcKeys.name] = "Hay Weevil",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{30.8, 28.2},{30.8, 28.6},{31, 26.2},{32.2, 31},{54, 38.4},{54.2, 38.6},{57, 39.6},{61.2, 55.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217590] = {
            [npcKeys.name] = "Flesh Picker",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{49, 58},{49.8, 55.8},{50, 59},{51.2, 58},{51.4, 59.8},{51.6, 57},{51.6, 59.8},{52.4, 58},{52.4, 58.6},{52.6, 56.4},{52.6, 58.2},{53, 59.2},{53.6, 62},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217605] = {
            [npcKeys.name] = "Void Seeker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{29.2, 63},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217620] = {
            [npcKeys.name] = "Reckless Warlock",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{30.8, 47},},
                [36] = {{54.4, 49.4},{54.4, 49.6},},
                [400] = {{10.8, 40.8},},
                [405] = {{74.8, 13.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217669] = {
            [npcKeys.name] = "Scorched Screeching Roguefeather",
            [npcKeys.minLevel] = 29,
            [npcKeys.maxLevel] = 30,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{26.4, 46.4},{26.4, 47.2},{26.6, 47.4},{26.6, 47.6},{26.8, 46.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217683] = {
            [npcKeys.name] = "Altar of the Wind Spirit",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217689] = {
            [npcKeys.name] = "Ziri \"The Wrench\" Littlesprocket <Gearhead>",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79705},
            [npcKeys.questEnds] = nil,
        },
        [217703] = {
            [npcKeys.name] = "Singed Highperch Consort",
            [npcKeys.minLevel] = 28,
            [npcKeys.maxLevel] = 29,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{10.4, 40.2},{10.4, 40.6},{10.8, 40.8},{11, 39.2},{11.2, 40},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217706] = {
            [npcKeys.name] = "Kazragore",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{74, 60.4},{74, 60.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {79098},
        },
        [217707] = {
            [npcKeys.name] = "Felore Moonray",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 331,
            [npcKeys.spawns] = {
                [331] = {{36.8, 49.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {79090},
        },
        [217711] = {
            [npcKeys.name] = "Seared Needles Cougar",
            [npcKeys.minLevel] = 27,
            [npcKeys.maxLevel] = 28,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{23.4, 23.4},{23.4, 23.6},{23.4, 25},{23.6, 24.4},{23.6, 25},{23.8, 23.4},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217733] = {
            [npcKeys.name] = "Caverndeep Pillager",
            [npcKeys.minLevel] = 41,
            [npcKeys.maxLevel] = 41,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217783] = {
            [npcKeys.name] = "Bloodscalp Guerrilla",
            [npcKeys.minLevel] = 34,
            [npcKeys.maxLevel] = 34,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{30.4, 14.2},{30.4, 15.4},{30.8, 16.4},{31.2, 14.2},{31.6, 14.2},{31.8, 15.6},{32.4, 14.8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217836] = {
            [npcKeys.name] = "Needletooth",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{66.4, 66.8},{67, 64.6},{67.4, 66.2},{67.4, 67},{67.4, 67.8},{67.6, 67.6},{67.8, 65.2},{67.8, 65.6},{68, 64},{68, 67.4},{68.2, 63.2},{68.6, 64.6},{68.8, 63.8},{69.4, 63.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217862] = {
            [npcKeys.name] = "Reconstructed Staff of Des'Altek",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217907] = {
            [npcKeys.name] = "Chicken TEST",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217908] = {
            [npcKeys.name] = "Willing Sacrifice",
            [npcKeys.minLevel] = 39,
            [npcKeys.maxLevel] = 39,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{81.8, 80.4},{82, 80.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217942] = {
            [npcKeys.name] = "Trogg Monster Random",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217943] = {
            [npcKeys.name] = "Scarlet Crusade Assassin",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217945] = {
            [npcKeys.name] = "Domesticated Attack Chicken",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217956] = {
            [npcKeys.name] = "Chomper",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217957] = {
            [npcKeys.name] = "Slain Scarlet Crusader",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217969] = {
            [npcKeys.name] = "Blastmaster Emi Shortfuse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217980] = {
            [npcKeys.name] = "Nervous-Looking Mage",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217987] = {
            [npcKeys.name] = "Giga Dummy",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [217996] = {
            [npcKeys.name] = "Aeonas",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 796,
            [npcKeys.spawns] = {
                [796] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = {79963,79970},
            [npcKeys.questEnds] = {79946,79963},
        },
        [218007] = {
            [npcKeys.name] = "Josh's Kill Credit Creature 1",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218019] = {
            [npcKeys.name] = "Combat Dummy",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{68, 89.4},{68, 89.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218020] = {
            [npcKeys.name] = "Combat Dummy",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{67.8, 89.4},{68, 89.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218021] = {
            [npcKeys.name] = "Combat Dummy",
            [npcKeys.minLevel] = 1,
            [npcKeys.maxLevel] = 1,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{67.8, 89.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218029] = {
            [npcKeys.name] = "Witherbark Champion",
            [npcKeys.minLevel] = 34,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{68, 79.4},{68.4, 81},{69, 81.4},{69.4, 80.4},{69.4, 81.6},{69.6, 81.4},{69.6, 81.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218032] = {
            [npcKeys.name] = "Witherbark Goliath",
            [npcKeys.minLevel] = 35,
            [npcKeys.maxLevel] = 35,
            [npcKeys.zoneID] = 45,
            [npcKeys.spawns] = {
                [45] = {{68, 79.2},{68.2, 80.2},{68.4, 80.6},{68.6, 81},{69.4, 81.6},{69.6, 81.6},{69.8, 81},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218057] = {
            [npcKeys.name] = "Crowd Pummeler Bleacher Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218089] = {
            [npcKeys.name] = "Scarlet Squire Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218090] = {
            [npcKeys.name] = "Scarlet Crusader Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218093] = {
            [npcKeys.name] = "Burning Blade Acolyte Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218095] = {
            [npcKeys.name] = "Burning Blade Acolyte Corpse",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218115] = {
            [npcKeys.name] = "Mai'zin",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{31.2, 48.4},{31.4, 48.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218160] = {
            [npcKeys.name] = "Aeonas the Vindicated",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1519,
            [npcKeys.spawns] = {
                [1519] = {{37.4, 31.8},{38, 31.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = {79970},
        },
        [218229] = {
            [npcKeys.name] = "Captain Aransas",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{27.6, 76.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218230] = {
            [npcKeys.name] = "Wendel Mathers",
            [npcKeys.minLevel] = 25,
            [npcKeys.maxLevel] = 25,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{44.2, 8},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218231] = {
            [npcKeys.name] = "Highlands Knight",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218232] = {
            [npcKeys.name] = "Kha'damu",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218233] = {
            [npcKeys.name] = "Forest Troll Shadow Priest",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218234] = {
            [npcKeys.name] = "Ogre Magi",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218235] = {
            [npcKeys.name] = "Gnoll Overseer",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218236] = {
            [npcKeys.name] = "Mud Golem",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218237] = {
            [npcKeys.name] = "Wirdal Wondergear",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218238] = {
            [npcKeys.name] = "Josh's Kill Credit Creature 2",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218240] = {
            [npcKeys.name] = "Tower Defense Automaton",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218241] = {
            [npcKeys.name] = "Shadowy Figure",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218242] = {
            [npcKeys.name] = "STX-04/BD",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218243] = {
            [npcKeys.name] = "STX-13/LL",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218244] = {
            [npcKeys.name] = "STX-25/NB",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218245] = {
            [npcKeys.name] = "STX-37/CN",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218246] = {
            [npcKeys.name] = "Gurgthock",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 12,
            [npcKeys.spawns] = {
                [12] = {{33.2, 50.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218249] = {
            [npcKeys.name] = "Slitherblade Tide Priestess",
            [npcKeys.minLevel] = 33,
            [npcKeys.maxLevel] = 34,
            [npcKeys.zoneID] = 405,
            [npcKeys.spawns] = {
                [405] = {{29.4, 5.8},{29.6, 6},{33, 11.4},{34.4, 20.2},{34.4, 25.4},{34.4, 26.4},{34.4, 26.8},{34.4, 29.8},{34.6, 20},{34.6, 21},{34.6, 26.2},{34.6, 27},{34.6, 30},{35.6, 31.4},{36.2, 27.6},{36.4, 27},{36.6, 29.8},{36.8, 26.2},{36.8, 26.6},{38.4, 23.4},{38.4, 24},{38.8, 24},{39, 23},{39.6, 13.4},{39.6, 13.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218262] = {
            [npcKeys.name] = "Leprous Assistant",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218263] = {
            [npcKeys.name] = "Leprous Machinesmith",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218273] = {
            [npcKeys.name] = "Wandering Swordsman",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 3,
            [npcKeys.spawns] = {
                [3] = {{14.4, 44.8},{15.6, 45.8},{20, 54.6},{20, 57},{24.8, 58.8},{25.2, 60.2},{25.4, 61.2},{26.4, 71},{28, 68.4},{33.2, 68.4},{33.4, 68.6},{33.8, 69.2},{35.2, 58.8},{35.2, 59.8},{35.6, 59.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218343] = {
            [npcKeys.name] = "Leprous Defender",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218344] = {
            [npcKeys.name] = "Explosive Egg",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218345] = {
            [npcKeys.name] = "Gnomeregan Items",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218349] = {
            [npcKeys.name] = "Walking Bomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218386] = {
            [npcKeys.name] = "Breath Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218434] = {
            [npcKeys.name] = "[DNT] Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218460] = {
            [npcKeys.name] = "[DNT] Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218461] = {
            [npcKeys.name] = "[DNT] Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218462] = {
            [npcKeys.name] = "[DNT] Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218537] = {
            [npcKeys.name] = "Mekgineer Thermaplugg",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218538] = {
            [npcKeys.name] = "STX-96/FR",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218610] = {
            [npcKeys.name] = "Test Creature",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218683] = {
            [npcKeys.name] = "Wendel Mathers",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218690] = {
            [npcKeys.name] = "Kha'damu",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{31.6, 48.8},{32.4, 49.6},{33.2, 15.4},{33.2, 57.6},{33.4, 56.2},{39.2, 39.6},{49, 18.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218706] = {
            [npcKeys.name] = "Test Visual",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218776] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218792] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218813] = {
            [npcKeys.name] = "Swamp of Sorrows Dart Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218869] = {
            [npcKeys.name] = "Death's Head Geomancer",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218870] = {
            [npcKeys.name] = "Death's Head Necromancer",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{54, 41},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218871] = {
            [npcKeys.name] = "Death's Head Cultist",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{55, 40.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218873] = {
            [npcKeys.name] = "Death's Head Warrior",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{53.6, 40.6},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218891] = {
            [npcKeys.name] = "Skeletal Servant",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 400,
            [npcKeys.spawns] = {
                [400] = {{54, 40.8},{55, 40.8},{55, 42},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218908] = {
            [npcKeys.name] = "Scarlet Crusade Assassin",
            [npcKeys.minLevel] = 40,
            [npcKeys.maxLevel] = 40,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218920] = {
            [npcKeys.name] = "Dalaran Agent",
            [npcKeys.minLevel] = 45,
            [npcKeys.maxLevel] = 45,
            [npcKeys.zoneID] = 41,
            [npcKeys.spawns] = {
                [41] = {{52, 34.6},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = {80098,80120,80147,80148,80149,80150,80151,80152},
            [npcKeys.questEnds] = {80098,80120,80147,80148,80149,80150,80151,80152},
        },
        [218930] = {
            [npcKeys.name] = "Deadwind Steed",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218931] = {
            [npcKeys.name] = "Dark Rider",
            [npcKeys.minLevel] = 41,
            [npcKeys.maxLevel] = 41,
            [npcKeys.zoneID] = 3,
            [npcKeys.spawns] = {
                [3] = {{52.4, 55.2},{52.6, 56},{54.2, 54.8},{54.4, 54},{54.8, 56.2},{55, 52.2},{55.2, 52.8},{55.2, 54},{56.2, 51},{56.2, 52.8},{56.2, 53.6},{56.4, 51.6},{56.4, 55.4},{56.4, 59.2},{56.6, 54},{57, 51.2},{57, 53.2},{57.2, 52.4},{57.4, 54.8},{57.4, 59.2},{57.6, 52.4},{57.8, 56.6},{58, 54.2},{58.2, 54.6},{58.2, 56.2},{58.4, 53.2},{58.6, 50.8},{58.6, 51.8},{58.6, 52.8},{58.6, 53.6},{58.6, 55.2},{58.6, 57},{58.8, 49.6},{58.8, 56},{59, 63.8},{60.6, 65.6},{61, 55.4},},
                [8] = {{59.6, 33.2},{61.2, 39.8},{62, 39.2},{62.8, 38.2},{65.6, 28.8},{66.2, 36.2},{66.6, 32.2},{66.8, 25.8},{66.8, 29.4},{67, 28.4},{67.2, 24.8},{67.2, 29.8},{67.4, 31},{67.6, 30},{68.2, 15.2},{68.2, 26.6},{68.2, 29.4},{68.4, 22.4},{68.4, 24.8},{68.4, 26.2},{68.4, 27.6},{68.6, 25.2},{68.6, 26},{68.8, 27},{68.8, 27.8},{69, 22.2},{69, 23.4},{69, 31},{69.4, 28.8},{69.4, 29.6},{69.4, 32.2},{69.6, 26.6},{69.6, 29},{69.6, 30},{69.8, 26.2},{70, 24.8},{70.2, 28.2},{70.8, 28.2},{70.8, 32.6},{71, 29.4},{71, 30},{71, 31.8},{71.4, 31.4},{71.8, 31.2},{71.8, 32},{72.4, 28.2},{72.6, 32.6},},
                [10] = {{17.6, 44.8},{18, 46.2},{18.4, 46.6},{18.6, 48.4},{18.8, 45},{19.2, 50},{19.2, 50.6},{19.8, 45},{20.6, 50.6},{21, 49.4},{21, 49.6},{21, 54},{21.4, 46.4},{21.8, 44.2},{21.8, 47.2},{22.2, 47.8},{22.4, 44.8},{22.4, 46.4},{22.4, 48.8},{22.4, 50.6},{22.6, 46.2},{22.6, 46.8},{22.6, 51.4},{22.8, 45},{22.8, 47.6},{23.2, 52.6},{23.4, 49.2},{23.8, 47},{23.8, 49.4},{24, 48.4},{24, 49.6},{24.2, 45},{24.6, 46},{24.6, 48.6},{25.2, 46.6},{25.8, 57.8},{26.2, 40.8},{26.6, 34.4},{40.8, 63.2},{42, 63.8},{43.8, 65.8},{53.4, 68.6},},
                [17] = {{51.4, 36.6},{52.2, 36.4},{52.2, 36.6},{52.6, 37.6},{53.2, 37.4},},
                [41] = {{34, 35.6},{38.8, 33.6},{41.8, 30.4},{42.4, 27.4},{42.4, 27.8},{42.6, 27.6},{42.6, 34.4},{42.8, 27.4},{42.8, 28.8},{43.2, 26.4},{43.6, 25.4},{43.8, 33.8},{44, 32},{44, 32.6},{44.4, 29},{44.4, 30},{44.4, 30.8},{44.6, 31},{44.8, 29.6},{45, 25.8},{45, 28.6},{45, 34.2},{45.4, 28},{45.6, 25},{45.8, 27},{46.2, 26.2},},
                [45] = {{50.4, 44.2},{50.4, 44.8},{52.2, 43},{57.4, 42.8},{58.8, 42.8},{59.4, 39.6},{60.2, 41},{60.2, 41.6},{60.4, 39.4},{60.4, 39.8},{60.6, 40.2},{60.6, 40.8},{60.8, 39.2},{60.8, 42},{61.2, 37.8},{61.4, 37},{61.4, 45},{61.6, 38},{61.6, 40.8},{62, 39.4},{62, 42},{62.2, 40.4},{62.2, 46.2},{62.6, 40.8},{63, 42.6},{64.6, 39.6},{64.8, 39},{65, 41},},
                [405] = {{61.8, 18.2},{63.8, 21.8},{63.8, 24.2},{64, 25.6},{64.2, 25.4},{64.4, 23},{64.6, 24.2},{65, 23.4},{65, 24.8},{65.2, 26.4},{65.6, 22.6},{65.6, 25.2},{66, 25.8},{66.2, 23.8},{67, 25},{67.4, 22.4},{67.4, 23.6},{68.8, 24.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218935] = {
            [npcKeys.name] = "Walking Bomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218969] = {
            [npcKeys.name] = "STX-96/FR",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218970] = {
            [npcKeys.name] = "STX-97/IC",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218971] = {
            [npcKeys.name] = "STX-97/IC",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218972] = {
            [npcKeys.name] = "STX-98/PO",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218973] = {
            [npcKeys.name] = "STX-98/PO",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218974] = {
            [npcKeys.name] = "STX-99/XD",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [218975] = {
            [npcKeys.name] = "STX-99/XD",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219110] = {
            [npcKeys.name] = "Incendiary Bomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219111] = {
            [npcKeys.name] = "Frost Bomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219112] = {
            [npcKeys.name] = "Radioactive Bomb",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219113] = {
            [npcKeys.name] = "Radiation Leak",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219177] = {
            [npcKeys.name] = "Omen",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219199] = {
            [npcKeys.name] = "Minion of Omen",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219659] = {
            [npcKeys.name] = "High Tinker Mekkatorque",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219663] = {
            [npcKeys.name] = "Nogg",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 1637,
            [npcKeys.spawns] = {
                [1637] = {{75.8, 25.2},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219704] = {
            [npcKeys.name] = "Gurgthock",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219791] = {
            [npcKeys.name] = "Tim's Stalker 01",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219822] = {
            [npcKeys.name] = "Chained Spirit",
            [npcKeys.minLevel] = 60,
            [npcKeys.maxLevel] = 60,
            [npcKeys.zoneID] = 33,
            [npcKeys.spawns] = {
                [33] = {{22.6, 15.4},{22.6, 15.6},{23.8, 52.6},{30.2, 73.2},{33.6, 62.4},{34.8, 13.4},{34.8, 13.6},{38.4, 8.8},{39, 45.8},{45.4, 13.4},{47.6, 34.2},},
            },
            [npcKeys.friendlyToFaction] = "AH",
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [219999] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [220007] = {
            [npcKeys.name] = "Viscous Fallout",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [220072] = {
            [npcKeys.name] = "Electrocutioner 6000",
            [npcKeys.minLevel] = 99,
            [npcKeys.maxLevel] = 99,
            [npcKeys.zoneID] = 721,
            [npcKeys.spawns] = {
                [721] = {{-1, -1},},
            },
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [220075] = {
            [npcKeys.name] = "[DNT] Teleporter Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [220077] = {
            [npcKeys.name] = "[DNT] Teleporter Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
        [220358] = {
            [npcKeys.name] = "Invisible Stalker",
            [npcKeys.minLevel] = 0,
            [npcKeys.maxLevel] = 0,
            [npcKeys.zoneID] = 0,
            [npcKeys.spawns] = nil,
            [npcKeys.friendlyToFaction] = nil,
            [npcKeys.questStarts] = nil,
            [npcKeys.questEnds] = nil,
        },
    }
end
